//
//  VirtualButtonLayer.swift
//
//  Copyright (C) 2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2016/04/10	H.Kanemoto
//	Modified:
//
import Foundation

public enum VRBUTTON_ALIGN : Int {case left, right}

open class VirtualButtonLayer : CALayer {
	fileprivate var _visible:Bool = true
	fileprivate var _location:CGPoint!
	fileprivate var _color:UIColor!
	//-------------------------------------------------------------------------
	//	プロパティ
	//-------------------------------------------------------------------------
	open var Size:CGFloat = 0
	open var Location:CGPoint {
		get {
			return self._location!
		}
		set(value) {
			self._location = value
			self.move()
		}
	}
	open var Visible:Bool {
		get {
			return self._visible
		}
		set(value) {
			if self._visible != value {
				self._visible = value
				self.move()
			}
		}
	}
	//-------------------------------------------------------------------------
	//	コンストラクタ
	//-------------------------------------------------------------------------
	public init(pt:CGPoint, size:CGFloat, color:UIColor) {
		super.init()
		self.backgroundColor = UIColor.clear.cgColor
		self.opacity = 0.3
		self.setNeedsDisplay()
		self._color = color
		self._location = pt
		self.Size = size
		self.Visible = false
	}
	
	public override init(layer: Any) {
		super.init(layer: layer)
	}
	
	required public init?(coder aDecoder: NSCoder) {
		fatalError("init(coder:) has not been implemented")
	}
	//-------------------------------------------------------------------------
	//	移動処理
	//-------------------------------------------------------------------------
	func move() {
		if self._visible {
			self.frame = self.calcFrame()
			self.setNeedsDisplay(self.bounds)
		} else {
			self.frame = CGRect(x: 0, y: 0, width: 0, height: 0)
		}
	}
	func calcFrame() -> CGRect {
		let x = self.Location.x - self.Size / 2
		let y = self.Location.y + self.Size / 2
		return CGRect(x: x, y: y, width: self.Size, height: self.Size)
	}
	//-------------------------------------------------------------------------
	//	描画処理
	//-------------------------------------------------------------------------
	open override func draw(in ctx: CGContext) {
		ctx.setFillColor(self._color.cgColor)
		ctx.setStrokeColor(self._color.cgColor)
		ctx.setLineWidth(2)
		ctx.addEllipse(in: self.bounds.insetBy(dx: 2, dy: 2))
		ctx.strokePath()
	}
}
