//
//  VirtualPadView.swift
//
//  Copyright (C) 2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2016/04/09	H.Kanemoto
//	Modified:
//
import Foundation
import GameController

open class VirtualPadView : UIView, iCadeEventDelegate {
	fileprivate var _Touches:CFMutableDictionary!
	fileprivate var _dispsync:CADisplayLink!
	fileprivate var _keyview:VirtualKBLayer!
	fileprivate var _stick:VirtualStickLayer? = nil
	fileprivate var _button1:VirtualButtonLayer!
	fileprivate var _button2:VirtualButtonLayer!
	fileprivate var _button3:VirtualButtonLayer!
	fileprivate var _indicator:KBIndicatorLayer! = nil
	fileprivate var _kbvisible:Bool = true
	//-------------------------------------------------------------------------
	//	プロパティ
	//-------------------------------------------------------------------------
	open var callbackButtonOn:((UInt8) -> (Void))? = nil
	open var callbackButtonOff:((UInt8) -> (Void))? = nil
	open var keyview:VirtualKBLayer {
		return self._keyview
	}
	//-------------------------------------------------------------------------
	//	コンストラクタ
	//-------------------------------------------------------------------------
	override init(frame: CGRect) {
		super.init(frame: frame)
		// Key:Touch Value:TouchRecordのDictionary配列を作成する
		var keyCallbacks = kCFTypeDictionaryKeyCallBacks
		var valueCallbacks = kCFTypeDictionaryValueCallBacks
		self._Touches = CFDictionaryCreateMutable(nil, 10, &keyCallbacks, &valueCallbacks)
		
		// Viewの初期化
		self.isUserInteractionEnabled = true
		self.isMultipleTouchEnabled = true
		self.layer.isOpaque = true
		self.backgroundColor = UIColor.clear
		
		// Virtual Key Indicator
		self._indicator = KBIndicatorLayer(rct: CGRect(x: 10, y: 10, width: 50, height: 30))
		self.layer.addSublayer(self._indicator)
		
		// Virtual Key Layer
		self._keyview = VirtualKBLayer(rct: UIScreen.main.bounds)
		self.layer.addSublayer(self._keyview)
		
		// Virtual Button Layers
		let appDelegate:AppDelegate = UIApplication.shared.delegate as! AppDelegate
		self._button1 = VirtualButtonLayer(pt: appDelegate.Button1Pt, size: appDelegate.Button1Size, color:UIColor.blue)
		self._button2 = VirtualButtonLayer(pt: appDelegate.Button2Pt, size: appDelegate.Button2Size, color:UIColor.red)
		self._button3 = VirtualButtonLayer(pt: appDelegate.Button3Pt, size: appDelegate.Button3Size, color:UIColor.green)
		self.layer.addSublayer(self._button1)
		self.layer.addSublayer(self._button2)
		self.layer.addSublayer(self._button3)
		
		// Virtual Stick Layer
		self._stick = VirtualStickLayer(size: 100, color: UIColor.white)
		self.layer.addSublayer(self._stick!)
		self._stick?.callbackStickOn = {(status:VPSTICK) -> Void in
			switch status {
			case VPSTICK.left:
				self.callbackButtonOn?(0x04)
			case VPSTICK.right:
				self.callbackButtonOn?(0x06)
			case VPSTICK.up:
				self.callbackButtonOn?(0x08)
			case VPSTICK.down:
				self.callbackButtonOn?(0x02)
			case VPSTICK.lUp:
				self.callbackButtonOn?(0x07)
			case VPSTICK.rUp:
				self.callbackButtonOn?(0x09)
			case VPSTICK.lDn:
				self.callbackButtonOn?(0x01)
			case VPSTICK.rDn:
				self.callbackButtonOn?(0x03)
			default:
				break
			}
		}
		self._stick?.callbackStickOff = {(status:VPSTICK) -> Void in
			switch status {
			case VPSTICK.left:
				self.callbackButtonOff?(0x04)
			case VPSTICK.right:
				self.callbackButtonOff?(0x06)
			case VPSTICK.up:
				self.callbackButtonOff?(0x08)
			case VPSTICK.down:
				self.callbackButtonOff?(0x02)
			case VPSTICK.lUp:
				self.callbackButtonOff?(0x07)
			case VPSTICK.rUp:
				self.callbackButtonOff?(0x09)
			case VPSTICK.lDn:
				self.callbackButtonOff?(0x01)
			case VPSTICK.rDn:
				self.callbackButtonOff?(0x03)
			default:
				break
			}
		}
	}
	
	required public init?(coder aDecoder: NSCoder) {
		fatalError("init(coder:) has not been implemented")
	}
	//-------------------------------------------------------------------------
	//	Button移動
	//-------------------------------------------------------------------------
	open func alignButtons() {
		let appDelegate:AppDelegate = UIApplication.shared.delegate as! AppDelegate
		self._button1.Location = appDelegate.Button1Pt
		self._button2.Location = appDelegate.Button2Pt
		self._button3.Location = appDelegate.Button3Pt
		self._button1.Size = appDelegate.Button1Size
		self._button2.Size = appDelegate.Button2Size
		self._button3.Size = appDelegate.Button3Size
	}
	//-------------------------------------------------------------------------
	//	touchイベント処理
	//-------------------------------------------------------------------------
	override open func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent?) {
		for touch in touches {
			let point = touch.location(in: self)
			let tr = self.hitTest(point)
			if tr.Key != nil {
				if tr.Key!.Code == 0x46 {
					N80Key.Shift = true
				}
				self.callbackButtonOn?(tr.Key!.Code)
			}
			CFDictionarySetValue(self._Touches, Unmanaged.passUnretained(touch).toOpaque(), Unmanaged.passUnretained(tr).toOpaque())
		}
	}
	
	override open func touchesMoved(_ touches: Set<UITouch>, with event: UIEvent?) {
		for touch in touches {
			let v = CFDictionaryGetValue(self._Touches, Unmanaged.passUnretained(touch).toOpaque())
			let tr:TouchRecord? = unsafeBitCast(v, to: TouchRecord.self)
			if tr != nil {
				tr!.CurrentPos = touch.location(in: self)
				if tr!.Stick != nil {
					tr!.Stick?.tracking(tr!.CurrentPos)
				}
			}
		}
	}
	
	override open func touchesEnded(_ touches: Set<UITouch>, with event: UIEvent?) {
		for touch in touches {
			let v = CFDictionaryGetValue(self._Touches, Unmanaged.passUnretained(touch).toOpaque())
			let tr:TouchRecord? = unsafeBitCast(v, to: TouchRecord.self)
			if tr != nil {
				if tr!.Key != nil {
					if tr!.Key!.Code == 0x46 {
						N80Key.Shift = false
					}
					tr!.Key!.Pushed = false
					self.callbackButtonOff?(tr!.Key!.Code)
					self._keyview.setNeedsDisplay(self._keyview.bounds)
				} else if tr!.Stick != nil {
					tr!.Stick?.Uncapture()
					tr!.Stick = nil
				}
				tr!.Key = nil
			}
			CFDictionaryRemoveValue(self._Touches, Unmanaged.passUnretained(touch).toOpaque())
		}
	}
	
	override open func touchesCancelled(_ touches: Set<UITouch>?, with event: UIEvent?) {
		let appDelegate:AppDelegate = UIApplication.shared.delegate as! AppDelegate
		appDelegate.clearKey()
		self._keyview.setNeedsDisplay(self._keyview.bounds)
		CFDictionaryRemoveAllValues(self._Touches)
	}
	//-------------------------------------------------------------------------
	//	HitTest処理
	//-------------------------------------------------------------------------
	func hitTest(_ pt:CGPoint) -> TouchRecord {
		let tr = TouchRecord(initPosition: pt)
		
		if self._indicator.frame.contains(pt) {
			self._kbvisible = self._keyview.toggleShow()
			self._button1.Visible = !self._kbvisible
			self._button2.Visible = !self._kbvisible
			self._button3.Visible = !self._kbvisible
			self.alignButtons()
			return tr
		}
		
		if self._button1.frame.contains(pt) {
			let appDelegate:AppDelegate = UIApplication.shared.delegate as! AppDelegate
			let k = appDelegate.findKeyByLable(appDelegate.Button1Label)
			if k != nil && !k!.Pushed {
				tr.Key = k!
				k!.Pushed = true
			}
			return tr
		}
		
		if self._button2.frame.contains(pt) {
			let appDelegate:AppDelegate = UIApplication.shared.delegate as! AppDelegate
			let k = appDelegate.findKeyByLable(appDelegate.Button2Label)
			if k != nil && !k!.Pushed {
				tr.Key = k!
				k!.Pushed = true
			}
			return tr
		}
		
		if self._button3.frame.contains(pt) {
			let appDelegate:AppDelegate = UIApplication.shared.delegate as! AppDelegate
			let k = appDelegate.findKeyByLable(appDelegate.Button3Label)
			if k != nil && !k!.Pushed {
				tr.Key = k!
				k!.Pushed = true
			}
			return tr
		}
		
		if self._keyview.frame.contains(pt) {
			let pt2 = CGPoint(x: pt.x - self._keyview.frame.origin.x, y: pt.y - self._keyview.frame.origin.y)
			let k = self._keyview.hitKeyTest(pt2)
			if k != nil {
				tr.Key = k
				k?.Pushed = true
			}
			return tr
		}
		// 仮想スティック
		if self._stick!.Capture(pt) {
			tr.Stick = self._stick
		}
		return tr
	}
	//-------------------------------------------------------------------------
	//	MFI Game Controller処理
	//-------------------------------------------------------------------------
	func hitMFIButton(_ ctr:GCController) {
		let appDelegate:AppDelegate = UIApplication.shared.delegate as! AppDelegate
		
		if ctr.extendedGamepad!.buttonA.isPressed != appDelegate.mfiButton1Pushed {
			let k = appDelegate.findKeyByLable(appDelegate.Button1Label)
			if k != nil {
				appDelegate.mfiButton1Pushed = ctr.extendedGamepad!.buttonA.isPressed
				k!.Pushed = appDelegate.mfiButton1Pushed
				if appDelegate.mfiButton1Pushed {
                    appDelegate.N80EM.n80kb.onKeyDown(k!.Code)
				} else {
                    appDelegate.N80EM.n80kb.onKeyUp(k!.Code)
				}
			}
		}
		if ctr.extendedGamepad!.buttonB.isPressed != appDelegate.mfiButton2Pushed {
			let k = appDelegate.findKeyByLable(appDelegate.Button2Label)
			if k != nil {
				appDelegate.mfiButton2Pushed = ctr.extendedGamepad!.buttonB.isPressed
				k!.Pushed = appDelegate.mfiButton2Pushed
				if appDelegate.mfiButton2Pushed {
                    appDelegate.N80EM.n80kb.onKeyDown(k!.Code)
				} else {
                    appDelegate.N80EM.n80kb.onKeyUp(k!.Code)
				}
			}
		}
		if ctr.extendedGamepad!.buttonX.isPressed != appDelegate.mfiButton3Pushed {
			let k = appDelegate.findKeyByLable(appDelegate.Button3Label)
			if k != nil {
				appDelegate.mfiButton3Pushed = ctr.extendedGamepad!.buttonX.isPressed
				k!.Pushed = appDelegate.mfiButton3Pushed
				if appDelegate.mfiButton3Pushed {
                    appDelegate.N80EM.n80kb.onKeyDown(k!.Code)
				} else {
                    appDelegate.N80EM.n80kb.onKeyUp(k!.Code)
				}
			}
		}
	}
	func hitMFITest(_ ctl:GCController) {
		let appDelegate:AppDelegate = UIApplication.shared.delegate as! AppDelegate
		
		if ctl.extendedGamepad!.dpad.left.isPressed && ctl.extendedGamepad!.dpad.left.value > 0.2 {
			if appDelegate.mfiDir != VPSTICK.left {
				self._stick?.callbackStickOff?(appDelegate.mfiDir)
				self._stick?.callbackStickOn?(VPSTICK.left)
				appDelegate.mfiDir = VPSTICK.left
			}
		} else if ctl.extendedGamepad!.dpad.right.isPressed && ctl.extendedGamepad!.dpad.right.value > 0.2 {
			if appDelegate.mfiDir != VPSTICK.right {
				self._stick?.callbackStickOff?(appDelegate.mfiDir)
				self._stick?.callbackStickOn?(VPSTICK.right)
				appDelegate.mfiDir = VPSTICK.right
			}
		} else if ctl.extendedGamepad!.dpad.up.isPressed && ctl.extendedGamepad!.dpad.up.value > 0.2 {
			if appDelegate.mfiDir != VPSTICK.up {
				self._stick?.callbackStickOff?(appDelegate.mfiDir)
				self._stick?.callbackStickOn?(VPSTICK.up)
				appDelegate.mfiDir = VPSTICK.up
			}
		} else if ctl.extendedGamepad!.dpad.down.isPressed && ctl.extendedGamepad!.dpad.down.value > 0.2 {
			if appDelegate.mfiDir != VPSTICK.down {
				self._stick?.callbackStickOff?(appDelegate.mfiDir)
				self._stick?.callbackStickOn?(VPSTICK.down)
				appDelegate.mfiDir = VPSTICK.down
			}
		} else {
			self._stick?.callbackStickOff?(appDelegate.mfiDir)
			appDelegate.mfiDir = VPSTICK.newtoral
		}
	}
	//-------------------------------------------------------------------------
	//	iCade Game Controller処理
	//-------------------------------------------------------------------------
	open func buttonDown(_ button:iCadeState) {
		let appDelegate:AppDelegate = UIApplication.shared.delegate as! AppDelegate
		
		switch button {
		case iCadeJoystickDown:
			self.callbackButtonOn?(0x02)
		case iCadeJoystickLeft:
			self.callbackButtonOn?(0x04)
		case iCadeJoystickRight:
			self.callbackButtonOn?(0x06)
		case iCadeJoystickUp:
			self.callbackButtonOn?(0x08)
		case iCadeJoystickUpLeft:
			self.callbackButtonOn?(0x07)
		case iCadeJoystickUpRight:
			self.callbackButtonOn?(0x09)
		case iCadeJoystickDownLeft:
			self.callbackButtonOn?(0x01)
		case iCadeJoystickDownRight:
			self.callbackButtonOn?(0x03)
		case iCadeButtonA:
			let k = appDelegate.findKeyByLable(appDelegate.Button1Label)
			if k != nil && !k!.Pushed {
				self.callbackButtonOn?((k?.Code)!)
			}
		case iCadeButtonB:
			let k = appDelegate.findKeyByLable(appDelegate.Button2Label)
			if k != nil && !k!.Pushed {
				self.callbackButtonOn?((k?.Code)!)
			}
		case iCadeButtonC:
			let k = appDelegate.findKeyByLable(appDelegate.Button3Label)
			if k != nil && !k!.Pushed {
				self.callbackButtonOn?((k?.Code)!)
			}
		default:
			break
		}
	}
	open func buttonUp(_ button:iCadeState) {
		let appDelegate:AppDelegate = UIApplication.shared.delegate as! AppDelegate
		
		switch button {
		case iCadeJoystickDown:
			self.callbackButtonOff?(0x02)
		case iCadeJoystickLeft:
			self.callbackButtonOff?(0x04)
		case iCadeJoystickRight:
			self.callbackButtonOff?(0x06)
		case iCadeJoystickUp:
			self.callbackButtonOff?(0x08)
		case iCadeJoystickUpLeft:
			self.callbackButtonOff?(0x07)
		case iCadeJoystickUpRight:
			self.callbackButtonOff?(0x09)
		case iCadeJoystickDownLeft:
			self.callbackButtonOff?(0x01)
		case iCadeJoystickDownRight:
			self.callbackButtonOff?(0x03)
		case iCadeButtonA:
			let k = appDelegate.findKeyByLable(appDelegate.Button1Label)
			if k != nil && !k!.Pushed {
				self.callbackButtonOff?((k?.Code)!)
			}
		case iCadeButtonB:
			let k = appDelegate.findKeyByLable(appDelegate.Button2Label)
			if k != nil && !k!.Pushed {
				self.callbackButtonOff?((k?.Code)!)
			}
		case iCadeButtonC:
			let k = appDelegate.findKeyByLable(appDelegate.Button3Label)
			if k != nil && !k!.Pushed {
				self.callbackButtonOff?((k?.Code)!)
			}
		default:
			break
		}
	}
}
