//
//  VirtualStickLayer.swift
//
//  Copyright (C) 2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2016/04/07	H.Kanemoto
//	Modified:
//
import Foundation

public enum VPSTICK_STICKTYPE : Int {case fixed = 0, adjust_CENTER, adjust_NEAR}
public enum VPSTICK_STICKDIR : Int {case dir_2, dir_4, dir_8}
public enum VPSTICK : Int {case newtoral = -1, left = 0, right, up, down, lUp, rUp, lDn, rDn}

open class VirtualStickLayer : CALayer {
	fileprivate var sticktype = VPSTICK_STICKTYPE.fixed
	fileprivate var stickdir = VPSTICK_STICKDIR.dir_8
	fileprivate var _vector:Vector? = nil
	fileprivate var _color:UIColor!
	fileprivate var _center_x:CGFloat = 0	//中心点のframe座標
	fileprivate var _center_y:CGFloat = 0
	fileprivate var _size:CGFloat = 0       //ボタン直径
	fileprivate var maxScalar:Double = 15
	fileprivate var playOfStick:Double = 5
	//-------------------------------------------------------------------------
	//	プロパティ
	//-------------------------------------------------------------------------
	open var Captured:Bool = false
	open var Status:VPSTICK = VPSTICK.newtoral
	open var callbackStickOn:((VPSTICK) -> Void)? = nil
	open var callbackStickOff:((VPSTICK) -> Void)? = nil
	
	open var center:Position {
		return Position(x: Double(self.frame.origin.x + self.bounds.width / 2), y: Double(self.frame.origin.y + self.bounds.height / 2))
	}
	open var centerR:Double {
		return Double(self.bounds.width / 4)
	}
	open var centerL:Double {
		return Double(self.bounds.width / 2)
	}
	open var Color:UIColor {
		get {
			return self._color
		}
		set(value) {
			self._color = value
		}
	}
	//-------------------------------------------------------------------------
	//	コンストラクタ
	//-------------------------------------------------------------------------
	public init(size:CGFloat, color:UIColor) {
		super.init()
		self._center_x = 0
		self._center_y = 0
		self._size = size
		self.backgroundColor = UIColor.clear.cgColor
		self.opacity = 0.3
		self.setNeedsDisplay()
		self.setFrameSilent(CGRect(x: 0, y: 0, width: 0, height: 0))
		self.Color = color
	}
	
	required public init?(coder aDecoder: NSCoder) {
		fatalError("init(coder:) has not been implemented")
	}
	//-------------------------------------------------------------------------
	//	デストラクタ
	//-------------------------------------------------------------------------
	deinit {
		self._vector = nil
		self.callbackStickOn = nil
		self.callbackStickOff = nil
	}
	//-------------------------------------------------------------------------
	//	中心点座標から矩形を求める
	//-------------------------------------------------------------------------
	open func calcFrameSize() -> CGRect {
		let x = self._center_x - (self._size / 2)
		let y = self._center_y - (self._size / 2)
		let rct = CGRect(x: x, y: y, width: self._size, height: self._size)
		return rct
	}
	//-------------------------------------------------------------------------
	//	Stick使用開始
	//-------------------------------------------------------------------------
	open func Capture(_ pt:CGPoint) -> Bool {
		if !self.Captured {
			self.Captured = true
			self.Status = VPSTICK.newtoral
			self.moveCenterTo(pt)
			self.setNeedsDisplay(self.bounds)
			return true
		}
		return false
	}
	//-------------------------------------------------------------------------
	//	Stick使用終了
	//-------------------------------------------------------------------------
	open func Uncapture() {
		if self.Captured {
			self.callbackStickOff?(self.Status)
			self.Captured = false
			self.Status = VPSTICK.newtoral
			self.setFrameSilent(CGRect(x: 0, y: 0, width: 0, height: 0))
		}
	}
	//-------------------------------------------------------------------------
	//	描画処理
	//-------------------------------------------------------------------------
	open override func draw(in ctx: CGContext) {
		if self.Captured {
			ctx.setFillColor(self.Color.cgColor)
			ctx.setStrokeColor(UIColor.white.cgColor)
		} else {
			ctx.setFillColor(UIColor.clear.cgColor)
			//CGContextSetStrokeColorWithColor(ctx,self.Color.CGColor)
			return
		}
		ctx.setLineWidth(6)
		let cnt = CGRect(x: self.bounds.width / 4, y: self.bounds.height / 4, width: self.bounds.width / 2, height: self.bounds.height / 2)
		ctx.addEllipse(in: cnt)
		ctx.strokePath()
		
		if self.Status != VPSTICK.newtoral && self._vector != nil {
			let vec = Vector(scalar: self.centerR, degree: self._vector!.Degree)
			let c = Position(x: Double(self.bounds.width / 2), y: Double(self.bounds.height / 2))
			c.move(vec.X, dy: vec.Y * -1)
			let r = self.centerR * 0.8
			let l = r * 2
			let rec = CGRect(x: CGFloat(c.X - r), y: CGFloat(c.Y - r), width: CGFloat(l), height: CGFloat(l))
			ctx.setFillColor(self.Color.cgColor)
			ctx.fillEllipse(in: rec)
			//                CGContextStrokePath(ctx)
		}
	}
	//-------------------------------------------------------------------------
	//	仮想Stickの中心座標を原点として、指定の座標のベクトルを求める
	//	- Vecotr.degree は時計周りの角度(0時=0度、3時=90度、6時=180度・・・)
	//	- Vector.scalar は原点からの距離(絶対値)
	//-------------------------------------------------------------------------
	open func getVector(_ pt:CGPoint) -> Vector {
		let pos = Position(x: Double(pt.x), y: Double(pt.y))
		let offset = self.center.offset(pos)	//Stick原点(中心点)からの相対座標を求める
		offset.Y *= -1  // Vector演算は数学座標で行うのでY座標の符号を反転する
		return Vector(position: offset)
	}
	//-------------------------------------------------------------------------
	//	touch moveイベント処理
	//	現在、仮想スティックの操作は、指の移動先の原点からの角度と距離で
	//	８方向を決定している。
	//	補正処理として、移動先にスティックの中心を移動させる。
	//	スティックを固定にした場合、ユーザがスティック位置をロストしてし
	//	まうと操作出来ないか、意図しない操作となる為
	//-------------------------------------------------------------------------
	open func tracking(_ pt:CGPoint) {
		let appDelegate:AppDelegate = UIApplication.shared.delegate as! AppDelegate
		self.sticktype = appDelegate.StickType
		self.stickdir = appDelegate.StickDir
		self.playOfStick = appDelegate.playOfStick
		self.maxScalar = appDelegate.maxScalar
		
		var redraw = false
		let vector = self.getVector(pt)
		if vector.Scalar < self.playOfStick {
			//スティック遊び分
			if self.sticktype != VPSTICK_STICKTYPE.adjust_CENTER {
				self._vector = nil
				if self.Status != VPSTICK.newtoral {
					self.callbackStickOff?(self.Status)
					redraw = true
				}
				self.Status = VPSTICK.newtoral
			}
		} else {
			self._vector = vector
			/*
			if vector.AbsX >= vector.AbsY {
				if vector.X > 0 {
					if self.Status != VPSTICK.RIGHT {
						self.callbackStickOff?(self.Status)
						self.callbackStickOn?(VPSTICK.RIGHT)
						redraw = true
					}
					self.Status = VPSTICK.RIGHT
				} else {
					if self.Status != VPSTICK.LEFT {
						self.callbackStickOff?(self.Status)
						self.callbackStickOn?(VPSTICK.LEFT)
						redraw = true
					}
					self.Status = VPSTICK.LEFT
				}
			} else {
				if vector.Y > 0 {
					if self.Status != VPSTICK.UP {
						self.callbackStickOff?(self.Status)
						self.callbackStickOn?(VPSTICK.UP)
						redraw = true
					}
					self.Status = VPSTICK.UP
				} else {
					if self.Status != VPSTICK.DOWN {
						self.callbackStickOff?(self.Status)
						self.callbackStickOn?(VPSTICK.DOWN)
						redraw = true
					}
					self.Status = VPSTICK.DOWN
				}
			}*/
			if self.stickdir == VPSTICK_STICKDIR.dir_2 {
				//２方向(左右)
				if vector.Degree > 45 && vector.Degree <= 135 {
					if self.Status != VPSTICK.right {
						self.callbackStickOff?(self.Status)
						self.callbackStickOn?(VPSTICK.right)
						redraw = true
					}
					self.Status = VPSTICK.right
					self.slideStick(pt, vector:vector, status: self.Status)
					
				} else if vector.Degree > 245 && vector.Degree <= 315 {
					if self.Status != VPSTICK.left {
						self.callbackStickOff?(self.Status)
						self.callbackStickOn?(VPSTICK.left)
						redraw = true
					}
					self.Status = VPSTICK.left
					self.slideStick(pt, vector:vector, status: self.Status)
				}
			} else if self.stickdir == VPSTICK_STICKDIR.dir_4 {
				//４方向
				if vector.Degree > 45 && vector.Degree <= 135 {
					if self.Status != VPSTICK.right {
						self.callbackStickOff?(self.Status)
						self.callbackStickOn?(VPSTICK.right)
						redraw = true
					}
					self.Status = VPSTICK.right
					self.slideStick(pt, vector:vector, status: self.Status)
					
				} else if vector.Degree > 135 && vector.Degree <= 245 {
					if self.Status != VPSTICK.down {
						self.callbackStickOff?(self.Status)
						self.callbackStickOn?(VPSTICK.down)
						redraw = true
					}
					self.Status = VPSTICK.down
					self.slideStick(pt, vector:vector, status: self.Status)
					
				} else if vector.Degree > 245 && vector.Degree <= 315 {
					if self.Status != VPSTICK.left {
						self.callbackStickOff?(self.Status)
						self.callbackStickOn?(VPSTICK.left)
						redraw = true
					}
					self.Status = VPSTICK.left
					self.slideStick(pt, vector:vector, status: self.Status)
					
				} else {
					if self.Status != VPSTICK.up {
						self.callbackStickOff?(self.Status)
						self.callbackStickOn?(VPSTICK.up)
						redraw = true
					}
					self.Status = VPSTICK.up
					self.slideStick(pt, vector:vector, status: self.Status)
				}
			} else {
				//８方向
				if vector.Degree > 30 && vector.Degree <= 60 {
					if self.Status != VPSTICK.rUp {
						self.callbackStickOff?(self.Status)
						self.callbackStickOn?(VPSTICK.rUp)
						redraw = true
					}
					self.Status = VPSTICK.rUp
					self.slideStick(pt, vector:vector, status: self.Status)
					
				} else if vector.Degree > 60 && vector.Degree <= 120 {
					if self.Status != VPSTICK.right {
						self.callbackStickOff?(self.Status)
						self.callbackStickOn?(VPSTICK.right)
						redraw = true
					}
					self.Status = VPSTICK.right
					self.slideStick(pt, vector:vector, status: self.Status)
					
				} else if vector.Degree > 120 && vector.Degree <= 150 {
					if self.Status != VPSTICK.rDn {
						self.callbackStickOff?(self.Status)
						self.callbackStickOn?(VPSTICK.rDn)
						redraw = true
					}
					self.Status = VPSTICK.rDn
					self.slideStick(pt, vector:vector, status: self.Status)
					
				} else if vector.Degree > 150 && vector.Degree <= 210 {
					if self.Status != VPSTICK.down {
						self.callbackStickOff?(self.Status)
						self.callbackStickOn?(VPSTICK.down)
						redraw = true
					}
					self.Status = VPSTICK.down
					self.slideStick(pt, vector:vector, status: self.Status)
					
				} else if vector.Degree > 210 && vector.Degree <= 240 {
					if self.Status != VPSTICK.lDn {
						self.callbackStickOff?(self.Status)
						self.callbackStickOn?(VPSTICK.lDn)
						redraw = true
					}
					self.Status = VPSTICK.lDn
					self.slideStick(pt, vector:vector, status: self.Status)
					
				} else if vector.Degree > 240 && vector.Degree <= 300 {
					if self.Status != VPSTICK.left {
						self.callbackStickOff?(self.Status)
						self.callbackStickOn?(VPSTICK.left)
						redraw = true
					}
					self.Status = VPSTICK.left
					self.slideStick(pt, vector:vector, status: self.Status)
					
				} else if vector.Degree > 300 && vector.Degree <= 330 {
					if self.Status != VPSTICK.lUp {
						self.callbackStickOff?(self.Status)
						self.callbackStickOn?(VPSTICK.lUp)
						redraw = true
					}
					self.Status = VPSTICK.lUp
					self.slideStick(pt, vector:vector, status: self.Status)
					
				} else {
					if self.Status != VPSTICK.up {
						self.callbackStickOff?(self.Status)
						self.callbackStickOn?(VPSTICK.up)
						redraw = true
					}
					self.Status = VPSTICK.up
					self.slideStick(pt, vector:vector, status: self.Status)
				}
			}
		}
		self.setNeedsDisplay(self.bounds)
		if redraw {
			//self.setNeedsDisplayInRect(self.bounds)
			//print("vector(degree="+round(vector.Degree).description+" scalar="+round(vector.Scalar).description+")")
		}
	}
	//-------------------------------------------------------------------------
	//	stick追従処理
	//	- 原点追従
	//	- 近接追従
	//-------------------------------------------------------------------------
	func slideStick(_ pt:CGPoint, vector:Vector, status:VPSTICK) {
		if self.sticktype == VPSTICK_STICKTYPE.adjust_CENTER {
			self.moveCenterTo(pt)
			
		} else if self.sticktype == VPSTICK_STICKTYPE.adjust_NEAR {
			if vector.Scalar > self.maxScalar {
				switch status {
				case VPSTICK.right:
					let dx:CGFloat = CGFloat(vector.Scalar - self.maxScalar)
					let dy:CGFloat = CGFloat(vector.Y * -1)
					self.moveCenter(dx, dy: dy)
				case VPSTICK.left:
					let dx:CGFloat = CGFloat(vector.Scalar - self.maxScalar) * -1
					let dy:CGFloat = CGFloat(vector.Y * -1)
					self.moveCenter(dx, dy: dy)
				case VPSTICK.up:
					let dy:CGFloat = CGFloat(vector.Scalar - self.maxScalar) * -1
					let dx:CGFloat = CGFloat(vector.X)
					self.moveCenter(dx, dy: dy)
				case VPSTICK.down:
					let dx:CGFloat = CGFloat(vector.Scalar - self.maxScalar)
					let dy:CGFloat = CGFloat(vector.Y * -1)
					self.moveCenter(dx, dy: dy)
				case VPSTICK.rUp:
					let r:CGFloat = CGFloat(self.playOfStick * 1.41421356237)
					let x:CGFloat = pt.x - r
					let y:CGFloat = pt.y + r
					self.moveCenterTo(CGPoint(x: x, y: y))
				case VPSTICK.lUp:
					let r:CGFloat = CGFloat(self.playOfStick * 1.41421356237)
					let x:CGFloat = pt.x + r
					let y:CGFloat = pt.y + r
					self.moveCenterTo(CGPoint(x: x, y: y))
				case VPSTICK.rDn:
					let r:CGFloat = CGFloat(self.playOfStick * 1.41421356237)
					let x:CGFloat = pt.x - r
					let y:CGFloat = pt.y - r
					self.moveCenterTo(CGPoint(x: x, y: y))
				case VPSTICK.lDn:
					let r:CGFloat = CGFloat(self.playOfStick * 1.41421356237)
					let x:CGFloat = pt.x + r
					let y:CGFloat = pt.y - r
					self.moveCenterTo(CGPoint(x: x, y: y))
				default:
					break
				}
			}
		}
	}
	//-------------------------------------------------------------------------
	//	Layer移動処理
	//-------------------------------------------------------------------------
	open func moveCenter(_ dx:CGFloat, dy:CGFloat) {
		let pt = self.center
		self._center_x = CGFloat(pt.X) + dx
		self._center_y = CGFloat(pt.Y) + dy
		self.moveCenterTo(CGPoint(x: self._center_x, y: self._center_y))
	}
	
	open func moveCenterTo(_ pt:CGPoint) {
		self._center_x = pt.x
		self._center_y = pt.y
		self.setFrameSilent(self.calcFrameSize())
	}
	// set frame without animation
	open func setFrameSilent(_ rct:CGRect) {
		CATransaction.begin()
		CATransaction.setValue(kCFBooleanTrue, forKey: kCATransactionDisableActions)
		self.frame = rct
		CATransaction.commit()
	}
}
