//
//  ClockDegree.swift
//
//  Copyright (C) 2015-2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2015/06/30	H.Kanemoto
//	Modified:
//
import Foundation

/*    時計回り角度       ラジアン角度
*          0                90
*          |                |
*    270  -+-   90    180  -+-   0
*          |                |
*         180              -90
*/
open class ClockDegree : NSObject {
    fileprivate var _degree:Double = 0.0
    
    //プロパティ
    var degree:Double {
        get {
            return self._degree
        }
        set(value) {
            self._degree = ClockDegree.offsetDegree(value)
        }
    }
    var radian:Double {
        get {
            return ClockDegree.radian2degree(self.degree)
        }
        set(value) {
            let deg:Double = ClockDegree.radian2degree(value)
            self.degree = deg
        }
    }
    var oppositeDegree:Double {
        get {
            let r:Double = self.degree + 180
            return ClockDegree.offsetDegree(r)
        }
    }
    
    //コンストラクタ
    override init() {
        super.init()
        self.degree = 0.0
    }
    init(degree:Double) {
        super.init()
        self.degree = degree
    }
    func rotate(_ value:Double) {
        let r:Double = self.degree + value
        self.degree = r
    }
    
    //クラスメソッド
    class func degree2radian(_ degree:Double) -> Double {
        let deg:Double = clock2radDeg(degree)
        return geometry.toRadian(deg)
    }
    class func radian2degree(_ radian:Double) -> Double {
        let deg:Double = geometry.toDegree(radian)
        return radDeg2Clock(deg)
    }
    fileprivate class func clock2radDeg(_ value:Double) -> Double {
        var deg:Double = value
        if deg <= 90 {          // 0 - 90
            deg = 90 - deg
        } else if deg >= 270 {  // 270 - 360
            deg = 360 - deg + 90
        } else {                // 90 - 270
            deg -= 90
            deg *= -1
        }
        return deg
    }
    fileprivate class func radDeg2Clock(_ value:Double) -> Double {
        var deg:Double = value
        if value < 0 {
            deg *= -1
            deg += 90
        } else if value <= 90 {
            deg = 90 - deg
        } else {
            deg = 180 - deg + 270
        }
        return deg
    }
    class func opposite(_ degree:Double) ->Double {
        if degree < 180 {
            return degree + 180
        } else {
            return degree - 180
        }
    }
    class func reflectH(_ degree:Double) ->Double {
        let incident:Double = ClockDegree.offsetDegree(degree)
        if incident > 0 && incident < 90 {
            return 90 + (90 - incident)
        } else if incident > 90 && incident <= 180 {
            return 90 - (incident - 90)
        } else if incident > 180 && incident < 270 {
            return 270 + (270 - incident)
        } else if incident > 270 && incident < 360 {
            return 270 - (incident - 270)
        }
        return incident
    }
    class func offsetDegree(_ value:Double) -> Double {
        var result:Double = value
        if value < 0 {
            let rev = value * -1
            let d:Int32 = Int32(floor(rev))
            let r:Int32 = d / 360
            result = 360 - (rev - Double(r * 360))
        } else if value >= 360 {
            let d:Int32 = Int32(floor(value))
            let r:Int32 = d / 360
            result = value - Double(r * 360)
        }
        return result
    }
    class func clockRange(_ s:Double, e:Double, target:Double) ->Bool {
        // 開始角から終了角までの範囲内ﾁｪｯｸを時計回りに行う
        var t:Double = target
        var e2:Double = e
        if s > e {
            // 開始角より終了角が狭い場合は360度を加算して判断
            if target <= e {
                t += 360
            }
            e2 += 360
        }
        return (t >= s && t <= e2)
    }
}
