//
//  Position.swift
//
//  Copyright (C) 2015-2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2015/06/30	H.Kanemoto
//	Modified:
//
import Foundation

open class Position : NSObject {
    fileprivate var _X:Double!
    fileprivate var _Y:Double!
    
    //プロパティ
    var X:Double {
        get {
            return self._X
        }
        set(value) {
            self._X = value
        }
    }
    var Y:Double {
        get {
            return self._Y
        }
        set(value) {
            self._Y = value
        }
    }
    var IntX:Int32 {
        get {
            return Int32(round(self.X))
        }
    }
    var IntY:Int32 {
        get {
            return Int32(round(self.Y))
        }
    }
    
    //コンストラクタ
    override init() {
        super.init()
        self.X = 0
        self.Y = 0
    }
    public init(x:Double, y:Double) {
        super.init()
        self.X = x
        self.Y = y
    }
    public init(p:Position) {
        super.init()
        self.setPosition(p)
    }

    //メソッド
    open func setPosition(_ p:Position) {
        self.X = p.X
        self.Y = p.Y
    }
    open func clear() {
        self.X = 0
        self.Y = 0
    }
    open func move(_ offset:Position) {
        self.move(offset.X, dy: offset.Y)
    }
    open func move(_ dx:Double, dy:Double) {
        self.X += dx
        self.Y += dy
    }
    open func rotate(_ degree:Double) {
        let p = geometry.rotate(self, degree: degree)
        self.setPosition(p)
    }
    open func offset(_ target:Position) -> Position {
        // このインスタンスの座標を原点（0, 0)とした場合の引数(target)の相対座標を返す
        return Position(x:(target.X - self.X), y:(target.Y - self.Y))
    }
    open func scale(_ factor:Double) {
        self.X *= factor
        self.Y *= factor
    }
}
