//
//  Vector.swift
//
//  Copyright (C) 2015-2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2015/07/04	H.Kanemoto
//	Modified:
//
import Foundation

open class Vector : NSObject {
    fileprivate var _dirty:Bool = false
    fileprivate var _vectX:Double!
    fileprivate var _vectY:Double!
    fileprivate var _degree:Double!
    fileprivate var _scalar:Double!
    
    //プロパティ
    var Dirty:Bool {
        get {
            return self._dirty
        }
        set(value) {
            self._dirty = value
        }
    }
    var X:Double {
        get {
            return self._vectX
        }
        set(value) {
            self._vectX = value
            self._dirty = true
        }
    }
    var Y:Double {
        get {
            return self._vectY
        }
        set(value) {
            self._vectY = value
            self._dirty = true
        }
    }
    var AbsX:Double {
        get {
            return abs(self.X)
        }
    }
    var AbsY:Double {
        get {
            return abs(self.Y)
        }
    }
    var Scalar:Double {
        get {
            if self.Dirty {
                self.reflesh()
            }
            return self._scalar
        }
        set(value) {
            self.setVector(value, degree: self.Degree)
        }
    }
    var Degree:Double {
        get {
            if self.Dirty {
                self.reflesh()
            }
            return self._degree
        }
        set(value) {
            self.setVector(self.Scalar, degree: value)
        }
    }
    
    //コンストラクタ
    override init() {
        super.init()
        self.clear()
    }
    public init(scalar:Double, degree:Double) {
        super.init()
        self.setVector(scalar, degree: degree)
    }
    public init(position:Position) {
        super.init()
        self.X = position.X
        self.Y = position.Y
    }
    public init(posStart:Position, posEnd:Position) {
        super.init()
        let offset = posStart.offset(posEnd)
        self.X = offset.X
        self.Y = offset.Y
    }
    
    //メソッド
    open func setVector(_ scalar:Double, degree:Double) {
        // 角度、スカラー値でベクトル設定
        let c = Vector.components(scalar, degree: degree)
        self._degree = degree
        self._scalar = scalar
        self._vectX = c.X
        self._vectY = c.Y
        self._dirty = false
    }
    
    open func clear() {
        self.X = 0
        self.Y = 0
        self._dirty = true
    }
    
    open func reflesh() {
        self._scalar = geometry.distance(self.X, y: self.Y)
        self._degree = Vector.originDegree(self.X, vy: self.Y)
        self._dirty = false
    }
    
    open func add(_ v:Vector) {
        self.X += v.X
        self.Y += v.Y
    }
    
    open func add(_ scalar:Double, degree:Double) {
        let c = Vector.components(scalar, degree: degree)
        self.X += c.X
        self.Y += c.Y
    }
    
    //クラスメソッド
    class func originDegree(_ p:Position) -> Double {
        return originDegree(p.X, vy: p.Y)
    }
    
    class func originDegree(_ vx:Double, vy:Double) -> Double {
        if vx == 0.0 {
            if vy == 0.0 {
                return 0
            } else if vy > 0.0 {
                return 0
            } else {
                return 180
            }
        } else {
            let rad = atan2(vy, vx)
            //print("atan("+vy.description+","+vx.description+")="+rad.description)
            let deg = ClockDegree.radian2degree(rad)
            return deg
        }
    }
    
    class func components(_ scalar:Double, degree:Double) -> Position {
        // ベクトルのX,Y方向の成分を返す
        let rad = ClockDegree.degree2radian(degree)
        return Position(x: cos(rad) * scalar, y: sin(rad) * scalar)
    }
}
