//
//  N80FileViewController.swift
//
//  Copyright (C) 2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2016/03/11	H.Kanemoto
//	Modified:
//
import Foundation

open class N80FileViewController : UITableViewController {
	var TopTreeIndex:Int = -1
	var Session:SMBSession? = nil
	var sub_queue:DispatchQueue!
	var file_info_list:NSMutableArray? = nil
	var folder_tree:[String] = [String]()
	var file_name_list:[String] = []
	var delegate:ViewController? = nil
	var proxy:subclass!
	
	@IBOutlet weak var txtIP: UITextField!
	@IBOutlet weak var txtUser: UITextField!
	@IBOutlet weak var txtPassword: UITextField!
	@IBOutlet weak var tblFiles: UITableView!
	@IBOutlet weak var btnConnect: UIButton!
    @IBOutlet weak var swSign: UISwitch!
    
	override open func viewDidLoad() {
		super.viewDidLoad()
		// Do any additional setup after loading the view, typically from a nib.
		self.proxy = subclass(parent: self)
		tblFiles.delegate = self.proxy
		tblFiles.dataSource = self.proxy
		self.sub_queue = DispatchQueue(label: "smb2test", attributes: [])
		let appDelegate:AppDelegate = UIApplication.shared.delegate as! AppDelegate
		txtIP.text = appDelegate.SMBIP
		txtUser.text = appDelegate.SMBUser
		txtPassword.text = appDelegate.SMBPassword
        swSign.setOn(appDelegate.SMBSign, animated: false)
	}
	
	override open func viewWillDisappear(_ animated: Bool) {
//		NSLog("unload $$$$$$$$$$$$$$$$$")
//		self.delegate?.icadeview.active = true
//		self.releaseResources()
	}
    override open func viewDidDisappear(_ animated: Bool) {
        NSLog("unload $$$$$$$$$$$$$$$$$")
		self.delegate?.icadeview?.active = true
        self.releaseResources()
    }
	
	open class subclass : NSObject, UITableViewDataSource, UITableViewDelegate {
		fileprivate var delegate:N80FileViewController!
		
		public init(parent:N80FileViewController) {
			self.delegate = parent
		}
		open func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
			return self.delegate.proxy_tableView(tableView, numberOfRowsInSection: section)
		}
		open func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
			return self.delegate.proxy_tableView(tableView, cellForRowAtIndexPath: indexPath)
		}
		
		open func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
			self.delegate.proxy_tableView(tableView, didSelectRowAtIndexPath: indexPath)
		}
	}
	
	open func proxy_tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
		return file_name_list.count
	}
 
	open func proxy_tableView(_ tableView: UITableView, cellForRowAtIndexPath indexPath: IndexPath) -> UITableViewCell {
		let cell: UITableViewCell = UITableViewCell(style: UITableViewCell.CellStyle.subtitle, reuseIdentifier: "Cell")
        let row = indexPath.row;
        cell.textLabel?.text = row < file_name_list.count ? file_name_list[row] : nil
		return cell
	}
	
	open func proxy_tableView(_ tableView: UITableView, didSelectRowAtIndexPath indexPath: IndexPath) {
        if self.Session == nil || !self.Session!.probeServer(true) {
			return
		}
		if self.TopTreeIndex == -1 {	//tree is selected from shares
			self.folder_tree.removeAll()
			let tree:SMBTreeConnect = self.Session!.shares[indexPath.row] as! SMBTreeConnect
			let list = self.build_file_list(tree, dir: "")
			if list != nil {
				self.TopTreeIndex = indexPath.row
				self.file_info_list = list
			}
		} else {	//file or folder is selected from tree
			let tree:SMBTreeConnect = self.Session!.shares[self.TopTreeIndex] as! SMBTreeConnect
			let info:FileIdBothDirInformation = self.file_info_list![indexPath.row] as! FileIdBothDirInformation
          if !info.isDir {
				self.file_read_operation(tree, info: info)
			} else {
				self.folder_operation(tree, info: info)
			}
		}
	}
	
	func show_share_list() -> Void {
		file_name_list.removeAll()
		for i:Int in 0 ..< self.Session!.shares.count {
			let item:SMBTreeConnect = (self.Session!.shares.object(at: i) as? SMBTreeConnect)!
			file_name_list.append(item.shareInfo.fileName!)
		}
		tblFiles.reloadData()
	}
	
	func folder_operation(_ tree:SMBTreeConnect, info:FileIdBothDirInformation) -> Void {
		if info.fileName == "." {
			return
		}
		if info.fileName == ".." {
			if self.folder_tree.count == 0 {
				self.TopTreeIndex = -1
				self.show_share_list()	//return to share list
				return
			}
			let last = self.folder_tree[self.folder_tree.count - 1]
			_ = self.folder_tree_up()
			self.sub_queue.async(execute: {() in
				let list = self.build_file_list(tree, dir: self.folder_path_name())
				if list == nil {
					_ = self.folder_tree_down(last)
					return
				}
				self.file_info_list = list
			})
			
		} else {
			self.sub_queue.async(execute: {() in
				_ = self.folder_tree_down(info.fileName)
				let list = self.build_file_list(tree, dir: self.folder_path_name())
				if list == nil {
					_ = self.folder_tree_up()
					return
				}
				self.file_info_list = list
			})
		}
	}
	
	func file_read_operation(_ tree:SMBTreeConnect, info:FileIdBothDirInformation) -> Void {
		let fullpath = self.file_full_path(info.fileName)
		let of:SMBOpen? = tree.openFile(fullpath, autoDisconnect: true, callback: nil)
        if of != nil {
            let data:bytecoder? = of!.read_once()
            if data != nil {
                let upstr = info.fileName.uppercased()
                if (upstr.range(of: ".CMT") != nil) {
                    self.delegate?.recvCMTData(data!)
                } else {
                    self.delegate?.recvN80Data(data!)
                }
                self.view.endEditing(true)
                self.dismiss(animated: true, completion: nil)
            }
        }
	}
	
	func build_file_list(_ tree:SMBTreeConnect, dir:String) -> NSMutableArray? {
		let dir = self.folder_path_name()
		let list = tree.enumDir(dir, callback:{(info:FileIdBothDirInformation?) -> Bool in
          if !info!.isDir {
				let upstr = info!.fileName.uppercased()
				if (upstr.range(of: ".CMT") != nil) || (upstr.range(of: ".N80") != nil) {
					return true
				} else {
					return false
				}
			}
			return true
		})
		if list != nil {
			file_name_list.removeAll()
			for i:Int in 0 ..< list!.count {
				let info:FileIdBothDirInformation = list![i] as! FileIdBothDirInformation;
				file_name_list.append(info.fileName)
				NSLog("enum file name = %@", info.fileName)
			}
			DispatchQueue.main.async(execute: {() in
				self.tblFiles.reloadData()
			})
		}
		return list
	}
	
	func folder_tree_up() -> String {
		if self.folder_tree.count > 0 {
			self.folder_tree.removeLast()
		}
		return self.folder_path_name()
	}
	func folder_tree_down(_ name:String) -> String {
		self.folder_tree.append(name)
		return self.folder_path_name()
	}
	func folder_path_name() -> String {
		if self.folder_tree.count == 0 {
			return ""
		}
		var path:String = ""
		for i:Int in 0 ..< self.folder_tree.count {
			if i > 0 {
				path += "\\"
			}
			path += self.folder_tree[i]
		}
		return path
	}
	func file_full_path(_ filename:String) -> String {
		let path = self.folder_path_name()
		if path == "" {
			return filename
		} else {
			return path + "\\" + filename
		}
	}
	
	func releaseResources() {
		self.Session?.connection.disconnect()
		self.Session = nil
		self.TopTreeIndex = -1
		self.file_name_list.removeAll()
		self.folder_tree.removeAll()
	}
	
	@IBAction func touchUpConnect(_ sender: AnyObject) {
		let ip = txtIP.text
		let user = txtUser.text
		let pass = txtPassword.text
		
		if self.Session == nil {
			let conn:SMBConnection = SMBConnection(ip, port: 445)
			if conn.negotiate() {
				self.Session = SMBSession(conn)
                self.Session?.sign = self.swSign.isOn
				if (self.Session!.login("Levorg", user: user, password: pass)) {
					self.view.endEditing(true)
					self.show_share_list()
					self.btnConnect.setTitle("切 断", for: UIControl.State())
					let appDelegate:AppDelegate = UIApplication.shared.delegate as! AppDelegate
					appDelegate.SMBIP = ip!
					appDelegate.SMBUser = user!
					appDelegate.SMBPassword = pass!
                    appDelegate.SMBSign = self.swSign.isOn
                    appDelegate.saveSMBDefault()
				}
			}
		} else {
			self.releaseResources()
			tblFiles.reloadData()
			self.btnConnect.setTitle("接 続", for: UIControl.State())
		}
	}

	@IBAction func touchUpQuit(_ sender: AnyObject) {
		self.view.endEditing(true)
		self.dismiss(animated: true, completion: nil)
	}
}
