//
//  DER.m
//
//  Copyright (C) 2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2016/03/14	H.Kanemoto
//	Modified:
//
#import <Foundation/Foundation.h>
#import "DER.h"
static const UInt8 _OID_SPNEGO[] = {0x2b,0x06,0x01,0x05,0x05,0x02};
static const UInt8 _OID_NTLMSSP[] = {0x2b,0x06,0x01,0x04,0x01,0x82,0x37,0x02,0x02,0x0a};
/*----------------------------------------------------------------------*
 *	ASN.1 DER
 *----------------------------------------------------------------------*/
@implementation DER
/*----------------------------------------*
 *	Init/Dealloc
 *----------------------------------------*/
-(id)init {
	if (self = [super init]) {
		self->tag = 0;
		self->contents = nil;
		self->childs = [[NSMutableArray<DER *> alloc] init];
	}
	return self;
}
-(id)initWith:(UInt8)_tag : (bytecoder *)_contents {
	if (self = [super init]) {
		self->tag = _tag;
		self->contents = _contents;
		self->childs = [[NSMutableArray<DER *> alloc] init];
	}
	return self;
}

-(void)dealloc {
	DLog(@"");
	[self->childs removeAllObjects];
}
/*----------------------------------------*
 *	Properties
 *----------------------------------------*/
-(UInt8)getTag {
	return self->tag;
}
-(void)setTag:(UInt8)Tag {
	self->tag = Tag;
}
-(int32_t)getContentsLen {
	if (self->contents != nil)
		return self->contents.Length;
	else
		return 0;
}
-(UInt8 *)getContentsData {
	if (self->contents != nil)
		return self->contents.Data;
	else
		return nil;
}
-(UInt8 *)getSerializedLen {
	[self len_serialze];
	return self->len_seriarized;
}
-(int32_t)getSerializedLenCnt {
	[self len_serialze];
	return self->len_seriarized_cnt;
}
-(NSMutableArray<DER*>*)getSubItems {
	return self->childs;
}
/*----------------------------------------*
 *	convert Length to DER LENGTH Format
 *----------------------------------------*/
-(void)len_serialze {
	[self len_serialze:self.ContentsLen];
}

-(void)len_serialze:(int32_t)size {
	if (size > 128) {
		if (size < 0x100) {
			self->len_seriarized_cnt = 2;
			self->len_seriarized[0] = 0x81;
			self->len_seriarized[1] = (UInt8)size;
		} else if (size < 0x10000) {
			self->len_seriarized_cnt = 3;
			self->len_seriarized[0] = 0x82;
			self->len_seriarized[1] = (UInt8)((size >> 8) & 0xff);
			self->len_seriarized[2] = (UInt8)(size & 0xff);
		} else if (size < 0x1000000) {
			self->len_seriarized_cnt = 4;
			self->len_seriarized[0] = 0x83;
			self->len_seriarized[1] = (UInt8)((size >> 16) & 0xff);
			self->len_seriarized[2] = (UInt8)((size >> 8) & 0xff);
			self->len_seriarized[3] = (UInt8)(size & 0xff);
		} else {
			self->len_seriarized_cnt = 5;
			self->len_seriarized[0] = 0x84;
			self->len_seriarized[1] = (UInt8)((size >> 24) & 0xff);
			self->len_seriarized[2] = (UInt8)((size >> 16) & 0xff);
			self->len_seriarized[3] = (UInt8)((size >> 8) & 0xff);
			self->len_seriarized[4] = (UInt8)(size & 0xff);
		}
	} else {
		self->len_seriarized_cnt = 1;
		self->len_seriarized[0] = (UInt8)size;
	}
}
/*----------------------------------------*
 *	serialize DER in binary
 *----------------------------------------*/
-(int32_t)serialize:(bytecoder *)_coder {
	int32_t der_len = 1 + self.SerializedLenCnt + self.ContentsLen;
	if (_coder.UnreadCnt >= der_len) {
		
	}
	if (_coder.Remain < der_len) {
		int lack = der_len - _coder.Remain;
		if (![_coder expand_buffer:lack])
			return -1;
	}
	if (![_coder push_byte:self.Tag])
		return -1;
	if (![_coder push:self.SerializedLen Length:self.SerializedLenCnt])
		return -1;
	UInt8 *ptr = self.ContentsData;
	if (ptr != nil) {
		if (![_coder push:ptr Length:self.ContentsLen])
			return -1;
	}
	return der_len;
}
/*----------------------------------------*
 *	add contents
 *----------------------------------------*/
-(BOOL)add:(DER *)target {
	if (self->contents == nil) {
		self->contents = [[bytecoder alloc] init];
		if (self->contents == nil)
			return false;
	}
	return [target serialize:self->contents];
}

-(BOOL)add:(const UInt8 *)target :(int)_len {
	if (self->contents == nil) {
		self->contents = [[bytecoder alloc] init];
		if (self->contents == nil)
			return false;
	}
	return [self->contents push:target Length:_len];
}
-(bytecoder*)detach {
	bytecoder *b = self->contents;
	self->contents = nil;
	return b;
}
/*----------------------------------------*
 *	parse binary
 *----------------------------------------*/
+(BOOL)parse:(DER_PARSE_RESULT *)r {
	if (r.Data.UnreadCnt == 0) {
		return true;
	}
	int len;
	UInt8 tag;
	if (![r.Data pull_byte:&tag])
		return false;
	UInt8 siz1, siz2, siz3 = 0, siz4 = 0;
	if (![r.Data pull_byte:&siz1])
		return false;
	if (siz1 < 128) {
		len = siz1;
	} else {
		switch (siz1) {
			case 0x81:
				if (![r.Data pull_byte:&siz2])
					return false;
				len = siz2;
				break;
			case 0x82:
				if (![r.Data pull_byte:&siz2] || ![r.Data pull_byte:&siz3])
					return false;
				len = ((int)siz2*256) + siz3;
				break;
			case 0x84:
				if (![r.Data pull_byte:&siz2] || ![r.Data pull_byte:&siz3] || ![r.Data pull_byte:&siz4])
					return false;
				len = ((int)siz2*256*256) + ((int)siz3*256) + siz4;
				break;
			default:
				return false;
		}
	}
	if (r.Data.UnreadCnt >= len) {
		//copy of contents
		bytecoder *c = [r.Data cutout:r.Data.Offset Length:len];
		[r.Data moveptr:len];
		if (c == nil)
			return  false;
		//create der item
		DER *d = [[DER alloc] initWith:tag :c];
		if (d == nil)
			return false;
		//parse inline der
		if (tag >=0x1d || (tag & 0x1f) == 0x10 || (tag & 0x1f) == 0x11) {
			if (![d parse])
				return false;
		}
		//add item to array
		[r.Items addObject:d];
		
		//parse remain if any
		if (r.Data.UnreadCnt > 2)
			return [DER parse:r];
		else
			return true;
	}
	return false;
}

-(BOOL)parse {
	DER_PARSE_RESULT *r = [[DER_PARSE_RESULT alloc] initWith:self->contents];
	if (r != nil) {
		if ([DER parse:r]) {
			[self->childs addObjectsFromArray:r.Items];
			return true;
		}
	}
	return false;
}
/*----------------------------------------*
 *	create OID DER
 *----------------------------------------*/
+(DER *)OID_SPNEGO {
	DER *d = [[DER alloc] initWith:0x06 :nil];
	if (d != nil) {
		[d add:_OID_SPNEGO :6];
	}
	return d;
}

+(DER *)OID_NTLM {
	DER *d = [[DER alloc] initWith:0x06 :nil];
	if (d != nil) {
		[d add:_OID_NTLMSSP :10];
	}
	return d;
}
@end
/*----------------------------------------------------------------------
 *	DER Sequence Constructed
 *----------------------------------------------------------------------*/
@implementation DER_SEQ_CONSTRUCTED

-(id)initWith:(UInt8)_tag : (bytecoder *)_contents {
	if (self = [super initWith:_tag :nil]) {
		self->seq = [[DER alloc] initWith:0x30 :_contents];
	}
	return self;
}

-(BOOL)add:(DER *)target {
	return [self->seq add:target];
}

-(BOOL)add:(const UInt8 *)target :(int)_len {
	return [self->seq add:target :_len];
}

-(int32_t)serialize:(bytecoder *)_coder {
	if ([super add:self->seq]) {
		return [super serialize:_coder];
	}
	return -1;
}

@end
/*----------------------------------------------------------------------
 *	DER Parser Parameter
 *----------------------------------------------------------------------*/
@implementation DER_PARSE_RESULT

-(id)initWith:(bytecoder *)_coder {
	if (self = [super init]) {
		self->items = [[NSMutableArray<DER *> alloc] init];
		self->data = _coder;
	}
	return self;
}

-(NSMutableArray<DER *> *)getItems {
	return self->items;
}
-(bytecoder *)getData {
	return self->data;
}

-(void)dealloc {
	[self->items removeAllObjects];
	self->data = nil;
}
@end
