/**
 * \file md4.h
 *
 * \brief MD4 message digest algorithm (hash function)
 *
 *  Copyright (C) 2006-2015, ARM Limited, All Rights Reserved
 *  SPDX-License-Identifier: Apache-2.0
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); you may
 *  not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  This file is part of mbed TLS (https://tls.mbed.org)
 */

#ifndef MD4_h
#define MD4_h
typedef struct
{
	UInt32 total[2];		/*!< number of bytes processed  */
	UInt32 state[4];		/*!< intermediate digest state  */
	UInt8 buffer[64];		/*!< data block being processed */
}
mbedtls_md4_context;

@interface MD4 : NSObject {
	@private
	mbedtls_md4_context ctx;
}
-(void)start;
-(void)update:(UInt8 *)buffer : (int)length;
-(void)finish:(UInt8[16]) result;

@end

#endif /* MD4_h */
