//
//  RPC.h
//
//  Copyright (C) 2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2016/03/18	H.Kanemoto
//	Modified:
//
#ifndef RPC_h
#define RPC_h
#import "bytecoder.h"
#import "UCS2.h"
#import "DCE.h"

@class	rpcCtxList;
@class	rpcCtxElement;
@class	rpcSyntaxId;
@class	rpcShareInfo1;
/*----------------------------------------------------------------------*
 *	RPC Packet Header
 *----------------------------------------------------------------------*/
@interface rpcPacketHeader : TBObject {
@protected
	rpc_pkt_hdr				hdr;
}
@property (nonatomic, readonly, getter=getPktHdr) rpc_pkt_hdr *Header;

-(void)initrpchdr:(rpc_pkt_hdr *)h;

-(int)serialize:(bytecoder*)coder;
-(BOOL)deserialize:(bytecoder *)coder;

@end
/*----------------------------------------------------------------------*
 *	RPC Request Header
 *----------------------------------------------------------------------*/
@interface rpcRequestHeader : rpcPacketHeader {
@protected
	rpcconn_request_hdr_t	req_hdr;
}
@property (nonatomic, readonly, assign, getter=getReqHdr) rpcconn_request_hdr_t *ReqHeader;

-(void)initreqhdr:(rpcconn_request_hdr_t *)h;

@end
/*----------------------------------------------------------------------*
 *	RPC Response Header
 *----------------------------------------------------------------------*/
@interface rpcResponseHeader : rpcPacketHeader {
@protected
	rpcconn_response_hdr_t	res_hdr;
}
@property (nonatomic, readonly, assign, getter=getResHdr) rpcconn_response_hdr_t *ResHeader;

-(void)initreshdr:(rpcconn_response_hdr_t *)h;

@end
/*----------------------------------------------------------------------*
 *	RPC BIND PDU
 *----------------------------------------------------------------------*/
@interface rpcConnBindHeader : rpcPacketHeader {
@protected
	rpcconn_bind_hdr_t		bind;
	rpcCtxList				*ctxlist;
}
@property (nonatomic, readonly, getter=getBindHdr) rpcconn_bind_hdr_t *Bind;
@property (nonatomic, readonly, getter=getCtxList) rpcCtxList *CtxList;

@end
/*----------------------------------------------------------------------*
 *	RPC BIND ACK PDU
 *----------------------------------------------------------------------*/
@interface rpcConnBindAckHeader : rpcPacketHeader {
@protected
	UInt8					buffer[256];
	port_any_t				*port;
	p_result_list_t			*result;
}
@property (nonatomic, readonly, getter=getBindHdr) rpcconn_bind_ack_hdr_t *Ack;
@property (nonatomic, readonly, getter=getResult) p_result_list_t *Result;

@end
/*----------------------------------------------------------------------*
 *	RPC NetShareEnumAll Request PDU
 *----------------------------------------------------------------------*/
@interface rpcReqNetShareEnummAll : rpcRequestHeader {
	rpcconn_request_netshareenumall_h	net_h;
	rpcconn_request_netshareenumall_l	net_l;
	UCS2								*server_name;
}
@property (nonatomic, readonly, assign, getter=getNetH) rpcconn_request_netshareenumall_h *NetH;
@property (nonatomic, readonly, assign, getter=getNetL) rpcconn_request_netshareenumall_l *NetL;
@property (nonatomic) UCS2 *ServerName;

-(id)initWith:(NSString*)servername;

@end
/*----------------------------------------------------------------------*
 *	RPC NetShareEnumAll Response PDU
 *----------------------------------------------------------------------*/
@interface rpcResNetShareEnummAll : rpcResponseHeader {
	rpcconn_response_netshareenumall_h	net_h;
	rpcconn_response_netshareenumall_l	net_l;
	NSMutableArray<rpcShareInfo1*>		*list;
}
@property (nonatomic, readonly, assign, getter=getNetH) rpcconn_response_netshareenumall_h *NetH;
@property (nonatomic, readonly, assign, getter=getNetL) rpcconn_response_netshareenumall_l *NetL;
@property (nonatomic, readonly, getter=getList) NSMutableArray<rpcShareInfo1*> *List;

@end
/*----------------------------------------------------------------------*
 *	RPC CONTEXT LIST
 *----------------------------------------------------------------------*/
@interface rpcCtxList : TBObject {
@protected
	p_cont_list_t					list;
	NSMutableArray<rpcCtxElement *>	*elem;
}
@property (nonatomic, readonly, getter=getCtxList) p_cont_list_t *List;
@property (nonatomic, readonly, getter=getCtxElem) NSMutableArray<rpcCtxElement *> *Element;

-(int)serialize:(bytecoder*)coder;

@end
/*----------------------------------------------------------------------*
 *	RPC CONTEXT ELEMENT
 *----------------------------------------------------------------------*/
@interface rpcCtxElement : TBObject {
@protected
	p_cont_elem_t					elem;
	NSMutableArray<rpcSyntaxId *>	*tran;
}
@property (nonatomic, readonly, getter=getCtxElement) p_cont_elem_t *Element;
@property (nonatomic, readonly, getter=getTransfer) NSMutableArray<rpcSyntaxId *> *Transfer;

-(void)setAbstract:(rpcSyntaxId*)synid;
-(int)serialize:(bytecoder*)coder;

@end
/*----------------------------------------------------------------------*
 *	RPC SYNTAX ID
 *----------------------------------------------------------------------*/
@interface rpcSyntaxId : TBObject {
@protected
	p_syntax_id_t					synid;
}
@property (nonatomic, readonly, getter=getSynId) p_syntax_id_t *SynId;

+(rpcSyntaxId*)createSRVSVC;
+(rpcSyntaxId*)createNDR;

-(int)serialize:(bytecoder*)coder;

@end
/*----------------------------------------------------------------------*
 *	RPC SHARE INFO 1
 *----------------------------------------------------------------------*/
@interface rpcShareInfo1 : TBObject {
@protected
	p_share_info_1	info;
}
@property (nonatomic, readonly, assign, getter=getInfo) p_share_info_1 *ShareInfo;
@property (nonatomic, readonly, copy, getter=getFileName) NSString* FileName;
@property (nonatomic, readonly, copy, getter=getComment) NSString* Comment;

-(id)initWith:(NSString*)filename Comment:(NSString*)comment;

@end

#endif /* RPC_h */
