//
//  RPC.m
//
//  Copyright (C) 2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2016/03/18	H.Kanemoto
//	Modified:
//
#import <Foundation/Foundation.h>
#import "RPC.h"

static const UInt8 PACKED_DREP[4] = {0x10,0x00,0x00,0x00};
static const UInt8 SRVSVC[] = {0xc8,0x4f,0x32,0x4b,0x70,0x16,0xd3,0x01,0x12,0x78,0x5a,0x47,0xbf,0x6e,0xe1,0x88};
static const UInt8 NDR[] =    {0x04,0x5d,0x88,0x8a,0xeb,0x1c,0xc9,0x11,0x9f,0xe8,0x08,0x00,0x2b,0x10,0x48,0x60};
/*----------------------------------------------------------------------*
 *	RPC Packet Header
 *----------------------------------------------------------------------*/
@implementation rpcPacketHeader

-(id)init {
	if (self = [super init]) {
		[self initrpchdr:self.Header];
	}
	return self;
}

-(rpc_pkt_hdr *)getPktHdr {
	return &self->hdr;
}

-(void)initrpchdr:(rpc_pkt_hdr *)h {
	h->rpc_vers = 5;
	h->rpc_vers_minor = 0;
	h->PTYPE = 0;
	h->pfc_flags = 0;
	memcpy(h->packed_drep, PACKED_DREP, 4);
	h->frag_length = 0x48;
	h->auth_length = 0;
	h->call_id = 0;
}

-(int)serialize:(bytecoder*)coder {
	if ([coder push:(const UInt8 *)self.Header Length:sizeof(rpc_pkt_hdr)])
		return sizeof(rpc_pkt_hdr);
	else
		return -1;
}
-(BOOL)deserialize:(bytecoder *)coder {
	return [coder pull:(UInt8*)self.Header Length:sizeof(rpc_pkt_hdr)];
}

@end
/*----------------------------------------------------------------------*
 *	RPC Request Header
 *----------------------------------------------------------------------*/
@implementation rpcRequestHeader

-(id)init {
	if (self = [super init]) {
		[self initreqhdr:self.ReqHeader];
	}
	return self;
}

-(rpcconn_request_hdr_t *)getReqHdr {
	return &self->req_hdr;
}

-(void)initreqhdr:(rpcconn_request_hdr_t *)h {
	self->req_hdr.alloc_hint = 0;
	self->req_hdr.p_cont_id = 0;
	self->req_hdr.opnum = 0;
}

-(int)serialize:(bytecoder*)coder {
	int top_offset = coder.Offset;
	if ([super serialize:coder] > 0) {
		if ([coder push:(const UInt8 *)self.ReqHeader Length:sizeof(rpcconn_request_hdr_t)])
			return coder.Offset - top_offset;
	}
	return -1;
}

@end
/*----------------------------------------------------------------------*
 *	RPC Response Header
 *----------------------------------------------------------------------*/
@implementation rpcResponseHeader

-(id)init {
	if (self = [super init]) {
		[self initreshdr:self.ResHeader];
	}
	return self;
}

-(rpcconn_response_hdr_t *)getResHdr {
	return &self->res_hdr;
}

-(void)initreshdr:(rpcconn_response_hdr_t *)h {
	self->res_hdr.alloc_hint = 0;
	self->res_hdr.p_cont_id = 0;
	self->res_hdr.cancel_count = 0;
	self->res_hdr.reserved = 0;
}

-(int)serialize:(bytecoder*)coder {
	int top_offset = coder.Offset;
	if ([super serialize:coder] > 0) {
		if ([coder push:(const UInt8 *)self.ResHeader Length:sizeof(rpcconn_response_hdr_t)])
			return coder.Offset - top_offset;
	}
	return -1;
}
-(BOOL)deserialize:(bytecoder *)coder {
	if ([super deserialize:coder]) {
		return [coder pull:(UInt8*)self.ResHeader Length:sizeof(rpcconn_response_hdr_t)];
	}
	return false;
}

@end
/*----------------------------------------------------------------------*
 *	RPC BIND PDU
 *----------------------------------------------------------------------*/
@implementation rpcConnBindHeader

-(id)init {
	if (self = [super init]) {
		[self initrpcbind:self.Bind];
		self->ctxlist = [[rpcCtxList alloc] init];
	}
	return self;
}

-(rpcconn_bind_hdr_t *)getBindHdr {
	return &self->bind;
}
-(rpcCtxList *)getCtxList {
	return self->ctxlist;
}

-(void)initrpcbind:(rpcconn_bind_hdr_t *)b {
	self.Header->PTYPE = 11;	//bind
	self.Header->pfc_flags = 3;
	b->max_xmit_frag = 0xffff;
	b->max_recv_frag = 0xffff;
	b->assoc_group_id = 0;
}

-(int)serialize:(bytecoder*)coder {
	int top_offset = coder.Offset;
	if ([super serialize:coder] > 0) {
		if ([coder push:(const UInt8 *)self.Bind Length:sizeof(rpcconn_bind_hdr_t)]) {
			[coder pad4bytes_align:true];	//align 4bytes from begining of RPC Header
			if ([self->ctxlist serialize:coder] >= 0) {
				return coder.Offset - top_offset;
			}
		}
	}
	return -1;
}
@end
/*----------------------------------------------------------------------*
 *	RPC BIND PDU
 *----------------------------------------------------------------------*/
@implementation rpcConnBindAckHeader

-(id)init {
	if (self = [super init]) {
		self->_Ack = nil;
		self->_Result = nil;
	}
	return self;
}

-(BOOL)deserialize:(bytecoder *)coder {
	[super deserialize:coder];
	UInt16 v;
	if (![coder pull_le_short:&v])
		return false;
	self.Ack->max_xmit_frag = v;
	if (![coder pull_le_short:&v])
		return false;
	self.Ack->max_recv_frag = v;
	
	return true;
}
@end
/*----------------------------------------------------------------------*
 *	RPC NetShareEnumAll Request PDU
 *----------------------------------------------------------------------*/
@implementation rpcReqNetShareEnummAll

-(id)initWith:(NSString*)servername {
	if (self = [super init]) {
		self.Header->PTYPE = 0;	//request
		self.Header->pfc_flags = 3;
		self.ReqHeader->opnum = 15;
		self.NetL->level = 1;
		self.NetL->max_buffer = 0xffffffff;
		self.NetL->ctl.ctl = 1;
		self.NetL->ctl.ref_id = 1;
		self.NetL->ctl.count = 0;
		self.NetL->ctl.pt2array = 0;
		self.NetL->handle.ref_id = 1;
		self.NetL->handle.handle = 0;
		self.ServerName = [UCS2 create:servername];
		self.NetH->server_name.ref_id = 1;
		self.NetH->server_name.offset = 0;
		self.NetH->server_name.act_count = (UInt32)[servername length];
		self.NetH->server_name.max_count = (UInt32)[servername length];
	}
	return self;
}

-(rpcconn_request_netshareenumall_h*)getNetH {
	return &self->net_h;
}
-(rpcconn_request_netshareenumall_l*)getNetL {
	return &self->net_l;
}

-(int)serialize:(bytecoder*)coder {
	coder.TopOffset = -1*coder.Offset;	//global offset = 0
	int top_offset = coder.Offset;
	if ([super serialize:coder] > 0) {
		[coder pad8bytes_align:true];
		if (![coder push:(const UInt8 *)self.NetH Length:sizeof(rpcconn_request_netshareenumall_h)])
			return -1;
		if (![coder push:self.ServerName.Ptr Length:self.ServerName.ByteLen])
			return -1;
		[coder pad4bytes_align:true];
		if (![coder push:(const UInt8 *)self.NetL Length:sizeof(rpcconn_request_netshareenumall_l)])
			return -1;
		int size = coder.Offset - top_offset;
		((rpc_pkt_hdr*)&coder.Data[top_offset])->frag_length = size;
		((rpcconn_request_hdr_t*)&coder.Data[top_offset+sizeof(rpc_pkt_hdr)])->alloc_hint = size - 24;
		return size;
	}
	return -1;
}

@end
/*----------------------------------------------------------------------*
 *	RPC NetShareEnumAll Response PDU
 *----------------------------------------------------------------------*/
@implementation rpcResNetShareEnummAll

-(id)init {
	if (self = [super init]) {
		self->list = [[NSMutableArray<rpcShareInfo1*> alloc] init];
	}
	return self;
}

-(rpcconn_response_netshareenumall_h*)getNetH {
	return &self->net_h;
}
-(rpcconn_response_netshareenumall_l*)getNetL {
	return &self->net_l;
}
-(NSMutableArray<rpcShareInfo1*>*)getList {
	return self->list;
}

-(BOOL)deserialize:(bytecoder *)coder {
	coder.TopOffset = -1*coder.Offset;	//global offset = 0
	[super deserialize:coder];
	[coder skip8bytes_align:true];
	[coder pull:(UInt8*)self.NetH Length:sizeof(rpcconn_response_netshareenumall_h)];
	p_share_info_1 array[self.NetH->ctl.count];
	for (int i = 0; i < self.NetH->ctl.count; ++i) {
		[coder skip4bytes_align:true];
		[coder pull:(UInt8*)&array[i] Length:sizeof(p_share_info_1)];
	}
	p_string2 name;
	p_string2 comment;
	for (int i = 0; i < self.NetH->ctl.count; ++i) {
		[coder skip4bytes_align:true];
		[coder pull:(UInt8*)&name Length:sizeof(p_string2)];
		UInt8 name_byte[name.max_count*2];
		[coder pull:name_byte Length:name.max_count*2];
		NSString *fn = [[UCS2 create:name_byte length:(name.act_count - 1)*2] text];
		
		[coder skip4bytes_align:true];
		[coder pull:(UInt8*)&comment Length:sizeof(p_string2)];
		UInt8 comment_byte[comment.max_count*2];
		[coder pull:(UInt8*)comment_byte Length:comment.max_count*2];
		NSString *cm = [[UCS2 create:comment_byte length:(comment.act_count - 1)*2] text];
		
		rpcShareInfo1 *item = [[rpcShareInfo1 alloc] initWith:fn Comment:cm];
		*item.ShareInfo = array[i];
		[self.List addObject:item];
	}
	[coder skip4bytes_align:true];
	[coder pull:(UInt8*)&self.NetL->total_count Length:4];
	[coder pull:(UInt8*)&self.NetL->handle Length:sizeof(p_resume_handle)];
	return true;
}

@end
/*----------------------------------------------------------------------*
 *	RPC CONTEXT LIST
 *----------------------------------------------------------------------*/
@implementation rpcCtxList

-(id)init {
	if (self = [super init]) {
		[self initctxlist:self.List];
		self->elem =[[NSMutableArray<rpcCtxElement *> alloc] init];
	}
	return self;
}

-(p_cont_list_t *)getCtxList {
	return &self->list;
}
-(NSMutableArray<rpcCtxElement *> *)getCtxElem {
	return self->elem;
}

-(void)initctxlist:(p_cont_list_t *)l {
	l->n_context_elem = 0;
	l->reserved = 0;
	l->reserved2 = 0;
}

-(int)serialize:(bytecoder*)coder {
	int top_offset = coder.Offset;
	self.List->n_context_elem = [self.Element count];
	if (![coder push:(const UInt8*)self.List Length:sizeof(p_cont_list_t)])
		return -1;
	for (int i = 0; i < self.List->n_context_elem; ++i) {
		[coder pad4bytes_align:true];	//align 4bytes from begining of RPC Header
		rpcCtxElement *e = [self.Element objectAtIndex:i];
		if ([e serialize:coder] < 0)
			return -1;
	}
	return coder.Offset - top_offset;
}
@end
/*----------------------------------------------------------------------*
 *	RPC CONTEXT ELEMENT
 *----------------------------------------------------------------------*/
@implementation rpcCtxElement

-(id)init {
	if (self = [super init]) {
		[self initctxelem:self.Element];
		self->tran =[[NSMutableArray<rpcSyntaxId *> alloc] init];
	}
	return self;
	return self;
}

-(p_cont_elem_t *)getCtxElement {
	return &self->elem;
}
-(NSMutableArray<rpcSyntaxId *> *)getTransfer {
	return self->tran;
}

-(void)initctxelem:(p_cont_elem_t *)e {
	e->p_cont_id = 0;
	e->n_transfer_syn = 0;
	e->reserved = 0;
}

-(void)setAbstract:(rpcSyntaxId*)synid {
	self.Element->abstract_syntax.if_version = synid.SynId->if_version;
	memcpy(self.Element->abstract_syntax.if_uuid, synid.SynId->if_uuid, 16);
}

-(int)serialize:(bytecoder*)coder {
	int top_offset = coder.Offset;
	self.Element->n_transfer_syn = [self.Transfer count];
	if (![coder push:(UInt8 *)self.Element Length:sizeof(p_cont_elem_t)])
		return -1;
	for (int i = 0; i < self.Element->n_transfer_syn; ++i) {
		[coder pad4bytes_align:true];	//align 4bytes from begining of RPC Header
		rpcSyntaxId *s = [self.Transfer objectAtIndex:i];
		if ([s serialize:coder] < 0)
			return -1;
	}
	return coder.Offset - top_offset;
}
@end
/*----------------------------------------------------------------------*
 *	RPC SYNTAX ID
 *----------------------------------------------------------------------*/
@implementation rpcSyntaxId

-(id)init {
	if (self = [super init]) {
		memset(self.SynId, 0, sizeof(p_syntax_id_t));
	}
	return self;
}

-(p_syntax_id_t *)getSynId {
	return &self->synid;
}

-(int)serialize:(bytecoder*)coder {
	if ([coder push:(const UInt8 *)self.SynId Length:sizeof(p_syntax_id_t)])
		return sizeof(p_syntax_id_t);
	else
		return -1;
}

+(rpcSyntaxId*)createSRVSVC {
	rpcSyntaxId *syn = [[rpcSyntaxId alloc] init];
	memcpy(syn.SynId->if_uuid, SRVSVC, 16);
	syn.SynId->if_version = 3;
	return syn;
}

+(rpcSyntaxId*)createNDR {
	rpcSyntaxId *syn = [[rpcSyntaxId alloc] init];
	memcpy(syn.SynId->if_uuid, NDR, 16);
	syn.SynId->if_version = 2;
	return syn;
}
@end
/*----------------------------------------------------------------------*
 *	RPC SHARE INFO 1
 *----------------------------------------------------------------------*/
@implementation rpcShareInfo1

-(id)initWith:(NSString *)filename Comment:(NSString *)comment {
	if (self = [super init]) {
		memset(&self->info, 0, sizeof(p_share_info_1));
		self->_FileName = filename;
		self->_Comment = comment;
	}
	return self;
}

-(p_share_info_1*)getInfo {
	return &self->info;
}

@end
