//
//  NTLMSSP.h
//
//  Copyright (C) 2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2016/03/13	H.Kanemoto
//	Modified:
//
#ifndef NTLMSSP_h
#define NTLMSSP_h
#import "bytecoder.h"
#import "SMBUtils.h"

#define	NtLmNegotiate		(1U)
#define	NtLmChallenge		(2U)
#define	NtLmAuthenticate	(3U)

#define NTLMSSP_NEGOTIATE_UNICODE 					(0x00000001U)
#define NTLMSSP_NEGOTIATE_OEM 						(0x00000002U)
#define NTLMSSP_NEGOTIATE_REQUEST_TARGET 			(0x00000004U)
#define NTLMSSP_NEGOTIATE_SIGN 						(0x00000010U)
#define NTLMSSP_NEGOTIATE_SEAL 						(0x00000020U)
#define NTLMSSP_NEGOTIATE_DATAGRAM 					(0x00000040U)
#define NTLMSSP_NEGOTIATE_LM_KEY 					(0x00000080U)
#define NTLMSSP_NEGOTIATE_NETWARE 					(0x00000100U)
#define NTLMSSP_NEGOTIATE_NTLM 						(0x00000200U)
#define NTLMSSP_NEGOTIATE_ANONYMOUS 				(0x00000800U)
#define NTLMSSP_NEGOTIATE_OEM_DOMAIN_SUPPLIED 		(0x00001000U)
#define NTLMSSP_NEGOTIATE_OEM_WORKSTATION_SUPPLIED	(0x00002000U)
#define NTLMSSP_NEGOTIATE_LOCALCALL 				(0x00004000U)
#define NTLMSSP_NEGOTIATE_ALWAYS_SIGN 				(0x00008000U)
#define NTLMSSP_NEGOTIATE_TARGET_TYPE_DOMAIN 		(0x00010000U)
#define NTLMSSP_NEGOTIATE_TARGET_TYPE_SERVER 		(0x00020000U)
#define NTLMSSP_NEGOTIATE_TargetTypeShare 			(0x00040000U)
//#define NTLMSSP_NEGOTIATE_NTLM2 					(0x00080000U)
#define NTLMSSP_NEGOTIATE_EXTENDED_SESSIONSECURITY 	(0x00080000U)	//alt name NTLM2
#define NTLMSSP_NEGOTIATE_IDENTIFY 					(0x00100000U)
#define NTLMSSP_NEGOTIATE_RequestAcceptResponse 	(0x00200000U)
#define NTLMSSP_NEGOTIATE_REQUEST_NON_NT_SESSION_KEY (0x00400000U)
#define NTLMSSP_NEGOTIATE_TARGETINFO 				(0x00800000U)
#define NTLMSSP_NEGOTIATE_VERSION 					(0x02000000U)
#define NTLMSSP_NEGOTIATE_128 						(0x20000000U)
#define NTLMSSP_NEGOTIATE_KEY_EXCH 					(0x40000000U)
#define NTLMSSP_NEGOTIATE_56 						(0x80000000U)

#define	MsvAvEOL				(0x0U)
#define	MsvAvNbComputerName		(0x1U)	//NetBIOS Name
#define	MsvAvNbDomainName		(0x2U)	//NetBIOS Domain
#define	MsvAvDnsComputerName	(0x3U)
#define	MsvAvDnsDomainName		(0x4U)
#define	MsvAvDnsTreeName		(0x5U)
#define	MsvAvFlags				(0x6U)	//A 32-bit value indicating server or client configuration.
#define	MsvAvTimestamp			(0x7U)
#define	MsvAvSingleHost			(0x8U)
#define	MsvAvTargetName			(0x9U)
#define	MsvChannelBindings		(0xaU)

typedef struct __attribute__((packed)) {
	int16_t	Size;
	int16_t	Max;
	UInt32	Offset;
}	SecurityBuffer;

typedef struct __attribute__((packed)) {
	UInt8	SIGN[8];
	UInt32	Type;
	UInt32	Flags;
	SecurityBuffer	Domain;
	SecurityBuffer	Workstation;
	UInt8	Version[8];
	UInt8	buffer[];
}	NTLM_TYPE1;

typedef struct __attribute__((packed)) {
	UInt8	SIGN[8];
	UInt32	Type;
	SecurityBuffer	TargetName;
	UInt32	Flags;
	UInt8	Challenge[8];
	UInt8	Context[8];
	SecurityBuffer	TargetInfo;
	UInt8	buffer[];
}	NTLM_TYPE2;

typedef struct __attribute__((packed)) {
	UInt8	SIGN[8];
	UInt32	Type;
	SecurityBuffer	LMRes;
	SecurityBuffer	NTLMRes;
	SecurityBuffer	Domain;
	SecurityBuffer	User;
	SecurityBuffer	WS;
	SecurityBuffer	SessionKey;
	UInt32	Flags;
	UInt8	Version[8];
	//UInt8	MIC[16];
	UInt8	buffer[];
}	NTLM_TYPE3;

typedef struct __attribute__((packed)) {
	UInt8	RespType;
	UInt8	HiRespType;
	UInt16	Reserved1;
	UInt32	Reserved2;
	UInt8	TimeStamp[8];
	UInt8	ChallengeFromClient[8];
	UInt32	Reserved3;
	UInt8	buffer[];	//TargetInfo+Reserved4+Padding
}	NTLMv2_CLIENT_CHALLENGE;

typedef struct __attribute__((packed)) {
	UInt8	Response[16];	//NTProofStr
	NTLMv2_CLIENT_CHALLENGE	cc;
}	NTLMv2_RESPONSE;

typedef struct __attribute__((packed)) {
	UInt16	AvId;
	int16_t	AvLen;
	UInt8	Value[];
}	AV_PAIR;
/*----------------------------------------------------------------------*
 *	NTLM Helper
 *----------------------------------------------------------------------*/
@interface NTLM	: TBObject {
}
+(void)NtLmHash:(NSString*)password Result:(UInt8[16])hash;
+(void)NtLmV2Hash:(NSString*)target User:(NSString*)user Password:(NSString*)pass Result:(UInt8[16])hash;
+(void)HMACMD5:(UInt8*)target Length:(int)target_len Key:(UInt8*)key KeyLength:(int)key_len Result:(UInt8[16])hash;
+(void)HMACSHA256:(UInt8*)target Length:(int)target_len Key:(UInt8*)key KeyLength:(int)key_len Result:(UInt8[32])hash;
@end
/*----------------------------------------------------------------------*
 *	AvPair
 *----------------------------------------------------------------------*/
@interface AvPair : packetbase {
}
@property (nonatomic, readonly, assign, getter=getAV_PAIR) AV_PAIR *Pair;

@end
/*----------------------------------------------------------------------*
 *	NTLMv2 RESPONSE/NTLMv2 CLIENT CHALLENGE
 *----------------------------------------------------------------------*/
@interface NtLmResponse : packetbase {
	NTLMv2_RESPONSE			*response;	//for debugger
	NTLMv2_CLIENT_CHALLENGE	*cc;		//for debugger
	NSMutableArray<AvPair*>	*infos;
}
@property (nonatomic, readonly, assign, getter=getRes) NTLMv2_RESPONSE *Res;
@property (nonatomic, readonly, assign, getter=getCC) NTLMv2_CLIENT_CHALLENGE *CC;
@property (nonatomic, readonly, getter=getInfos) NSMutableArray<AvPair*> *TargetInfo;

-(BOOL)serialize;

@end
/*----------------------------------------------------------------------*
 *	NTLM NEGOTIATE MESSAGE (TYPE1)
 *----------------------------------------------------------------------*/
@interface NtLmType1 : packetbase {
@protected
	NTLM_TYPE1	*type;
}
@property (nonatomic, assign, getter=getType1, setter=setType1:) NTLM_TYPE1 *Type1;

-(id)initWith:(bytecoder *)bytecoder;

@end
/*----------------------------------------------------------------------*
 *	NTLM CHALLENGE MESSAGE (TYPE2)
 *----------------------------------------------------------------------*/
@interface NtLmType2 : packetbase {
@protected
	NTLM_TYPE2				*type;
	NSMutableArray<AvPair*>	*infos;
	UInt8					*target_name;
	int						target_name_len;
	UCS2					*target_name_ucs2;
}
@property (nonatomic, readonly, assign, getter=getType2) NTLM_TYPE2 *Type2;
@property (nonatomic, readonly, getter=getInfos) NSMutableArray<AvPair*> *TargetInfo;
@property (nonatomic, readonly, getter=getTargetname) NSString *TargetName;

-(BOOL)deserialize;

@end
/*----------------------------------------------------------------------*
 *	NTLM AUTHENTICATE MESSAGE (TYPE3)
 *----------------------------------------------------------------------*/
@interface NtLmType3 : packetbase {
@protected
	NTLM_TYPE3				*type;
	UInt8					mic[16];
	UCS2					*user;
	UCS2					*domain;
	UCS2					*ws;
	UInt8					lm[24];
}
@property (nonatomic, readonly, assign, getter=getType3) NTLM_TYPE3 *Type3;

-(BOOL)serialize:(NSString*)userdom Machine:(NSString*)workstation User:(NSString*)user
	NTLMResponse:(bytecoder*)ntlmres SessionKey:(UInt8*)sessionkey;

@end

#endif /* NTLMSSP_h */
