//
//  NTLMSSP.m
//
//  Copyright (C) 2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2016/03/14	H.Kanemoto
//	Modified:
//
#import <Foundation/Foundation.h>
#import <CommonCrypto/CommonHMAC.h>
#import "NTLMSSP.h"
#import "SMBUtils.h"
#import "MD4.h"
#import "MD5.h"
#import "RC4.h"
#include <malloc/malloc.h>

static const UInt8 NTLMSSP[] = {0x4e,0x54,0x4c,0x4d,0x53,0x53,0x50,0x00};
static const UInt8 VERSION[] = {0x06,0x01,0xb0,0x1d,0x00,0x00,0x00,0x0f};
/*----------------------------------------------------------------------*
 *	NTLM Helper
 *----------------------------------------------------------------------*/
@implementation NTLM
+(void)NtLmHash:(NSString*)password Result:(UInt8[16])hash {
	UCS2 *u = [UCS2 create:password];
	MD4 *md4 = [[MD4 alloc] init];
	[md4 start];
	[md4 update:u.Ptr :u.ByteLen];
	[md4 finish:hash];
}

+(void)NtLmV2Hash:(NSString*)target User:(NSString*)user Password:(NSString*)pass Result:(UInt8[16])hash {
	UInt8 ntlmhash[16];
	[self NtLmHash:pass Result:ntlmhash];
	
	NSString *USER = [user uppercaseString];
	NSString *TARGET = [target uppercaseString];
	NSString *cat = [USER stringByAppendingString:TARGET];
	UCS2 *catucs = [UCS2 create:cat];
	[self HMACMD5:catucs.Ptr Length:catucs.ByteLen Key:ntlmhash KeyLength:16 Result:hash];
}

+(void)HMACMD5:(UInt8*)target Length:(int)target_len Key:(UInt8*)key KeyLength:(int)key_len Result:(UInt8[16])hash {
	UInt8 ipad[64], opad[64], data[16];
	for (int i = 0; i < 64; ++i) {
		ipad[i] = 0x36;
		opad[i] = 0x5c;
	}
	for (int i = key_len - 1; i >= 0; i--) {
		ipad[i] ^= key[i];
		opad[i] ^= key[i];
	}
	UInt8 content[target_len + 64];
	memcpy(content, ipad, 64);
	memcpy(&content[64], target, target_len);
	MD5 *md5 = [[MD5 alloc] init];
	[md5 start];
	[md5 update:content :target_len+64];
	[md5 finish:data];
	
	UInt8 content2[16 + 64];
	memcpy(content2, opad, 64);
	memcpy(&content2[64], data, 16);
	[md5 start];
	[md5 update:content2 :16 + 64];
	[md5 finish:hash];
}
+(void)HMACSHA256:(UInt8*)target Length:(int)target_len Key:(UInt8*)key KeyLength:(int)key_len Result:(UInt8[32])hash {
	CCHmac(kCCHmacAlgSHA256, key, key_len, target, target_len, hash);
}
@end
/*----------------------------------------------------------------------*
 *	AVPair
 *----------------------------------------------------------------------*/
@implementation AvPair

-(id)init {
	if (self = [super init]) {
		if ([self.Coder alloc_buffer:128])
			[self.Coder consume:sizeof(AV_PAIR)];
		else
			return nil;
	}
	return self;
}

-(id)initWith:(bytecoder *)_coder {
	if (self = [super initWith:_coder]) {
	}
	return self;
}

-(AV_PAIR*)getAV_PAIR {
	return (AV_PAIR*)self.Coder.Data;
}
@end
/*----------------------------------------------------------------------*
 *	NTLMv2 RESPONSE/NTLMv2 CHALLENGE
 *----------------------------------------------------------------------*/
@implementation NtLmResponse

-(id)init {
	if (self = [super init]) {
		if ([self.Coder alloc_buffer:1024]) {
			self->response = (NTLMv2_RESPONSE *)self.Coder.Data;
			self->cc = &self->response->cc;
			[self initntlmv2:self.CC];
			[self.Coder consume:sizeof(NTLMv2_RESPONSE)];
			self->infos = [[NSMutableArray<AvPair*> alloc] init];
		} else
			return nil;
	}
	return self;
}

-(void)dealloc {
	DLog(@"");
	[self.TargetInfo removeAllObjects];
}

-(void)initntlmv2:(NTLMv2_CLIENT_CHALLENGE*)v2 {
	v2->RespType = 1;
	v2->HiRespType = 1;
	v2->Reserved1 = 0;
	v2->Reserved2 = 0;
	UInt8 stamp[8];
	[SMBUtils SMBTimeStamp:stamp];
	memcpy(v2->TimeStamp, stamp, 8);
	[SMBUtils RandomArray:v2->ChallengeFromClient Length:8];
	v2->Reserved3 = 0;
}

-(NTLMv2_RESPONSE*)getRes {
	return (NTLMv2_RESPONSE*)self.Coder.Data;
}
-(NTLMv2_CLIENT_CHALLENGE*)getCC {
	return &self.Res->cc;
}
-(NSMutableArray<AvPair*>*)getInfos {
	return self->infos;
}

-(BOOL)serialize {
	NSArray<AvPair*> *arr = [self->infos sortedArrayUsingComparator:^NSComparisonResult(id  _Nonnull obj1, id  _Nonnull obj2) {
		if (((AvPair*)obj1).Pair->AvId == 0)
			return 1;
		if (((AvPair*)obj2).Pair->AvId == 0)
			return -1;
		if (((AvPair*)obj1).Pair->AvId == ((AvPair*)obj2).Pair->AvId)
			return 0;
		if (((AvPair*)obj1).Pair->AvId > ((AvPair*)obj2).Pair->AvId)
			return 1;
		return -1;
	}];
	int cnt = (int)[arr count];
	for (int i = 0; i < cnt; ++i) {
		AvPair *item = [arr objectAtIndex:i];
		if (![self.Coder push:item.Coder.Data Length:item.Coder.Length])
			return false;
	}
	[self.Coder push_le_int:0];	//Reserved4
	[self.Coder push_le_int:0];	//Padding
	return true;
}

@end
/*----------------------------------------------------------------------*
 *	NTLM NEGOTIATE MESSAGE (TYPE1)
 *----------------------------------------------------------------------*/
@implementation NtLmType1

-(id)init {
	if (self = [super init]) {
		if ([self.Coder alloc_buffer:sizeof(NTLM_TYPE1)]) {
			self->type = (NTLM_TYPE1 *)self.Coder.Data;
			[self inittype1:self->type];
			[self.Coder consume:sizeof(NTLM_TYPE1)];
		} else
			return nil;
	}
	return self;
}

-(id)initWith:(bytecoder *)bytecoder {
	if (self = [super init]) {
		self->coder = bytecoder;
		self->type = self.Coder.Pointer;
	}
	return self;
}

-(NTLM_TYPE1 *)getType1 {
	return self->type;
}

-(void)inittype1:(NTLM_TYPE1 *)t {
	memcpy(t->SIGN, NTLMSSP, 8);
	t->Type = NtLmNegotiate;
	t->Flags =
	NTLMSSP_NEGOTIATE_KEY_EXCH +
	NTLMSSP_NEGOTIATE_128 +
	NTLMSSP_NEGOTIATE_VERSION +
	NTLMSSP_NEGOTIATE_TARGETINFO +
	NTLMSSP_NEGOTIATE_EXTENDED_SESSIONSECURITY +
	NTLMSSP_NEGOTIATE_ALWAYS_SIGN +
	NTLMSSP_NEGOTIATE_NTLM +
//	NTLMSSP_NEGOTIATE_SEAL +
	NTLMSSP_NEGOTIATE_SIGN +
	NTLMSSP_NEGOTIATE_REQUEST_TARGET +
	NTLMSSP_NEGOTIATE_UNICODE;
	memcpy(t->Version, VERSION, 8);
}

@end
/*----------------------------------------------------------------------*
  *	NTLM CHALLENGE MESSAGE (TYPE2)
 *----------------------------------------------------------------------*/
@implementation NtLmType2

-(id)initWith:(bytecoder *)_coder {
	if (self = [super initWith:_coder]) {
		self.Coder.TopOffset = 0;
		self->type = (NTLM_TYPE2 *)self.Coder.Data;
		[self.Coder moveptrto:sizeof(NTLM_TYPE2)];
		self->target_name = nil;
		self->target_name_len = 0;
		self->infos = [[NSMutableArray<AvPair*> alloc] init];
	}
	return self;
}

-(void)dealloc {
	NSLog(@"#### NtLmType2.dealloc ####");
	if (self->target_name != nil) {
		free(self->target_name);
		assert(malloc_zone_check(NULL));
		self->target_name = nil;
	}
	[self.TargetInfo removeAllObjects];
}

-(NTLM_TYPE2 *)getType2 {
	return self->type;
}
-(NSMutableArray<AvPair*>*)getInfos {
	return self->infos;
}
-(NSString*)getTargetname {
	return self->target_name_ucs2.text;
}

-(BOOL)deserialize {
	if (![self decode_targetname])
		return false;
	if (![self decode_targetinfo])
		return false;
	return true;
}

-(BOOL)decode_targetname {
	int offset = self.Type2->TargetName.Offset;
	int len = self.Type2->TargetName.Max;
	int eof = offset + len;
	if ((eof-1) < self.Coder.Length) {
		[self.Coder moveptrto:offset];
		self->target_name = calloc(len+2, 1);
		if (self->target_name != nil) {
			if ([self.Coder pull:self->target_name Length:len]) {
				self->target_name_len = len;
				self->target_name_ucs2 = [UCS2 create:self->target_name length:self->target_name_len];
				NSString *str = [self->target_name_ucs2 text];
				NSLog(@"%@", str);
				return true;
			}
		}
	}
	return false;
}

-(BOOL)decode_targetinfo {
	int offset = self.Type2->TargetInfo.Offset;
	int len = self.Type2->TargetInfo.Max;
	int eof = offset + len;
	if ((eof-1) < self.Coder.Length) {
		[self.Coder moveptrto:offset];
		while (self.Coder.Offset < eof) {
			UInt32 begin_offset = self.Coder.Offset;
			UInt16 avid;
			if (![self.Coder pull_le_short:&avid])
				return false;
			int16_t avlen;
			if (![self.Coder pull_le_short:(UInt16*)&avlen])
				return false;
			bytecoder *b = [self.Coder cutout:begin_offset Length:avlen+4];
			if (b == nil)
				return false;
			AvPair *item = [[AvPair alloc] initWith:b];
			if (item == nil)
				return false;
			[self.TargetInfo addObject:item];
			[self.Coder moveptr:avlen];
			if (avid == 0) {	//end of TypeInfo list
				break;
			}
		}
		return true;
	}
	return false;
}

@end
/*----------------------------------------------------------------------*
 *	NTLM AUTHENTICATE MESSAGE (TYPE3)
 *----------------------------------------------------------------------*/
@implementation NtLmType3

-(id)init {
	if (self = [super init]) {
		if ([self.Coder alloc_buffer:1024]) {
			self->type = (NTLM_TYPE3 *)self.Coder.Data;
			[self inittype3:self->type];
			[self.Coder consume:sizeof(NTLM_TYPE3)];
		} else
			return nil;
	}
	return self;
}

-(NTLM_TYPE3 *)getType3 {
	return self->type;
}

-(void)inittype3:(NTLM_TYPE3 *)t {
	memcpy(t->SIGN, NTLMSSP, 8);
	t->Type = NtLmAuthenticate;
	t->Flags =
	NTLMSSP_NEGOTIATE_KEY_EXCH +
	NTLMSSP_NEGOTIATE_128 +
	NTLMSSP_NEGOTIATE_VERSION +
	NTLMSSP_NEGOTIATE_TARGETINFO +
	NTLMSSP_NEGOTIATE_EXTENDED_SESSIONSECURITY +
	NTLMSSP_NEGOTIATE_ALWAYS_SIGN +
	NTLMSSP_NEGOTIATE_NTLM +
	//	NTLMSSP_NEGOTIATE_SEAL +
	NTLMSSP_NEGOTIATE_SIGN +
	NTLMSSP_NEGOTIATE_REQUEST_TARGET +
	NTLMSSP_NEGOTIATE_UNICODE;
	memcpy(t->Version, VERSION, 8);
	memset(self->lm, 0, 24);
}

-(BOOL)serialize:(NSString*)userdom
		Machine:(NSString*)workstation
		User:(NSString*)_user
		NTLMResponse:(bytecoder*)ntlmres
		SessionKey:(UInt8*)sessionkey {

	BOOL result = false;
	for (;;) {
		self->domain = [UCS2 create:userdom];
		self->ws = [UCS2 create:workstation];
		self->user = [UCS2 create:_user];
		
		self.Type3->User.Max = self->user.ByteLen;
		self.Type3->User.Size = self->user.ByteLen;
		self.Type3->User.Offset = self.Coder.Offset;
		if (![self.Coder push:self->user.Ptr Length:self->user.ByteLen])
			break;
		
		self.Type3->Domain.Max = self->domain.ByteLen;
		self.Type3->Domain.Size = self->domain.ByteLen;
		self.Type3->Domain.Offset = self.Coder.Offset;
		if (![self.Coder push:self->domain.Ptr Length:self->domain.ByteLen])
			break;
		
		self.Type3->WS.Max = self->ws.ByteLen;
		self.Type3->WS.Size = self->ws.ByteLen;
		self.Type3->WS.Offset = self.Coder.Offset;
		if (![self.Coder push:self->ws.Ptr Length:self->ws.ByteLen])
			break;
		
		self.Type3->LMRes.Size = 24;
		self.Type3->LMRes.Max = 24;
		self.Type3->LMRes.Offset = self.Coder.Offset;
		if (![self.Coder push:self->lm Length:24])
			break;
		
		self.Type3->NTLMRes.Size = ntlmres.Length;
		self.Type3->NTLMRes.Max = ntlmres.Length;
		self.Type3->NTLMRes.Offset = self.Coder.Offset;
		if (![self.Coder push:ntlmres.Data	Length:ntlmres.Length])
			break;
		
		self.Type3->SessionKey.Size = 16;
		self.Type3->SessionKey.Max = 16;
		self.Type3->SessionKey.Offset = self.Coder.Offset;
		if ([self.Coder push:sessionkey Length:16])
			result = true;
		break;
	}
	return result;
}
@end
