//
//  FSCC.h
//
//  Copyright (C) 2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2016/03/22	H.Kanemoto
//	Modified:
//
#ifndef FSCC_h
#define FSCC_h
#import "bytecoder.h"
#import "UCS2.h"

typedef struct  __attribute__((packed)) {
	UInt32	NextEntryOffset;
	UInt32	FileIndex;
	UInt64	CreationTime;
	UInt64	LastAccessTime;
	UInt64	LastWriteTime;
	UInt64	ChangeTime;
	int64_t	EndOfFile;
	int64_t	AllocationSize;
	UInt32	FileAttributes;
	int32_t	FileNameLength;
	int32_t	EaSize;
	int8_t	ShortNameLength;
	UInt8	Reserved1;
	UInt8	ShortName[24];
	UInt16	Reserved2;
	UInt8	FileId[8];
	//UInt8	FileName[];
}	FILE_ID_BOTH_DIR_INFORMATION;

typedef struct  __attribute__((packed)) {
	int64_t	AllocationSize;
	int64_t	EndOfFile;
	int32_t	NumberOfLinks;
	UInt8	DeletePending;
	UInt8	Directory;
	UInt16	Reserved;
}	FILE_STANDARD_INFORMATION;

typedef enum {
	FILE_ATTRIBUTE_ARCHIVE =				0x00000020U,
	FILE_ATTRIBUTE_COMPRESSED =				0x00000800U,
	FILE_ATTRIBUTE_DIRECTORY =				0x00000010U,
	FILE_ATTRIBUTE_ENCRYPTED =				0x00004000U,
	FILE_ATTRIBUTE_HIDDEN =					0x00000002U,
	FILE_ATTRIBUTE_NORMAL =					0x00000080U,
	FILE_ATTRIBUTE_NOT_CONTENT_INDEXED =	0x00002000U,
	FILE_ATTRIBUTE_OFFLINE =				0x00001000U,
	FILE_ATTRIBUTE_READONLY =				0x00000001U,
	FILE_ATTRIBUTE_REPARSE_POINT =			0x00000400U,
	FILE_ATTRIBUTE_SPARSE_FILE =			0x00000200U,
	FILE_ATTRIBUTE_SYSTEM =					0x00000004U,
	FILE_ATTRIBUTE_TEMPORARY =				0x00000100U,
	FILE_ATTRIBUTE_INTEGRITY_STREAM =		0x00008000U,
	FILE_ATTRIBUTE_NO_SCRUB_DATA =			0x00020000U,
}	FileAttributes;

/*----------------------------------------------------------------------*
 *	FileIdBothDirInformation
 *----------------------------------------------------------------------*/
@interface FileIdBothDirInformation : TBObject {
@protected
	FILE_ID_BOTH_DIR_INFORMATION	info;
	UCS2				*filename;
}
@property (nonatomic, readonly, assign, getter=getInfo) FILE_ID_BOTH_DIR_INFORMATION *Info;
@property (nonatomic, readonly, copy, getter=getFileName) NSString *FileName;
@property (nonatomic, readonly, assign, getter=getIsDir) BOOL IsDir;
@property (nonatomic, readonly, assign, getter=getIsReadonly) BOOL IsReadonly;

-(BOOL)deserialize:(bytecoder*)coder;

@end
/*----------------------------------------------------------------------*
 *	FileStandardInformation
 *----------------------------------------------------------------------*/
@interface FileStandardInformation : TBObject {
@protected
	FILE_STANDARD_INFORMATION	info;
}
@property (nonatomic, readonly, assign, getter=getInfo) FILE_STANDARD_INFORMATION *Info;
@property (nonatomic, readonly, assign, getter=getAlloc) int64_t AllocationSize;
@property (nonatomic, readonly, assign, getter=getEOF) int64_t EndOfFile;
@property (nonatomic, readonly, assign, getter=getDeletePending) BOOL DeletePending;
@property (nonatomic, readonly, assign, getter=getDirectory) BOOL Directory;

-(BOOL)deserialize:(bytecoder*)coder;

@end

#endif /* FSCC_h */
