//
//  SMB2Create.m
//
//  Copyright (C) 2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2016/03/18	H.Kanemoto
//	Modified:
//
#import <Foundation/Foundation.h>
#import "SMB2Create.h"

static const UInt8	MXAC[] = {0x4d,0x78,0x41,0x63};
/*----------------------------------------------------------------------*
 *	SMB2 CREATE REQUEST
 *----------------------------------------------------------------------*/
@implementation SMB2ReqCreate
/*----------------------------------------*
 *	Init/Dealloc
 *----------------------------------------*/
-(id)init {
	if (self = [super init]) {
		if ([self.Coder alloc_buffer:1024]) { //1Kbyte for payload
			[super initheader:self.Header];
			self.Header->Command = SMB2_COMMAND_CREATE;
			self.CREATE->StructureSize = CREATEREQSIZE;
			self.CREATE->NameOffset = 0x78;
			self->create = self.CREATE;	//for debugging
			//move pointer to payload top but except for dummy buffer[1]
			[self.Coder consume:PACKETHEADERSIZE+CREATEREQSIZE-1];
			self->ctxes = [[NSMutableArray<SMB2CreateContext*> alloc] init];
		} else {
			//[self release];
			return nil;
		}
	}
	return self;
}
/*----------------------------------------*
 *	Properties
 *----------------------------------------*/
-(reqCREATE *)getCreate {
	return (reqCREATE*) &self.Coder.Data[64];
}
-(UInt8)getOplockLevel {
	return self.CREATE->RequestedOplockLevel;
}
-(void)setOplockLevel:(UInt8)value {
	self.CREATE->RequestedOplockLevel = value;
}
-(UInt32)getDesiredAccess {
	return self.CREATE->DesiredAccess;
}
-(void)setDesiredAccess:(UInt32)value {
	self.CREATE->DesiredAccess = value;
}
-(UInt32)getFileAttributes {
	return self.CREATE->FileAttributes;
}
-(void)setFileAttributes:(UInt32)value {
	self.CREATE->FileAttributes = value;
}
-(UInt32)getShareAccess {
	return self.CREATE->ShareAccess;
}
-(void)setShareAccess:(UInt32)value {
	self.CREATE->ShareAccess = value;
}
-(UInt32)getCreateDisposition {
	return self.CREATE->CreateDisposition;
}
-(void)setCreateDisposition:(UInt32)value {
	self.CREATE->CreateDisposition = value;
}
-(UInt32)getCreateOptions {
	return self.CREATE->CreateOptions;
}
-(void)setCreateOptions:(UInt32)value {
	self.CREATE->CreateOptions = value;
}
/*----------------------------------------*
 *	Serialize/Deserialize
 *----------------------------------------*/
-(BOOL)serialize {
	//move pointer to payload top (except for dummy buffer[1])
	[self.Coder rewind];
	[self.Coder consume:PACKETHEADERSIZE+CREATEREQSIZE-1];
	
	//set 8byte-aligned length of filename
	int ln = 8; BOOL hasname = false;
	if (self->FileName != nil && self->FileName.ByteLen > 0) {
		ln = [SMBUtils Align8bytes:self->FileName.ByteLen];
		hasname = true;
	}
	UInt8 name[ln];
	memset(name, 0, ln);
	if (hasname)
		memcpy(name, self->FileName.Ptr, self->FileName.ByteLen);
	if (![self.Coder push:name Length:ln])
		return false;
	//Context
	if (![self.Coder pad8bytes_align:true])
		return false;
	if ([self->ctxes count] > 0) {
		self.CREATE->CreateContextOffset = self.Coder.Offset;
		self.Coder.TopOffset = -1*self.Coder.Offset;	//global offset = 0
		SMB2_CREATE_CONTEXT *prev = nil;
		for (int i = 0; i < [self->ctxes count]; ++i) {
			SMB2CreateContext *ctx = [self->ctxes objectAtIndex:i];
			//Context must begin in 8byte-aligned offset
			if (![self.Coder pad8bytes_align:true]) {
				return false;
			}
			//set Context chain
			if (prev != nil)
				prev->Next = self.Coder.GlobalOffset;
			prev = (SMB2_CREATE_CONTEXT *)&self.Coder.Data[self.Coder.Offset];
			//write Context
			if (![ctx serialize:self.Coder])
				return false;
		}
		if (![self.Coder pad8bytes_align:true])
			return false;
		self.CREATE->CreateContextLength = self.Coder.Offset - self.CREATE->CreateContextOffset;
	}
	return true;
}
/*----------------------------------------*
 *	Factory
 *----------------------------------------*/
+(SMB2ReqCreate*)createPipe:(UInt32)TreeId PipeName:(NSString*)pipename {
	SMB2ReqCreate *req = [[SMB2ReqCreate alloc] init];
	req.Header->s.Sync.TreeId = TreeId;
	req.CREATE->RequestedOplockLevel = 0;	//NONE
	req.CREATE->ImpersonationLevel = 2;		//Impersonation
	req.CREATE->SmbCreateFlags = 0;
	req.CREATE->DesiredAccess = FPPAM_GENERIC_READ + FPPAM_GENERIC_WRITE;
	req.CREATE->FileAttributes = FILE_ATTRIBUTE_NORMAL;
	req.CREATE->ShareAccess = FILE_SHARE_READ + FILE_SHARE_WRITE;
	req.CREATE->CreateDisposition = CD_FILE_OPEN;
	req->FileName = [UCS2 create:pipename];
	req.CREATE->NameLength = req->FileName.ByteLen;
	[req->ctxes addObject:[SMB2CreateContext createMxAc]];
	return req;
}

+(SMB2ReqCreate*)createDir:(UInt32)TreeId DirName:(NSString*)dirname {
	SMB2ReqCreate *req = [[SMB2ReqCreate alloc] init];
	req.Header->s.Sync.TreeId = TreeId;
	req.CREATE->RequestedOplockLevel = 0;	//NONE
	req.CREATE->ImpersonationLevel = 2;		//Impersonation
	req.CREATE->SmbCreateFlags = 0;
	req.CREATE->DesiredAccess = DAM_FILE_LIST_DIRECTORY + DAM_FILE_READ_ATTRIBUTES + DAM_SYNCHRONIZE;
	req.CREATE->FileAttributes = FILE_ATTRIBUTE_DIRECTORY;
	req.CREATE->ShareAccess = FILE_SHARE_READ + FILE_SHARE_WRITE;
	req.CREATE->CreateDisposition = CD_FILE_OPEN;
	req.CREATE->CreateOptions = CO_FILE_DIRECTORY_FILE;
	req->FileName = [UCS2 create:dirname];
	req.CREATE->NameLength = req->FileName.ByteLen;
	return req;
}

+(SMB2ReqCreate*)createFile:(UInt32)TreeId FileName:(NSString*)filename {
	SMB2ReqCreate *req = [[SMB2ReqCreate alloc] init];
	req.Header->s.Sync.TreeId = TreeId;
	req.Header->CreditCharge = 1;
	req.Header->CreditRequest = 256;
	req.CREATE->RequestedOplockLevel = 0;	//NONE
	req.CREATE->ImpersonationLevel = 2;		//Impersonation
	req.CREATE->SmbCreateFlags = 0;
	req.CREATE->DesiredAccess =
		FPPAM_FILE_READ_DATA +
		FPPAM_FILE_READ_ATTRIBUTES +
		FPPAM_FILE_WRITE_DATA +
		FPPAM_FILE_WRITE_ATTRIBUTES +
		FPPAM_DELETE;
	req.CREATE->FileAttributes = FILE_ATTRIBUTE_NORMAL;
	req.CREATE->ShareAccess = FILE_SHARE_READ + FILE_SHARE_WRITE + FILE_SHARE_DELETE;
	req.CREATE->CreateDisposition = CD_FILE_OPEN;
	req.CREATE->CreateOptions = 0;
	req->FileName = [UCS2 create:filename];
	req.CREATE->NameLength = req->FileName.ByteLen;
	return req;
}

@end
/*----------------------------------------------------------------------*
 *	SMB2 CREATE RESPONSE
 *----------------------------------------------------------------------*/
@implementation SMB2ResCreate

-(id)init {
	if (self = [super init]) {
		if ([self.Coder alloc_buffer:PACKETHEADERSIZE+CREATERESSIZE]) {
		} else
			return nil;
	}
	return self;
}

-(resCREATE *)getCreate {
	return (resCREATE*) &self.Coder.Data[64];
}
@end
/*----------------------------------------------------------------------*
 *	SMB2 CREATE CONTEXT
 *----------------------------------------------------------------------*/
@implementation SMB2CreateContext

-(id)init {
	if (self = [super init]) {
		memset(self.CONTEXT, 0, sizeof(SMB2_CREATE_CONTEXT));
		self.CONTEXT->NameOffset = 0x10;
	}
	return self;
}
-(SMB2_CREATE_CONTEXT *)getContext {
	return &self->context;
}

-(BOOL)serialize:(bytecoder*)coder {
	self.CONTEXT->NameLength = self->name_len;
	if ([coder push:(UInt8 *)self.CONTEXT Length:sizeof(SMB2_CREATE_CONTEXT)]) {
		int nl = [SMBUtils Align8bytes:self->name_len];
		return [coder push:self->name Length:nl];
	}
	return false;
}

+(SMB2CreateContext*)createMxAc {
	SMB2CreateContext *ctx = [[SMB2CreateContext alloc] init];
	memset(ctx->name, 0, 16);
	memcpy(ctx->name, MXAC, 4);
	ctx->name_len = 4;
	return ctx;
}
@end
