//
//  SMB2Header.m
//
//  Copyright (C) 2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2016/03/12	H.Kanemoto
//	Modified:
//
#import <Foundation/Foundation.h>
#import "SMB2Header.h"

static const UInt8 SMB2PROTOCOLID[] = {0xfe, 0x53, 0x4d, 0x42};
static const UInt8 SMB1PROTOCOLID[] = {0xff, 0x53, 0x4d, 0x42};
/*----------------------------------------------------------------------*
 *	SMB2 PACKET HEADER
 *----------------------------------------------------------------------*/
@implementation SMB2Header
/*----------------------------------------*
 *	Init/Dealloc
 *----------------------------------------*/
-(id)init {
	if (self = [super init]) {
		self->transport = 0;
	}
	return self;
}

-(void)initheader:(pPacketHeader)ph {
	ph->ProtocolId[0]		= 0xfe;
	ph->ProtocolId[1]		= 0x53;
	ph->ProtocolId[2]		= 0x4d;
	ph->ProtocolId[3]		= 0x42;
	ph->StructureSize		= PACKETHEADERSIZE;
	ph->CreditCharge		= 1;
	ph->Status				= 0;
	ph->Command				= 0;
	ph->CreditRequest		= 1;
	ph->Flags				= 0;
	ph->NextCommand			= 0;
	ph->MessageId			= 0;
	ph->s.Sync.Reserved		= 0x0000feff;
	ph->s.Sync.TreeId		= 0;
	ph->SessionId			= 0;
	memset(ph->Signature, 0, 16);
	self->_packhead = ph;	//for xcode debugger
}
/*----------------------------------------*
 *	Properties
 *----------------------------------------*/
-(pPacketHeader)getPacketHeader {
	return (pPacketHeader) self.Coder.Data;
}

-(resERROR *)getError {
	return (resERROR *)self.Header->Payload;
}
/*----------------------------------------*
 *	recv packet from TCP
 *----------------------------------------*/
-(BOOL)recvResponse:(TcpClient*)tcp {
	return [self recvResponse:tcp mSec:3000];
}
-(BOOL)recvResponse:(TcpClient*)tcp mSec:(int)msec {
	UInt8 buff[4];
	//recv Transport(size of whole SMB2 Packet)
	int cnt = [tcp recv:(char *)buff Lenght:4 mSec:msec];
	if (cnt == 4) {
		self->transport = [SMBUtils Byte2IntBE:buff];
		if (self->transport < 0x1000000) {
			//alloc SMB Packet buffer
			if ([self.Coder recycle_buffer:self->transport]) {
				// recv SMB2 PacketHeader
				cnt = [tcp recv:(char *)self.Coder.Data Lenght:self->transport mSec:msec];
				if (cnt == self->transport) {
					self->_packhead = (pPacketHeader)self.Coder.Data;	//for xcode debugger
					[self.Coder rewind];
					[self.Coder consume:cnt];
					[self.Coder moveptrto:PACKETHEADERSIZE];
					//confirm protocol id ('¥xfe','S','M','B')
					if (memcmp(SMB1PROTOCOLID, self.Coder.Data, 4) == 0 ||
						memcmp(SMB2PROTOCOLID, self.Coder.Data, 4) == 0) {
						return [self analyzePacket];
					}
				}
				[self.Coder freeptr];
			}
		}
	}
	[tcp close];
	return false;
}

-(BOOL)analyzePacket {
	return true;
}
/*----------------------------------------*
 *	send packet to TCP
 *----------------------------------------*/
-(BOOL)sendRequest:(TcpClient *)tcp {
	return [self sendRequest:tcp mSec:3000];
}
-(BOOL)sendRequest:(TcpClient *)tcp mSec:(int)msec {
	if (self.Coder.Data != nil && self.Coder.Length < 0x1000000) {
		self->transport = self.Coder.Length;
		//[self ShowPacker];
		UInt8 buff[4];
		[SMBUtils Int2ByteBE:self->transport :buff];
		int cnt = [tcp send:(char *)buff Lenght:4 mSec:msec];
		if (cnt == 4) {
			cnt = [tcp send:(char *)self.Coder.Data Lenght:self->transport mSec:msec];
			if (cnt == self->transport)
				return true;
		}
	}
	[tcp close];
	return false;
}
@end
