//
//  SMB2Messages.h
//
//  Copyright (C) 2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2016/03/13	H.Kanemoto
//	Modified:	2017/05/24	H.Kanemoto  SMB Signature
//
#ifndef SMB2Messages_h
#define SMB2Messages_h

typedef struct  __attribute__((packed)) {
	UInt32	Reserved;
	UInt32	TreeId;
}	SyncPack;

typedef struct  __attribute__((packed)) {
	UInt8	ProtocolId[4];
	int16_t	StructureSize;
	int16_t	CreditCharge;
	UInt32	Status;
	int16_t	Command;
	int16_t	CreditRequest;
	UInt32	Flags;
	UInt32	NextCommand;
	int64_t	MessageId;
	union {
		UInt64		AsyncId;
		SyncPack	Sync;
	} s;
	UInt64	SessionId;
	UInt8	Signature[16];
	UInt8	Payload[];
} PacketHeader, *pPacketHeader;

typedef struct  __attribute__((packed)) {
	int16_t	StructureSize;
	int8_t	ErrorContextCount;
	UInt8	Reserved;
	int32_t	ByteCount;
	UInt8	ErrorData[1];
}	resERROR;

typedef struct __attribute__((packed)) {
	int16_t	StructureSize;
	int16_t	DialectCount;
	int16_t	SecurityMode;
	UInt16	Reserved;
	UInt32	Capabilities;
	UInt8	ClientGuid[16];
	UInt8	ClientStartTime[8];
	UInt16	Dialects[];
}	reqNEGOTIATE;

typedef struct __attribute__((packed)) {
	int16_t	StructureSize;
	UInt16	SecurityMode;
	UInt16	DilectRevision;
	int16_t	NegotiateContextCount;
	UInt8	ServerGuid[16];
	UInt32	Capabilities;
	UInt32	MaxTransactSize;
	UInt32	MaxReadSize;
	UInt32	MaxWriteSize;
	UInt64	SystemTime;
	UInt64	ServerStartTime;
	UInt16	SecurityBufferOffset;
	int16_t	SecurityBufferLength;
	UInt32	NegotiateContextOffset;
	UInt8	Buffer[1];
}	resNEGOTIATE;

typedef struct __attribute__((packed)) {
	int16_t	StructureSize;
	UInt8	Flags;
	UInt8	SecurityMode;
	UInt32	Capabilities;
	UInt32	Channel;
	UInt16	SecurityBufferOffset;
	int16_t	SecurityBufferLength;
	UInt64	PreviousSessionId;
	UInt8	Buffer[1];
}	reqSESSION_SETUP;

typedef struct __attribute__((packed)) {
	int16_t	StructureSize;
	UInt16	SessionFlags;
	UInt16	SecurityBufferOffset;
	int16_t	SecurityBufferLength;
	UInt8	Buffer[1];
}	resSESSION_SETUP;

typedef struct __attribute__((packed)) {
	int16_t	StructureSize;
	UInt16	Flags;
	UInt16	PathOffset;
	int16_t	PathLength;
	UInt8	Buffer[1];
}	reqTREE_CONNECT;

typedef struct __attribute__((packed)) {
	int16_t	StructureSize;
	UInt8	ShareType;
	UInt8	Reserved;
	UInt32	ShareFlags;
	UInt32	Capabilities;
	int32_t	MaximalAccess;
}	resTREE_CONNECT;

typedef struct __attribute__((packed)) {
	int16_t	StructureSize;
	UInt16	Reserved;
}	reqTREE_DISCONNECT;

typedef struct __attribute__((packed)) {
	int16_t	StructureSize;
	UInt16	Reserved;
}	resTREE_DISCONNECT;

typedef struct __attribute__((packed)) {
	int16_t	StructureSize;
	UInt8	SecurityFlags;
	UInt8	RequestedOplockLevel;
	UInt32	ImpersonationLevel;
	UInt64	SmbCreateFlags;
	UInt64	Reserved;
	UInt32	DesiredAccess;
	UInt32	FileAttributes;
	UInt32	ShareAccess;
	UInt32	CreateDisposition;
	UInt32	CreateOptions;
	UInt16	NameOffset;
	int16_t	NameLength;
	UInt32	CreateContextOffset;
	int32_t	CreateContextLength;
	UInt8	Buffer[1];
}	reqCREATE;

typedef struct __attribute__((packed)) {
	int16_t	StructureSize;
	UInt8	OplockLevel;
	UInt8	Flags;
	UInt32	CreateAction;
	UInt64	CreationTime;
	UInt64	LastAccessTime;
	UInt64	LastWriteTime;
	UInt64	ChangeTime;
	UInt64	AllocationSize;
	UInt64	EndofFile;
	UInt32	FileAttributes;
	UInt32	Reserved2;
	UInt8	FileId[16];
	UInt32	CreateContextOffset;
	int32_t	CreateContextLength;
	UInt8	Buffer[1];
}	resCREATE;

typedef struct __attribute__((packed)) {
	UInt32	Next;
	UInt16	NameOffset;
	int16_t	NameLength;
	UInt16	Reserved;
	UInt16	DataOffset;
	int32_t	DataLength;
	//UInt8	Buffer[];
}	SMB2_CREATE_CONTEXT;

typedef struct __attribute__((packed)) {
	int16_t	StructureSize;
	UInt16	Flags;
	UInt32	Reserved;
	UInt8	FileId[16];
}	reqCLOSE;

typedef struct __attribute__((packed)) {
	int16_t	StructureSize;
	UInt16	Flags;
	UInt32	Reserved;
	UInt64	CreationTime;
	UInt64	LastAccessTime;
	UInt64	LastWriteTime;
	UInt64	ChangeTime;
	UInt64	AllocationSize;
	UInt64	EndofFile;
	UInt32	FileAttributes;
}	resCLOSE;

typedef struct __attribute__((packed)) {
	int16_t	StructureSize;
	UInt16	Reserved;
	UInt32	CtlCode;
	UInt8	FileId[16];
	UInt32	InputOffset;
	int32_t	InputCount;
	int32_t	MaxInputResponse;
	UInt32	OutputOffset;
	int32_t	OutputCount;
	int32_t	MaxOutputResponse;
	UInt32	Flags;
	UInt32	Reserved2;
	UInt8	Buffer[1];
}	reqIOCTL;

typedef struct __attribute__((packed)) {
	int16_t	StructureSize;
	UInt16	Reserved;
	UInt32	CtlCode;
	UInt8	FileId[16];
	UInt32	InputOffset;
	int32_t	InputCount;
	UInt32	OutputOffset;
	int32_t	OutputCount;
	UInt32	Flags;
	UInt32	Reserved2;
	UInt8	Buffer[1];
}	resIOCTL;

typedef struct __attribute__((packed)) {
	int16_t	StructureSize;
	UInt8	Padding;
	UInt8	Flags;
	int32_t	Length;
	UInt64	Offset;
	UInt8	FileId[16];
	int32_t	MinimumCount;
	UInt32	Channel;
	int32_t	RemainingBytes;
	UInt16	ReadChannelInfoOffset;
	int16_t	ReadChannelInfoLength;
	UInt8	Buffer[1];
}	reqREAD;

typedef struct __attribute__((packed)) {
	int16_t	StructureSize;
	UInt8	DataOffset;
	UInt8	Reserved;
	int32_t	DataLength;
	int32_t	DataRemaining;
	UInt32	Reserved2;
	UInt8	Buffer[1];
}	resREAD;

typedef struct __attribute__((packed)) {
	int16_t	StructureSize;
	UInt8	FileInformationClass;
	UInt8	Flags;
	UInt32	FileIndex;
	UInt8	FileId[16];
	UInt16	FileNameOffset;
	int16_t	FileNameLength;
	int32_t	OutputBufferLength;
	UInt8	Buffer[1];
}	reqQUERY_DIRECTORY;

typedef struct __attribute__((packed)) {
	int16_t	StructureSize;
	int16_t	OutputBufferOffset;
	int32_t	OutputBufferLength;
	UInt8	Buffer[1];
}	resQUERY_DIRECTORY;

typedef struct __attribute__((packed)) {
	int16_t	StructureSize;
	UInt8	InfoType;
	UInt8	FileInfoClass;
	int32_t	OutputBufferLength;
	UInt16	InputBufferOffset;
	UInt16	Reserved;
	int32_t	InputBufferLength;
	UInt32	AdditionalInformation;
	UInt32	Flags;
	UInt8	FileId[16];
	UInt8	Buffer[1];
}	reqQUERY_INFO;

typedef struct __attribute__((packed)) {
	int16_t	StructureSize;
	int16_t	OutputBufferOffset;
	int32_t	OutputBufferLength;
	UInt8	Buffer[1];
}	resQUERY_INFO;

typedef struct __attribute__((packed)) {
	int16_t	StructureSize;
	int16_t	Reserved;
}	reqECHO;

typedef struct __attribute__((packed)) {
	int16_t	StructureSize;
	int16_t	Reserved;
}	resECHO;

#define PACKETHEADERSIZE		(64)
#define NEGOTIATEREQSIZE		(36)
#define NEGOTIATERESSIZE		(65)
#define SESSIONSETUPREQSIZE		(25)
#define SESSIONSETUPRESSIZE		(9)
#define TREECONNECTREQSIZE		(9)
#define TREECONNECTRESSIZE		(16)
#define TREEDISCONNECTREQSIZE	(4)
#define TREEDISCONNECTRESSIZE	(4)
#define CREATEREQSIZE			(57)
#define CREATERESSIZE			(89)
#define CLOSEREQSIZE			(24)
#define CLOSERESSIZE			(60)
#define IOCTLREQSIZE			(57)
#define IOCTLRESSIZE			(49)
#define QUERYDIRREQSIZE			(33)
#define QUERYDIRRESSIZE			(9)
#define QUERYINFOREQSIZE		(41)
#define QUERYINFORESSIZE		(9)
#define READREQSIZE				(49)
#define READRESSIZE				(17)
#define ECHOREQSIZE				(4)
#define ECHORESSIZE				(4)

#define SMB2_FLAGS_SERVER_TO_REDIR		(1U)
#define SMB2_FLAGS_ASYNC_COMMAND		(2U)
#define SMB2_FLAGS_RELATED_OERATIONS	(4U)
#define SMB2_FLAGS_SIGNED				(8U)
#define SMB2_FLAGS_PRIORITY_MASK		(0x70U)
#define SMB2_FLAGS_DFS_OPERATIONS		(0x10000000U)
#define SMB2_FLAGS_REPLAY_OPERATION		(0x20000000U)

#define SMB2_COMMAND_NEGOTIATE			(0)
#define SMB2_COMMAND_SESSION_SETUP		(1)
#define SMB2_COMMAND_LOGOFF				(2)
#define SMB2_COMMAND_TREE_CONNECT		(3)
#define SMB2_COMMAND_TREE_DISCONNECT	(4)
#define SMB2_COMMAND_CREATE				(5)
#define SMB2_COMMAND_CLOSE				(6)
#define SMB2_COMMAND_FLUSH				(7)
#define SMB2_COMMAND_READ				(8)
#define SMB2_COMMAND_WRITE				(9)
#define SMB2_COMMAND_LOCK				(10)
#define SMB2_COMMAND_IOCTL				(11)
#define SMB2_COMMAND_CANCEL				(12)
#define SMB2_COMMAND_ECHO				(13)
#define SMB2_COMMAND_QUERY_DIRECTORY	(14)
#define SMB2_COMMAND_CHANGE_NOTIFY		(15)
#define SMB2_COMMAND_QUERY_INFO			(16)
#define SMB2_COMMAND_SET_INFO			(17)
#define SMB2_OPLOCK_BREAK				(18)

#define SMB2_GLOBAL_CAP_DFS					(1U)
#define SMB2_GLOBAL_CAP_LEASING				(2U)
#define SMB2_GLOBAL_CAP_LARGE_MTU			(4U)
#define SMB2_GLOBAL_CAP_MULTI_CHANNEL		(8U)
#define SMB2_GLOBAL_CAP_PERSISTENT_HANDLES	(0x10U)
#define SMB2_GLOBAL_CAP_DIRECTORY_LEASING	(0x20U)
#define	SMB2_GLOBAL_CAP_ENCRYPTION			(0x40U)

typedef enum {
	FPPAM_FILE_READ_DATA =			0x00000001U,
	FPPAM_FILE_WRITE_DATA =			0x00000002U,
	FPPAM_FILE_APPEND_DATA =		0x00000004U,
	FPPAM_FILE_READ_EA =			0x00000008U,
	FPPAM_FILE_WRITE_EA =			0x00000010U,
	FPPAM_FILE_DELETE_CHILD =		0x00000040U,
	FPPAM_FILE_EXECUTE =			0x00000020U,
	FPPAM_FILE_READ_ATTRIBUTES =	0x00000080U,
	FPPAM_FILE_WRITE_ATTRIBUTES =	0x00000100U,
	FPPAM_DELETE =					0x00010000U,
	FPPAM_READ_CONTROL =			0x00020000U,
	FPPAM_WRITE_DAC =				0x00040000U,
	FPPAM_WRITE_OWNER =				0x00080000U,
	FPPAM_SYNCHRONIZE =				0x00100000U,
	FPPAM_ACCESS_SYSTEM_SECURITY =	0x01000000U,
	FPPAM_MAXIMUM_ALLOWED =			0x02000000U,
	FPPAM_GENERIC_ALL =				0x10000000U,
	FPPAM_GENERIC_EXECUTE =			0x20000000U,
	FPPAM_GENERIC_WRITE =			0x40000000U,
	FPPAM_GENERIC_READ =			0x80000000U,
}	File_Pipe_Printer_Access_Mask;

typedef enum {
	DAM_FILE_LIST_DIRECTORY =		0x00000001U,
	DAM_FILE_ADD_FILE =				0x00000002U,
	DAM_FILE_ADD_SUBDIRECTORY =		0x00000004U,
	DAM_FILE_READ_EA =				0x00000008U,
	DAM_FILE_WRITE_EA =				0x00000010U,
	DAM_FILE_TRAVERSE =				0x00000020U,
	DAM_FILE_DELETE_CHILD =			0x00000040U,
	DAM_FILE_READ_ATTRIBUTES =		0x00000080U,
	DAM_FILE_WRITE_ATTRIBUTES =		0x00000100U,
	DAM_DELETE =					0x00010000U,
	DAM_READ_CONTROL =				0x00020000U,
	DAM_WRITE_DAC =					0x00040000U,
	DAM_WRITE_OWNER =				0x00080000U,
	DAM_SYNCHRONIZE =				0x00100000U,
	DAM_ACCESS_SYSTEM_SECURITY =	0x01000000U,
	DAM_MAXIMUM_ALLOWED =			0x02000000U,
	DAM_GENERIC_ALL =				0x10000000U,
	DAM_GENERIC_EXECUTE =			0x20000000U,
	DAM_GENERIC_WRITE =				0x40000000U,
	DAM_GENERIC_READ =				0x80000000U,
}	Directory_Access_Mask;

typedef enum {
	FILE_SHARE_READ =	0x00000001U,
	FILE_SHARE_WRITE =	0x00000002U,
	FILE_SHARE_DELETE =	0x00000004U,
}	ShareAccess;

typedef enum {
	CD_FILE_SUPERSEDE =		0x00000000U,
	CD_FILE_OPEN =			0x00000001U,
	CD_FILE_CREATE =		0x00000002U,
	CD_FILE_OPEN_IF =		0x00000003U,
	CD_FILE_OVERWRITE =		0x00000004U,
	CD_FILE_OVERWRITE_IF =	0x00000005U,
}	CreateDisposition;

typedef enum {
	CO_FILE_DIRECTORY_FILE =				0x00000001U,
	CO_FILE_WRITE_THROUGH =					0x00000002U,
	CO_FILE_SEQUENTIAL_ONLY =				0x00000004U,
	CO_FILE_NO_INTERMEDIATE_BUFFERING =		0x00000008U,
	CO_FILE_SYNCHRONOUS_IO_ALERT =			0x00000010U,
	CO_FILE_SYNCHRONOUS_IO_NONALERT =		0x00000020U,
	CO_FILE_NON_DIRECTORY_FILE =			0x00000040U,
	CO_FILE_COMPLETE_IF_OPLOCKED =			0x00000100U,
	CO_FILE_NO_EA_KNOWLEDGE =				0x00000200U,
	CO_FILE_RANDOM_ACCESS =					0x00000800U,
	CO_FILE_DELETE_ON_CLOSE =				0x00001000U,
	CO_FILE_OPEN_BY_FILE_ID =				0x00002000U,
	CO_FILE_OPEN_FOR_BACKUP_INTENT =		0x00004000U,
	CO_FILE_NO_COMPRESSION =				0x00008000U,
	CO_FILE_OPEN_REMOTE_INSTANCE =			0x00000400U,
	CO_FILE_OPEN_REQUIRING_OPLOCK =			0x00010000U,
	CO_FILE_DISALLOW_EXCLUSIVE =			0x00020000U,
	CO_FILE_RESERVE_OPFILTER =				0x00100000U,
	CO_FILE_OPEN_REPARSE_POINT =			0x00200000U,
	CO_FILE_OPEN_NO_RECALL =				0x00400000U,
	CO_FILE_OPEN_FOR_FREE_SPACE_QUERY =		0x00800000U,
}	CreateOptions;

typedef enum {
	FSCTL_DFS_GET_REFERRALS =				0x00060194U,
	FSCTL_PIPE_PEEK =						0x0011400CU,
	FSCTL_PIPE_WAIT =						0x00110018U,
	FSCTL_PIPE_TRANSCEIVE =					0x0011C017U,
	FSCTL_SRV_COPYCHUNK =					0x001440F2U,
	FSCTL_SRV_ENUMERATE_SNAPSHOTS =			0x00144064U,
	FSCTL_SRV_REQUEST_RESUME_KEY =			0x00140078U,
	FSCTL_SRV_READ_HASH =					0x001441bbU,
	FSCTL_SRV_COPYCHUNK_WRITE =				0x001480F2U,
	FSCTL_LMR_REQUEST_RESILIENCY =			0x001401D4U,
	FSCTL_QUERY_NETWORK_INTERFACE_INFO =	0x001401FCU,
	FSCTL_SET_REPARSE_POINT =				0x000900A4U,
	FSCTL_DFS_GET_REFERRALS_EX =			0x000601B0U,
	FSCTL_FILE_LEVEL_TRIM =					0x00098208U,
	FSCTL_VALIDATE_NEGOTIATE_INFO =			0x00140204U,
}	CtlCode;

typedef enum {
	FIC_FileDirectoryInformation =			0x01U,
	FIC_FileFullDirectoryInformation =		0x02U,
	FIC_FileIdFullDirectoryInformation =	0x26U,
	FIC_FileBothDirectoryInformation =		0x03U,
	FIC_FileIdBothDirectoryInformation =	0x25U,
	FIC_FileNamesInformation =				0x0CU,
	FIC_FileAllInformation =				18U,
	FIC_FileStandardInformation =			5U,
}	FileInformationClass;

#endif /* SMB2Messages_h */
