//
//  SMB2Negotiate.m
//
//  Copyright (C) 2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2016/03/13	H.Kanemoto
//	Modified:
//
#import <Foundation/Foundation.h>
#import "SMB2Negotiate.h"

static const UInt8 SMB1NEGOTIATE[] = {
	//0x00,0x00,0x00,0x39,	Transport=57bytes
	//SMB1 Header
	0xff,0x53,0x4d,0x42,	//¥xff,'S','M','B'
	0x72,					//Command=Negotiate
	0x00,0x00,0x00,0x00,	//Status
	0x08,					//Flags = Case Sensitivity
	0x01,0xc0,				//Flags2 = UNICODE+NTStatus+LONG_NAME
	0x00,0x00,				//Process ID (HI)
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,	//Signature
	0x00,0x00,				//Reserved
	0xff,0xff,				//TreeID
	0x01,0x00,				//Process ID
	0xff,0xff,				//User ID
	0x00,0x00,				//Multiplex ID
	//Negotiate Command
	0x00,					//Word Count
	0x16,0x00,				//Byte Count
	0x02,					//Buffer Format = Dialect
	0x53,0x4d,0x42,0x20,0x32,0x2e,0x30,0x30,0x32,0x00,	//SMB 2.002
	0x02,					//Buffer Format = Dialect
	0x53,0x4d,0x42,0x20,0x32,0x2e,0x3f,0x3f,0x3f,0x00	//SMB 2.???
};
/*----------------------------------------------------------------------*
 *	SMB1 NEGOTIATE REQUEST
 *----------------------------------------------------------------------*/
@implementation SMB1ReqNegotiate

-(id)init {
	if (self = [super init]) {
		self->transport = 57;
		if ([self.Coder alloc_buffer:57]) {
			[self.Coder push:SMB1NEGOTIATE Length:57];
		} else
			return nil;
	}
	return self;
}

@end
/*----------------------------------------------------------------------*
 *	SMB2 NEGOTIATE REQUEST
 *----------------------------------------------------------------------*/
@implementation SMB2ReqNegotiate

-(reqNEGOTIATE *)getNegotiate {
	return (reqNEGOTIATE *) self.Header->Payload;
}

-(id)initWith:(UInt8 *)uuid {
	if (self = [super init]) {
		if ([self.Coder alloc_buffer:PACKETHEADERSIZE+NEGOTIATEREQSIZE+2*2]) {
			[super initheader:self.Header];
			self.Header->Command = SMB2_COMMAND_NEGOTIATE;
			self.NEGOTIATE->StructureSize = NEGOTIATEREQSIZE;
			self.NEGOTIATE->SecurityMode = 1;		//Signing Enabled
			//DFS+LEASING+LARGE_MTU+PERSISTENT_HANDLES+DIRECTORY_LEASING+ENCRYPTION
			self.NEGOTIATE->Capabilities = 0x77;
			self.NEGOTIATE->DialectCount = 2;
			self.NEGOTIATE->Dialects[0] = 0x0202;	//Vista
			self.NEGOTIATE->Dialects[1] = 0x0210;	//Win7
			memcpy(self.NEGOTIATE->ClientGuid, uuid, 16);
			UInt8 stamp[8];
			[SMBUtils SMBTimeStamp:stamp];
			memcpy(self.NEGOTIATE->ClientStartTime, stamp, 8);
			[self.Coder consume:PACKETHEADERSIZE+NEGOTIATEREQSIZE+2*2];
		} else {
			//[self release];
			return nil;
		}
	}
	return self;
}

@end
/*----------------------------------------------------------------------*
 *	SMB2 NEGOTIATE RESPONSE
 *----------------------------------------------------------------------*/
@implementation SMB2ResNegotiate

-(resNEGOTIATE *)getNegotiate {
	return (resNEGOTIATE *)self.Header->Payload;
}

@end
