//
//  SMB2SessionSetup.m
//
//  Copyright (C) 2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2016/03/13	H.Kanemoto
//	Modified:
//
#import <Foundation/Foundation.h>
#import "SMB2SessionSetup.h"
#import "DER.h"
/*----------------------------------------------------------------------*
 *	SMB2 SESSION SETUP REQUEST
 *----------------------------------------------------------------------*/
@implementation SMB2ReqSessionSetup

-(id)init {
	if (self = [super init]) {
		if ([self.Coder alloc_buffer:PACKETHEADERSIZE+SESSIONSETUPREQSIZE+1024]) { //1Kbyte for payload
			[super initheader:self.Header];
			self.Header->Command = SMB2_COMMAND_SESSION_SETUP;
			self.SESSIONSETUP->StructureSize = SESSIONSETUPREQSIZE;
			self.SESSIONSETUP->Capabilities = SMB2_GLOBAL_CAP_DFS;
			self.SESSIONSETUP->SecurityMode = 1; //ENABLED
			self.SESSIONSETUP->SecurityBufferOffset = 0x58;
			//move pointer to begining of payload include dummy buffer[1]
			[self.Coder consume:PACKETHEADERSIZE+SESSIONSETUPREQSIZE-1];
		} else {
			//[self release];
			return nil;
		}
	}
	return self;
}

-(reqSESSION_SETUP *)getSessionSetup {
	return (reqSESSION_SETUP*) &self.Coder.Data[PACKETHEADERSIZE];
}
@end
/*----------------------------------------------------------------------*
 *	SMB2 SESSION SETUP REQUEST
 *----------------------------------------------------------------------*/
@implementation SMB2ResSessionSetup

-(id)init {
	if (self = [super init]) {
		if ([self.Coder alloc_buffer:PACKETHEADERSIZE+SESSIONSETUPRESSIZE+512]) { //512byte for payload
			[super initheader:self.Header];
			self.Header->Command = SMB2_COMMAND_SESSION_SETUP;
			self.SESSIONSETUP->StructureSize = SESSIONSETUPRESSIZE;
		} else
			return nil;
	}
	return self;
}

-(resSESSION_SETUP *)getSessionSetup {
	return (resSESSION_SETUP *) self.Header->Payload;
}
-(UInt8 *)getPayloadData {
	return &self.SESSIONSETUP->Buffer[0];
}

-(BOOL)recvResponse:(TcpClient*)tcp {
	if ([super recvResponse:tcp]) {
		//move pointer to payload top but except for dummy buffer[1]
		[self.Coder moveptrto:PACKETHEADERSIZE+SESSIONSETUPRESSIZE-1];
		return true;
	}
	return false;
}

-(NtLmType2*)findType2 {
	//move pointer to DER Tree top
	[self.Coder moveptrto:self.SESSIONSETUP->SecurityBufferOffset];
	if (self.Coder.UnreadCnt >= self.SESSIONSETUP->SecurityBufferLength) {
		bytecoder *payload = [self.Coder cutout:self.SESSIONSETUP->SecurityBufferOffset
										 Length:self.SESSIONSETUP->SecurityBufferLength];
		if (payload == nil)
			return nil;
		
		//parse DER tree
		DER_PARSE_RESULT *r = [[DER_PARSE_RESULT alloc] initWith:payload];
		if (![DER parse:r])
			return nil;
		
		//find TYPE2
		NSPredicate *pre1 = [NSPredicate predicateWithFormat:@"Tag = 0xa1"];
		NSArray<DER *> *find = [r.Items filteredArrayUsingPredicate:pre1];
		if ([find count] < 1)
			return nil;
		
		DER *a1 = [find firstObject];
		NSPredicate *pre2 = [NSPredicate predicateWithFormat:@"Tag = 0x30"];
		NSArray<DER *> *find2 = [a1.SubItems filteredArrayUsingPredicate:pre2];
		if ([find2 count] < 1)
			return nil;
		
		DER *seq = [find2 firstObject];
		NSPredicate *pre3 = [NSPredicate predicateWithFormat:@"Tag = 0xa2"];
		NSArray<DER *> *find3 = [seq.SubItems filteredArrayUsingPredicate:pre3];
		if ([find3 count] < 1)
			return nil;
		
		DER *a2 = [find3 firstObject];
		NSPredicate *pre4 = [NSPredicate predicateWithFormat:@"Tag = 0x04"];
		NSArray<DER *> *find4 = [a2.SubItems filteredArrayUsingPredicate:pre4];
		if ([find4 count] < 1)
			return nil;
		
		//deserialize Type2 message from DER contents
		DER *t2 = [find4 firstObject];
		NtLmType2 *type2 = [[NtLmType2 alloc] initWith:t2.detach];
		if ([type2 deserialize])
			return type2;
	}
	return nil;
}

@end
