//
//  SMB2TreeConnect.m
//
//  Copyright (C) 2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2016/03/17	H.Kanemoto
//	Modified:
//
#import <Foundation/Foundation.h>
#import "SMB2TreeConnect.h"
/*----------------------------------------------------------------------*
 *	SMB2 TREE CONNECT REQUEST
 *----------------------------------------------------------------------*/
@implementation SMB2ReqTreeConnect

-(id)initWith:(NSString*)Path {
	if (self = [super init]) {
		BOOL initialize = [self.Coder alloc_buffer:512];	//512byte for payload
		if (initialize) {
			[super initheader:self.Header];
			self.Header->Command = SMB2_COMMAND_TREE_CONNECT;
			self.TREECONNECT->StructureSize = TREECONNECTREQSIZE;
			self.TREECONNECT->PathOffset = 0x48;
			//move pointer to payload top but except for dummy buffer[1]
			[self.Coder consume:PACKETHEADERSIZE+TREECONNECTREQSIZE-1];
			//set path
			UCS2 *dir = [UCS2 create:Path];
			if (dir != nil) {
				self.TREECONNECT->PathLength = dir.ByteLen;
				initialize = [self.Coder push:dir.Ptr Length:dir.ByteLen];
			} else
				initialize = false;
		}
		if (!initialize)
			return nil;
	}
	return self;
}

-(reqTREE_CONNECT *)getTreeConn {
	return (reqTREE_CONNECT*) &self.Coder.Data[PACKETHEADERSIZE];
}
@end
/*----------------------------------------------------------------------*
 *	SMB2 TREE CONNECT RESPONSE
 *----------------------------------------------------------------------*/
@implementation SMB2ResTreeConnect

-(id)init {
	if (self = [super init]) {
		if ([self.Coder alloc_buffer:PACKETHEADERSIZE+TREECONNECTRESSIZE]) {
			[super initheader:self.Header];
			self.Header->Command = SMB2_COMMAND_TREE_CONNECT;
			self.TREECONNECT->StructureSize = TREECONNECTRESSIZE;
		} else
			return nil;
	}
	return self;
}

-(resTREE_CONNECT *)getTreeConn {
	return (resTREE_CONNECT*) &self.Coder.Data[PACKETHEADERSIZE];
}
@end
/*----------------------------------------------------------------------*
 *	SMB2 TREE DISCONNECT REQUEST
 *----------------------------------------------------------------------*/
@implementation SMB2ReqTreeDisconnect

-(id)init {
	if (self = [super init]) {
		if ([self.Coder alloc_buffer:PACKETHEADERSIZE+TREEDISCONNECTREQSIZE]) {
			[super initheader:self.Header];
			self.Header->Command = SMB2_COMMAND_TREE_DISCONNECT;
			self.TREEDISCONNECT->StructureSize = TREEDISCONNECTREQSIZE;
			//move pointer to payload top but except for dummy buffer[1]
			[self.Coder consume:PACKETHEADERSIZE+TREEDISCONNECTREQSIZE];
		} else
			return nil;
	}
	return self;
}

-(reqTREE_DISCONNECT *)getTreeDisConn {
	return (reqTREE_DISCONNECT*) &self.Coder.Data[PACKETHEADERSIZE];
}
@end
/*----------------------------------------------------------------------*
 *	SMB2 TREE DISCONNECT RESPONSE
 *----------------------------------------------------------------------*/
@implementation SMB2ResTreeDisconnect

-(id)init {
	if (self = [super init]) {
		if ([self.Coder alloc_buffer:PACKETHEADERSIZE+TREEDISCONNECTRESSIZE]) {
			[super initheader:self.Header];
			self.Header->Command = SMB2_COMMAND_TREE_DISCONNECT;
			self.TREEDISCONNECT->StructureSize = TREEDISCONNECTRESSIZE;
		} else
			return nil;
	}
	return self;
}

-(resTREE_DISCONNECT *)getTreeDisConn {
	return (resTREE_DISCONNECT*) &self.Coder.Data[PACKETHEADERSIZE];
}
@end
