//
//  SMBConnection.m
//
//  Copyright (C) 2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2016/03/15	H.Kanemoto
//	Modified:
//
#import <Foundation/Foundation.h>
#import "SMBConnection.h"
#import "SMBUtils.h"
/*----------------------------------------------------------------------
 *	SMB CONNECTION
 *----------------------------------------------------------------------*/
@implementation SMBConnection
/*----------------------------------------*
 *	Init/Dealloc
 *----------------------------------------*/
-(id)initWith:(NSString *)_ip Port:(int32_t)_port {
	if (self = [super init]) {
		self->tcp = nil;
		self->ip = _ip;
		self->port = _port;
		[SMBUtils GenUUID:self->clientguid];
		self->support_leasing = false;
		self->support_large_mtu = false;
		self->support_dir_leasing = false;
		self->support_multi_ch = false;
		self->support_persistent_hdl = false;
		self->support_encryption = false;
	}
	return self;
}

-(void)dealloc {
	[self Disconnect];
	DLog(@"");
}
/*----------------------------------------*
 *	Properties
 *----------------------------------------*/
-(TcpClient *)getTcp {
	return self->tcp;
}
-(NSString*)getIP {
	return self->ip;
}
-(int32_t)getPort {
	return self->port;
}
-(BOOL)getSupportMultiCredit {
	return self->support_large_mtu;
}
-(BOOL)getSupportLeasing {
	return self->support_leasing;
}
-(BOOL)getSupportDirLeaseing {
	return self->support_dir_leasing;
}
-(BOOL)getSupportMultiChannel {
	return self->support_multi_ch;
}
-(BOOL)getSupportPersistentHandle {
	return self->support_persistent_hdl;
}
-(BOOL)getSupportEncryption {
	return self->support_encryption;
}
-(UInt32)getMaxTransactSize {
	return self->max_transactsize;
}
-(UInt32)getMaxReadSize {
	return self->max_readsize;
}
-(UInt32)getMaxWriteSize {
	return self->max_writesize;
}
-(UInt16)getDialect {
	return self->dialect;
}
/*----------------------------------------*
 *	TCP connect
 *----------------------------------------*/
-(BOOL)Connect {
	if ([self TcpReconnect]) {
		self->msgid = 1;
		return true;
	}
	return false;
}

-(BOOL)TcpReconnect {
	[self Disconnect];
	self->tcp = [[TcpClient alloc] init];
	if (self.Tcp != nil) {
		if ([self.Tcp connect:self.IP Port:self.Port]) {
			return true;
		}
	}
	return false;
}

-(void)Disconnect {
	if (self->tcp != nil) {
		[self->tcp close];
		self->tcp = nil;
	}
}
/*----------------------------------------*
 *	issue MessageId
 *----------------------------------------*/
-(UInt64)SequenceWindow:(int16_t)credit {
	UInt64 newmsgid = self->msgid;
	if (self.SupportMultiCredit) {
		self->msgid += credit;
	} else {
		++self->msgid;
	}
	return newmsgid;
}
/*----------------------------------------*
 *	SMB Negotiate
 *----------------------------------------*/
-(BOOL)Negotiate {
	while ([self Connect]) {
		//send SMB1 Negotiate first
		SMB1ReqNegotiate* req1 = [[SMB1ReqNegotiate alloc] init];
		if (req1 == nil || ![req1 sendRequest:self.Tcp])
			break;
			
		SMB2ResNegotiate* res1 = [[SMB2ResNegotiate alloc] init];
		if (res1 == nil || ![res1 recvResponse:self.Tcp])
			break;
				
		//send SMB2 Negotiate request
		SMB2ReqNegotiate* req2 = [[SMB2ReqNegotiate alloc] initWith:self->clientguid];
		if (req2 == nil)
			break;
		req2.Header->Command = SMB2_COMMAND_NEGOTIATE;
		req2.Header->MessageId = [self SequenceWindow:1];
		if (![req2 sendRequest:self.Tcp])
			break;
					
		//recieve SMB2 Negotiate response
		SMB2ResNegotiate* res2 = [[SMB2ResNegotiate alloc] init];
		if (res2 == nil || ![res2 recvResponse:self.Tcp] || res2.Header->Status != 0)
			break;
		
		//save negotiated Status
		self->support_leasing = (res2.NEGOTIATE->Capabilities & SMB2_GLOBAL_CAP_LEASING);
		self->support_large_mtu = (res2.NEGOTIATE->Capabilities & SMB2_GLOBAL_CAP_LARGE_MTU);
		self->support_dir_leasing = (res2.NEGOTIATE->Capabilities & SMB2_GLOBAL_CAP_DIRECTORY_LEASING);
		self->support_multi_ch = (res2.NEGOTIATE->Capabilities & SMB2_GLOBAL_CAP_MULTI_CHANNEL);
		self->support_persistent_hdl = (res2.NEGOTIATE->Capabilities & SMB2_GLOBAL_CAP_PERSISTENT_HANDLES);
		self->support_encryption = (res2.NEGOTIATE->Capabilities & SMB2_GLOBAL_CAP_ENCRYPTION);
		self->max_transactsize = res2.NEGOTIATE->MaxTransactSize;
		self->max_readsize = res2.NEGOTIATE->MaxReadSize;
		self->max_writesize = res2.NEGOTIATE->MaxWriteSize;
		self->dialect = res2.NEGOTIATE->DilectRevision;
		memcpy(self->serverguid, res2.NEGOTIATE->ServerGuid, 16);
		return true;
	}
	[self Disconnect];
	return false;
}

@end
