//
//  SMBSession.h
//
//  Copyright (C) 2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2016/03/15	H.Kanemoto
//	Modified:	2017/05/24	H.Kanemoto  SMB Signature
//
#ifndef SMBSession_h
#define SMBSession_h
#import "SMBConnection.h"
#import "SMB2Negotiate.h"
#import "SMB2SessionSetup.h"
#import "NTLMSSP.h"
#import "DER.h"
#import "RPC.h"
@class SMBTreeConnect;
/*----------------------------------------------------------------------*
 *	SMB SESSION
 *----------------------------------------------------------------------*/
@interface SMBSession : TBObject {
	@protected
	SMBConnection*	conn;
	UInt64			sessionid;
	UInt64			lastsessonid;
	UInt8			reskeynt[16];
	UInt8			ntproof[16];
	UInt8			sesbasekey[16];
	UInt8			sessionkey[16];
	UInt8			encseskey[16];
	NSMutableArray<SMBTreeConnect*>	*shares;
}
@property (nonatomic, readonly, getter=getConnection) SMBConnection *Connection;
@property (nonatomic, copy) NSString *User;
@property (nonatomic, copy) NSString *Password;
@property (nonatomic, copy) NSString *Domain;
@property (nonatomic, copy) NSString *Machine;
@property (nonatomic, readonly, getter=getShares) NSMutableArray<SMBTreeConnect*> *Shares;
@property (nonatomic, readonly, assign, getter=getSessionId) UInt64 SessionId;
@property (nonatomic, readonly, assign, getter=getLastSessionId) UInt64 LastSessionId;
@property (nonatomic, readonly, assign, getter=getResponseKeyNT) UInt8 *ResponseKeyNT;
@property (nonatomic, readonly, assign, getter=getNTProofStr) UInt8 *NTProofStr;
@property (nonatomic, readonly, assign, getter=getSessionBaseKey) UInt8 *SessionBaseKey;
@property (nonatomic, readonly, assign, getter=getSessionKey) UInt8 *SessionKey;
@property (nonatomic, readonly, assign, getter=getEncrypedExportedSessionKey) UInt8 *EncrypedExportedSessionKey;
@property (nonatomic, readonly, assign, getter=getReadUnitSize) int64_t ReadUnitSize;
@property (nonatomic, assign) BOOL Sign;

-(id)initWith:(SMBConnection *)conn;
-(BOOL)MarkRequest:(SMB2Header *)req;
-(BOOL)Login:(NSString*)userdom Machine:(NSString*)workstatin User:(NSString*)user Password:(NSString*)pass;
-(BOOL)Login:(NSString*)workstatin User:(NSString*)user Password:(NSString*)pass;
-(BOOL)ProbeServer:(BOOL)relogin;
-(BOOL)Ping;

-(BOOL)sendRequest:(SMB2Header*)req mSec:(int)msec;
-(BOOL)sendRequest:(SMB2Header*)req;
@end

#endif /* SMBSession_h */
