//
//  SMBSession.m
//
//  Copyright (C) 2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2016/03/15	H.Kanemoto
//	Modified:	2017/05/24	H.Kanemoto  SMB Signature
//
#import <Foundation/Foundation.h>
#import "SMBSession.h"
#import "SMBTreeConnect.h"
#import "SMB2Create.h"
#import "SMB2Close.h"
#import "SMB2Ioctl.h"
#import "SMB2TreeConnect.h"
#import "SMB2Echo.h"
#import "RC4.h"
#import "RPC.h"
#import <sys/sysctl.h>
/*----------------------------------------------------------------------*
 *	SMB SESSION
 *----------------------------------------------------------------------*/
@implementation SMBSession
/*----------------------------------------*
 *	Init/Dealloc
 *----------------------------------------*/
-(id)initWith:(SMBConnection *)_conn {
	if (self = [super init]) {
		self->conn = _conn;
		self->shares = [[NSMutableArray<SMBTreeConnect*> alloc] init];
		self.Sign = NO;
	}
	return self;
}

-(void)dealloc {
	DLog(@"");
	[self->shares removeAllObjects];
}
/*----------------------------------------*
 *	Properties
 *----------------------------------------*/
-(SMBConnection*)getConnection {
	return self->conn;
}
-(UInt64)getSessionId {
	return self->sessionid;
}
-(UInt64)getLastSessionId {
	return self->lastsessonid;
}
-(UInt8*)getResponseKeyNT {
	return self->reskeynt;
}
-(UInt8*)getNTProofStr {
	return self->ntproof;
}
-(UInt8*)getSessionBaseKey {
	return self->sesbasekey;
}
-(UInt8*)getSessionKey {
	return self->sessionkey;
}
-(UInt8*)getEncrypedExportedSessionKey {
	return self->encseskey;
}
-(NSMutableArray<SMBTreeConnect*>*)getShares {
	return self->shares;
}
-(int64_t)getReadUnitSize {
	if (self.Connection.SupportMultiCredit)
		return self.Connection.MaxReadSize > 0x100000 ? 0x100000 : self.Connection.MaxReadSize;
	else
		return 0x10000;
}
/*----------------------------------------*
 *	Mark Request MessageId/SessionId
 *----------------------------------------*/
-(BOOL)MarkRequest:(SMB2Header *)req {
	if (req != nil) {
		req.Header->MessageId = [self.Connection SequenceWindow:req.Header->CreditCharge];
		req.Header->SessionId = self.SessionId;
		return true;
	}
	return false;
}
/*----------------------------------------*
 *	Login
 *----------------------------------------*/
-(BOOL)Login:(NSString *)userdom Machine:(NSString *)workstation User:(NSString *)user Password:(NSString *)pass {
	if ([self NTLMv2Auth:userdom Machine:workstation User:user Password:pass]) {
		if ([self NetShareEnumALl]) {
			self.Domain = userdom;
			self.Machine = workstation;
			self.User = user;
			self.Password = pass;
			return true;
		}
	}
	[self.Connection.Tcp close];
	return false;
}
-(BOOL)Login:(NSString *)workstation User:(NSString *)user Password:(NSString *)pass {
	char hostname[256];
	memset(hostname, 0, sizeof(hostname));
	gethostname(hostname, 255);
	//NSString *phoneName = [[UIDevice currentDevice] name];
	/*
	size_t size;
	sysctlbyname("hw.machine", NULL, &size, NULL, 0);
	char *machine = malloc(size);
	sysctlbyname("hw.machine", machine, &size, NULL, 0);
	NSString *platform = [NSString stringWithCString:machine encoding:NSUTF8StringEncoding];
	free(machine);
	*/
	
	
	return [self Login:workstation Machine:workstation User:user Password:pass];
}

-(BOOL)ProbeServer:(BOOL)relogin {
	//confirm connection status between client and server
	if ([self Ping])
		return true;
	if (relogin) {
		//try re-connect using last session id
		if ([self.Connection Negotiate]) {
			self->lastsessonid = self.SessionId;
			self->sessionid = 0;
			return [self NTLMv2Auth:self.Domain Machine:self.Machine User:self.User Password:self.Password];
		}
	}
	return false;
}
/*----------------------------------------*
 *	Authenticate
 *----------------------------------------*/
-(BOOL)NTLMv2Auth:(NSString*)userdom Machine:(NSString*)workstation User:(NSString*)user Password:(NSString*)pass {
	//NTLM Messages
	NtLmType1 *type1 = [[NtLmType1 alloc] init];	//NEGOTIATE_MESSAGE
	NtLmType2 *type2 = nil;							//CHALLENGE_MESSAGE
	NtLmType3 *type3 = [[NtLmType3 alloc] init];	//AUTHENTICATE_MESSAGE

	BOOL result = false;
	for (;;) {
		//1)send SESSION_SETUP Request
		SMB2ReqSessionSetup *req1 = [self reqNEGOTIATE_MESSAGE:type1];
		if (req1 == nil)
			break;
		req1.SESSIONSETUP->PreviousSessionId = self.LastSessionId;
		if (![req1 sendRequest:self.Connection.Tcp])
			break;
		
		//2)recieve SESSION_SETUP Response
		SMB2ResSessionSetup *res2 = [[SMB2ResSessionSetup alloc] init];
		if (res2 == nil || ![res2 recvResponse:self.Connection.Tcp])
			break;
		
		//NTSTATUS=STATUS_MORE_PROCESSING_REQUIRED
		if (res2.Header->Status != 0xc0000016)
			break;
		
		//find Type2 message from response payload (DER Tree)
		type2 = [res2 findType2];
		if (type2 == nil)
			break;
		
		//save SessionId
		self->sessionid = res2.Header->SessionId;
		self->lastsessonid = 0;
		//setup Type3 message
		if (![self setupType3:type1 :type2 :type3 :userdom :user :pass])
			break;
		
		//3)send SESSION_SETUP Request
		SMB2ReqSessionSetup *req3 = [self reqAUTHENTICATE_MESSAGE:type3];
		if (req3 == nil || ![req3 sendRequest:self.Connection.Tcp])
			break;
		
		//4)recieve Response
		SMB2ResSessionSetup *res4 = [[SMB2ResSessionSetup alloc] init];
		if (res4 != nil && [res4 recvResponse:self.Connection.Tcp] && res4.Header->Status == 0)
			result = true;
		break;
	}
	return result;
}

-(SMB2ReqSessionSetup*)reqNEGOTIATE_MESSAGE:(NtLmType1*)type1 {
	SMB2ReqSessionSetup *req = [[SMB2ReqSessionSetup alloc] init];
	if (req == nil)
		return nil;
	[self MarkRequest:req];
	
	//wrap Type1 around GSS
	DER *gss = [[DER alloc] initWith:0x60 :nil];
	[gss add:[DER OID_SPNEGO]];
	DER_SEQ_CONSTRUCTED *token = [[DER_SEQ_CONSTRUCTED alloc] initWith:0xa0 :nil];
	DER_SEQ_CONSTRUCTED *typlst = [[DER_SEQ_CONSTRUCTED alloc] initWith:0xa0 :nil];
	[typlst add:[DER OID_NTLM]];
	[token add:typlst];
	DER *mech = [[DER alloc] initWith:0xa2 :nil];
	DER *type = [[DER alloc] initWith:0x04 :type1.Coder];
	[mech add:type];
	[token add:mech];
	[gss add:token];
	
	//attach gss to request as payload
	int32_t len = [gss serialize:req.Coder];
	req.SESSIONSETUP->SecurityBufferLength = (int16_t)len;
	
	return req;
}

-(SMB2ReqSessionSetup*)reqAUTHENTICATE_MESSAGE:(NtLmType3*)type3 {
	SMB2ReqSessionSetup *req = [[SMB2ReqSessionSetup alloc] init];
	[self MarkRequest:req];
	
	//wrap Type3 around GSS
	DER *a1 = [[DER_SEQ_CONSTRUCTED alloc] initWith:0xa1 :nil];
	DER *el = [[DER alloc] initWith:0xa2 :nil];
	DER *type = [[DER alloc] initWith:0x04 :type3.Coder];
	[el add:type];
	[a1 add:el];
	
	//attach gss to request as payload
	int32_t len = [a1 serialize:req.Coder];
	req.SESSIONSETUP->SecurityBufferLength = (int16_t)len;
	
	return  req;
}

-(BOOL)setupType3:(NtLmType1*)type1 : (NtLmType2*)type2 : (NtLmType3*)type3 :
	(NSString*)domain : (NSString*)user : (NSString*)pass {
	
	//create NTLMv2 Response/NTLMv2 Client Challenge block
	NtLmResponse *ntlmres = [[NtLmResponse alloc] init];
	//copy TypeInfo from Type2
	[ntlmres.TargetInfo addObjectsFromArray:type2.TargetInfo];
	//enclode blob
	[ntlmres serialize];
	
	//concat server_challenge + blob
	bytecoder *cblob = [[bytecoder alloc] init];
	[cblob alloc_buffer:ntlmres.Coder.Length+8];
	[cblob push:type2.Type2->Challenge Length:8];
	[cblob push:(UInt8*)ntlmres.CC Length:ntlmres.Coder.Length-16];	//without NTProofStr area

	//calculate hash
	[NTLM NtLmV2Hash:type2.TargetName User:user Password:pass Result:self.ResponseKeyNT];
	//[NTLM NtLmV2Hash:domain User:user Password:pass Result:self.ResponseKeyNT];
	[NTLM HMACMD5:cblob.Data Length:cblob.Length Key:self.ResponseKeyNT KeyLength:16 Result:self.NTProofStr];
	memcpy(ntlmres.Res, self.NTProofStr, 16);
	[NTLM HMACMD5:self.NTProofStr Length:16 Key:self.ResponseKeyNT KeyLength:16 Result:self.SessionBaseKey];
	
	//create SessionKey
	if (type2.Type2->Flags & NTLMSSP_NEGOTIATE_KEY_EXCH) {
		[SMBUtils RandomArray:self.SessionKey Length:16];
		[RC4 digest:self.SessionKey :16 :self.SessionBaseKey :16 :self.EncrypedExportedSessionKey];
	} else {
		memcpy(self.SessionKey, self.SessionBaseKey, 16);
		memcpy(self.EncrypedExportedSessionKey, self.SessionBaseKey, 16);
	}
	//return [type3 serialize:domain Machine:domain User:user
	return [type3 serialize:type2.TargetName Machine:type2.TargetName User:user
			   NTLMResponse:ntlmres.Coder SessionKey:self.EncrypedExportedSessionKey];
}
/*----------------------------------------*
 *	NetShareEnumAll
 *----------------------------------------*/
-(BOOL)NetShareEnumALl {
	BOOL result = false;
	
	[self.Shares removeAllObjects];
	for(;;) {
		//Connect IPC$ Tree
		NSString *path = [NSString stringWithFormat:@"\\\\%@\\IPC$", self.Connection.IP];
		SMB2ReqTreeConnect *req = [[SMB2ReqTreeConnect alloc] initWith:path];
		[self MarkRequest:req];
		if (req == nil || ![self sendRequest:req])
			break;
		
		SMB2ResTreeConnect *res = [[SMB2ResTreeConnect alloc] init];
		if (res == nil || ![res recvResponse:self.Connection.Tcp])
			break;
		if (res.Header->Status == 0x103)
			if (![res recvResponse:self.Connection.Tcp])
				break;
		if (res.Header->Status != 0)
			break;
		
		//enum share by server
		result = [self NetShareEnumAllCore:res.Header->s.Sync.TreeId];
		
		//Disconnect Tree
		SMB2ReqTreeDisconnect *req2 = [[SMB2ReqTreeDisconnect alloc] init];
		if (req2 == nil) {
			result =false;
			break;
		}
		req2.Header->s.Sync.TreeId = res.Header->s.Sync.TreeId;
		[self MarkRequest:req2];
		if (![self sendRequest:req2]) {
			result = false;
			break;
		}
		SMB2ResTreeDisconnect *res2 = [[SMB2ResTreeDisconnect alloc] init];
		if (res2 == nil || ![res2 recvResponse:self.Connection.Tcp])
			result = false;
		if (res2.Header->Status == 0x103)
			if (![res2 recvResponse:self.Connection.Tcp]) {
				result =false;
				break;
			}
		if (res2.Header->Status != 0)
			result = false;
		break;
	}
	return result;
}

-(BOOL)NetShareEnumAllCore:(UInt32)TreeId {
	//Create
	SMB2ResCreate *res = nil;
	SMB2ReqCreate *req = [SMB2ReqCreate createPipe:TreeId PipeName:@"srvsvc"];
	[self MarkRequest:req];
	if (req != nil && [req serialize] && [self sendRequest:req]) {
		res = [[SMB2ResCreate alloc] init];
		if (res == nil || ![res recvResponse:self.Connection.Tcp] || res.Header->Status != 0)
			return false;
	}
	//NetShareEnumAll
	BOOL result = false;
	for (;;) {
		//IOCTL
		SMB2ReqIoctl *req3 = [[SMB2ReqIoctl alloc] initWith:TreeId FileId:res.CREATE->FileId];
		
		//RPC bind
		rpcConnBindHeader *rpc = [[rpcConnBindHeader alloc] init];
		rpcCtxElement *elm = [[rpcCtxElement alloc] init];
		[elm setAbstract:[rpcSyntaxId createSRVSVC]];
		[elm.Transfer addObject:[rpcSyntaxId createNDR]];
		[rpc.CtxList.Element addObject:elm];
		int size = [rpc serialize:req3.Coder];
		if (size < 0)
			break;
		
		// IOCTL request
		[self MarkRequest:req3];
		req3.IOCTL->InputCount = size;
		req3.IOCTL->Flags = 1;	//FSCTL
		req3.IOCTL->CtlCode = FSCTL_PIPE_TRANSCEIVE;
		if (![self sendRequest:req3])
			break;
		
		// IOCTL response
		SMB2ResIoctl *res3 = [[SMB2ResIoctl alloc] init];
		if (![res3 recvResponse:self.Connection.Tcp])
			break;
		if (res3.Header->Status == 0x103) {
			if (![res3 recvResponse:self.Connection.Tcp])
				break;
		}
		if (res3.Header->Status != 0)
			break;
		
		//deserialize bind ack
		
		//IOCTL
		SMB2ReqIoctl *req4 = [[SMB2ReqIoctl alloc] initWith:TreeId FileId:res.CREATE->FileId];
		
		//RPC netshareenumall
		NSString *servername = [NSString stringWithFormat:@"%@\0", self.Connection.IP];
		rpcReqNetShareEnummAll *net = [[rpcReqNetShareEnummAll alloc] initWith:servername];
		size = [net serialize:req4.Coder];
		if (size < 0)
			break;
		
		[self MarkRequest:req4];
		req4.IOCTL->InputCount = size;
		req4.IOCTL->Flags = 1;
		req4.IOCTL->CtlCode = FSCTL_PIPE_TRANSCEIVE;
		if (![self sendRequest:req4])
			break;
		
		// IOCTL response
		SMB2ResIoctl *res4 = [[SMB2ResIoctl alloc] init];
		if (![res4 recvResponse:self.Connection.Tcp])
			break;
		if (res4.Header->Status == 0x103) {
			if (![res4 recvResponse:self.Connection.Tcp])
				break;
		}
		if (res4.Header->Status != 0)
			break;
		[res4.Coder moveptrto:PACKETHEADERSIZE+IOCTLRESSIZE-1];
		
		rpcResNetShareEnummAll *net2 = [[rpcResNetShareEnummAll alloc] init];
		if (net2 == nil || ![net2 deserialize:res4.Coder])
			break;
		
		result = true;
		for (int i = 0; i < [net2.List count]; ++i) {
			SMBTreeConnect *tree = [[SMBTreeConnect alloc] initWith:self ShareInfo:[net2.List objectAtIndex:i]];
			if (tree == nil) {
				result = false;
				break;
			}
			[self.Shares addObject:tree];
		}
		break;
	}
	//Close
	SMB2ReqClose *req2 = [[SMB2ReqClose alloc] initWith:TreeId FileId:res.CREATE->FileId];
	[self MarkRequest:req2];
	if (req2 != nil && [self sendRequest:req2]) {
		SMB2ResClose *res2 = [[SMB2ResClose alloc] init];
		if (res2 == nil || ![res2 recvResponse:self.Connection.Tcp] || res2.Header->Status != 0)
			result = false;
	} else
		result = false;
	if (!result)
		[self.Shares removeAllObjects];
	return result;
}
/*----------------------------------------*
 *	Ping
 *----------------------------------------*/
-(BOOL)Ping {
	SMB2ReqEcho *req = [[SMB2ReqEcho alloc] init];
	[self MarkRequest:req];
	if (![req sendRequest:self.Connection.Tcp])
		return false;
	SMB2ResEcho *res = [[SMB2ResEcho alloc] init];
	if (![res recvResponse:self.Connection.Tcp] || res.Header->Status != 0)
		return false;
	return true;
}
/*----------------------------------------*
 *	Send Request with Sign
 *----------------------------------------*/
-(BOOL)sendRequest:(SMB2Header*)req mSec:(int)msec {
	if (self.Sign) {
		//clear sign field
		memset(req.getPacketHeader->Signature, 0, 16);
		//calc signature
		req.getPacketHeader->Flags |= SMB2_FLAGS_SIGNED;
		UInt8 hash[32];
		[NTLM HMACSHA256:req.getCoder.Data Length:req.getCoder.Length Key:self.SessionKey KeyLength:16 Result:hash];
		//set signature
		memcpy(req.getPacketHeader->Signature, hash, 16);
	}
	return [req sendRequest:self.Connection.Tcp mSec:msec];
}
-(BOOL)sendRequest:(SMB2Header*)req {
	if (self.Sign) {
		//clear sign field
		memset(req.getPacketHeader->Signature, 0, 16);
		//calc signature
		req.getPacketHeader->Flags |= SMB2_FLAGS_SIGNED;
		UInt8 hash[32];
		[NTLM HMACSHA256:req.getCoder.Data Length:req.getCoder.Length Key:self.SessionKey KeyLength:16 Result:hash];
		//set signature
		memcpy(req.getPacketHeader->Signature, hash, 16);
	}
	return [req sendRequest:self.Connection.Tcp];
}
@end

