//
//  SMBTreeConnect.m
//
//  Copyright (C) 2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2016/03/21	H.Kanemoto
//	Modified:	2017/05/24	H.Kanemoto  SMB Signature
//
#import <Foundation/Foundation.h>
#import "SMBTreeConnect.h"
#import "SMB2TreeConnect.h"
#import "SMB2Create.h"
#import "SMB2Close.h"
#import "SMB2QueryDirectory.h"
#import "SMBSession.h"
#import "SMBOpen.h"
/*----------------------------------------------------------------------*
 *	SMB SESSION
 *----------------------------------------------------------------------*/
@implementation SMBTreeConnect
/*----------------------------------------*
 *	Init/Dealloc
 *----------------------------------------*/
-(id)initWith:(SMBSession *)session ShareInfo:(rpcShareInfo1 *)info {
	if (self = [super init]) {
		self.Session = session;
		self.ShareInfo = info;
		self->treeid = 0;
		self->connected = false;
	}
	return self;
}

-(void)dealloc {
	DLog(@"");
	[self Disconnect];
}
/*----------------------------------------*
 *	Properties
 *----------------------------------------*/
-(NSString*)getShareName {
	return [[NSString alloc] initWithString:self.ShareInfo.FileName];
}
-(UInt32)getTreeId {
	return self->treeid;
}
-(BOOL)getConnected {
	return self->connected;
}
-(TcpClient*)getTcp {
	return self.Session.Connection.Tcp;
}
/*----------------------------------------*
 *	Tree Connect/Disconnect
 *----------------------------------------*/
-(BOOL)Connect {
	if (!self.Connected) {
		NSString *tn = [NSString stringWithFormat:@"\\\\%@\\%@", self.Session.Connection.IP, self.ShareName];
		SMB2ReqTreeConnect *req = [[SMB2ReqTreeConnect alloc] initWith:tn];
		[self.Session MarkRequest:req];
		if (![self.Session sendRequest:req])
			return false;
		
		SMB2ResTreeConnect *res = [[SMB2ResTreeConnect alloc] init];
		if (![res recvResponse:self.getTcp])
			return false;
		self->treeid = res.Header->s.Sync.TreeId;
		self->connected = true;
	}
	return true;
}
-(BOOL)Disconnect {
	if (self.Connected) {
		SMB2ReqTreeDisconnect *req = [[SMB2ReqTreeDisconnect alloc] init];
		if (req == nil)
			return false;
		req.Header->s.Sync.TreeId = self.TreeId;
		[self.Session MarkRequest:req];
		if (![self.Session sendRequest:req])
			return false;
			
		SMB2ResTreeDisconnect *res = [[SMB2ResTreeDisconnect alloc] init];
		if (![res recvResponse:self.getTcp] && res.Header->Status != 0)
			return false;
		
		self->connected = false;
	}
	return true;
}
/*----------------------------------------*
 *	EnumDir
 *----------------------------------------*/
-(NSMutableArray<FileIdBothDirInformation*>*)EnumDir:(NSString*)dirpath
											Callback:(EnumDirCallback)callback
{
	NSMutableArray<FileIdBothDirInformation*> *list = [[NSMutableArray<FileIdBothDirInformation*> alloc] init];
	BOOL result = false;
	if ([self Connect]) {
		for (;;) {
			//Create(Open) Dir
			SMB2ReqCreate *req1 = [SMB2ReqCreate createDir:self.TreeId DirName:dirpath];
			if (req1 == nil || ![req1 serialize])
				break;
			[self.Session MarkRequest:req1];
			if (![self.Session sendRequest:req1])
				break;
			
			SMB2ResCreate *res1 = [[SMB2ResCreate alloc] init];
			if (res1 == nil || ![res1 recvResponse:self.getTcp])
				break;
			if (res1.Header->Status == 0x103) {	//pending
				if (![res1 recvResponse:self.getTcp])
					break;
			}
			if (res1.Header->Status != 0)
				break;
			
			result = [self EnumDirCore:res1.Header->s.Sync.TreeId
								FileId:res1.CREATE->FileId FileList:list Callback:callback];
			
			//Close Dir
			SMB2ReqClose *req2 = [[SMB2ReqClose alloc] initWith:self.TreeId FileId:res1.CREATE->FileId];
			[self.Session MarkRequest:req2];
			if (req2 == nil || ![self.Session sendRequest:req2]) {
				result = false;
				break;
			}
			SMB2ResClose *res2 = [[SMB2ResClose alloc] init];
			if (res2 == nil || ![res2 recvResponse:self.getTcp])
				result = false;
			if (res2.Header->Status == 0x103) {
				if (![res2 recvResponse:self.getTcp])
					break;
			}
			if (res2.Header->Status != 0)
				result = false;
			break;
		}
		result = ![self Disconnect] ? false : result;
	}
	if (!result) {
		[list removeAllObjects];
		return nil;
	} else
		return list;
}

-(BOOL)EnumDirCore:(UInt32)TreeId
			FileId:(UInt8*)FileId
		  FileList:(NSMutableArray<FileIdBothDirInformation*>*)list
		  Callback:(EnumDirCallback)callback
{
	SMB2ResQueryDirectory *res = [[SMB2ResQueryDirectory alloc] init];
	SMB2ReqQueryDirectory *req = [SMB2ReqQueryDirectory create:TreeId FileId:FileId FileName:@"*"];
	if (res == nil || req == nil)
		return false;
	for (int i = 0;; ++i) {
		req.QUERYDIR->Flags = i == 0 ? 1 : 0;	//RESET_SCAN
		[self.Session MarkRequest:req];
		if (![self.Session sendRequest:req])
			break;
		
		if (![res recvResponse:self.getTcp])
			break;
		if (res.Header->Status == 0x80000006)
			return true;
		if (res.Header->Status == 0x103)
			if (![res recvResponse:self.getTcp])
				break;
		if (res.Header->Status != 0)
			break;
		
		[res.Coder moveptrto:res.QUERYDIR->OutputBufferOffset];
		if (![self decodeFileInfo:res.Coder FileList:list Callback:callback])
			break;
	}
	return false;
}

-(BOOL)decodeFileInfo:(bytecoder*)coder
			 FileList:(NSMutableArray<FileIdBothDirInformation*>*)list
			 Callback:(EnumDirCallback)callback
{
	BOOL result = false;
	coder.TopOffset = -1*coder.Offset;	//global offset = 0
	int global_offset = coder.GlobalOffset;
	for (;;) {
		FileIdBothDirInformation *info = [[FileIdBothDirInformation alloc] init];
		if (info == nil || ![info deserialize:coder])
			break;
		//callback to ask if add this info to array
		if (callback != nil && !callback(info))
			;
		else
			[list addObject:info];
		if (info.Info->NextEntryOffset == 0) {
			result = true;
			break;
		}
		global_offset += info.Info->NextEntryOffset;
		if (![coder moveglobalptrto:global_offset])
			break;
	}
	return result;
}
/*----------------------------------------*
 *	OpenFile
 *----------------------------------------*/
-(SMBOpen*)OpenFile:(NSString*)filename
	 AutoDisconnect:(BOOL)autodisconnect
		   Callback:(OpenFileCallback)callback
{
	for(;;) {
		if (![self Connect])
			break;
		//CREATE(Open) request
		SMB2ReqCreate *req = [SMB2ReqCreate createFile:self.TreeId FileName:filename];
		if (req == nil)
			break;
		if (callback != nil)	//callback if set
			callback(req);		//configure create params by caller
		[self.Session MarkRequest:req];
		if (![req serialize])
			break;
		if (![self.Session sendRequest:req])
			break;
		//CREATE response
		SMB2ResCreate *res = [[SMB2ResCreate alloc] init];
		if (res == nil || ![res recvResponse:self.getTcp])
			break;
		if (res.Header->Status == 0x103)
			if (![res recvResponse:self.getTcp])
				break;
		if (res.Header->Status != 0)
			break;
		SMBOpen *o = [[SMBOpen alloc] initWith:self FileId:res.CREATE->FileId AutoDisconnect:autodisconnect];
		if (o == nil)
			break;
		return o;
	}
	[self Disconnect];
	return nil;
}
@end
