//
//  SMBUtils.m
//
//  Copyright (C) 2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2016/03/12	H.Kanemoto
//	Modified:
//
#import <Foundation/Foundation.h>
#import "SMBUtils.h"

@implementation SMBUtils
/*----------------------------------------*
 *	convert int value from byte order
 *----------------------------------------*/
+(UInt32)Byte2IntBE:(UInt8 *)buffer {
	UInt32 result = 0;
	for (int i = 0; i < 4; ++i) {
		result <<= 8;
		result += buffer[i];
	}
	return result;
}

+(UInt32)Byte2IntLE:(UInt8 *)buffer {
	UInt32 result = 0;
	for (int i = 3; i >= 0; --i) {
		result <<= 8;
		result += buffer[i];
	}
	return result;
}

+(UInt16)Byte2ShortLE:(UInt8 *)buffer {
	UInt16 result = 0;
	result += buffer[0];
	result += ((UInt16)buffer[1] * 256);
	return result;
}

+(UInt64)Byte2LongLE:(UInt8 *)buffer {
	UInt64 result = 0;
	for (int i = 7; i >= 0; --i) {
		result <<= 8;
		result += buffer[i];
	}
	return result;
}
/*----------------------------------------*
 *	convert int value to byte order
 *----------------------------------------*/
+(void)Int2ByteBE:(UInt32)val :(UInt8 *)buffer {
	buffer[3] = (UInt8)(val & 0xff);
	buffer[2] = (UInt8)((val >> 8) & 0xff);
	buffer[1] = (UInt8)((val >> 16) & 0xff);
	buffer[0] = (UInt8)((val >> 24) & 0xff);
}

+(void)Short2ByteLE:(UInt16)val : (UInt8*)buffer {
	buffer[0] = (UInt8)(val & 0xff);
	buffer[1] = (UInt8)((val >> 8) & 0xff);
}

+(void)Int2ByteLE:(UInt32)val : (UInt8*)buffer {
	buffer[0] = (UInt8)(val & 0xff);
	buffer[1] = (UInt8)((val >> 8) & 0xff);
	buffer[2] = (UInt8)((val >> 16) & 0xff);
	buffer[3] = (UInt8)((val >> 24) & 0xff);
}

+(void)Long2ByteLE:(UInt64)val : (UInt8*)buffer {
	buffer[0] = (UInt8)(val & 0xff);
	buffer[1] = (UInt8)((val >> 8) & 0xff);
	buffer[2] = (UInt8)((val >> 16) & 0xff);
	buffer[3] = (UInt8)((val >> 24) & 0xff);
	buffer[4] = (UInt8)((val >> 32) & 0xff);
	buffer[5] = (UInt8)((val >> 40) & 0xff);
	buffer[6] = (UInt8)((val >> 48) & 0xff);
	buffer[7] = (UInt8)((val >> 56) & 0xff);
}
/*----------------------------------------*
 *	byte alignment
 *----------------------------------------*/
+(int32_t)Align4bytes:(int32_t)offset {
	int32_t align = (offset + 3) & 0xfffffffc;
	return align;
}

+(int32_t)Align8bytes:(int32_t)offset {
	int32_t align = (offset + 7) & 0xfffffff8;
	return align;
}
/*----------------------------------------*
 *	Misc
 *----------------------------------------*/
+(void)GenUUID:(UInt8[16]) buffer {
	NSUUID *uuid = [NSUUID UUID]; //[[NSUUID alloc] init];
	[uuid getUUIDBytes:buffer];
}

+(void)SMBTimeStamp:(UInt8[8])stamp {
	time_t sec;
	time(&sec);
	UInt64 epoc = (UInt64)sec * 1000;
	epoc += 11644473600000L;
	epoc *= 10000;
	for (int i = 0; i < 8; ++i) {
		stamp[i] = (UInt8)(epoc & 0xff);
		epoc >>= 8;
	}
}

+(void)RandomArray:(UInt8 *)array Length:(int)len {
	for (int i = 0; i < len; ++i) {
		int r = arc4random() % 256;
		array[i] = (UInt8)(r & 0xff);
	}
}
/*----------------------------------------*
 *	Factory
 *----------------------------------------*/
+(NSError*)buildErr:(TBErrors)code {
	NSDictionary *errorDic = nil;
	switch (code) {
		case SMB2_CONNECTION_ERR:
			errorDic = @{
						 NSLocalizedDescriptionKey:@"Connection failed",
						 NSLocalizedRecoverySuggestionErrorKey:@"Hasta la vista, baby."
						 };
			break;
		case SMB2_NEGOTIATE_REQ_ERR:
			errorDic = @{
						 NSLocalizedDescriptionKey:@"Negotiation failed",
						 NSLocalizedRecoverySuggestionErrorKey:@"Go, ahead. Make my day."
						 };
			break;
		case SMB2_AUTHENTICATE_ERR:
			errorDic = @{
						 NSLocalizedDescriptionKey:@"NTLM Authentication failed",
						 NSLocalizedRecoverySuggestionErrorKey:@"My name is Bond, James Bond."
						 };
			break;
		case SMB2_QUERY_INFO_REQ_ERR:
			errorDic = @{
						 NSLocalizedDescriptionKey:@"Query Information failed",
						 NSLocalizedRecoverySuggestionErrorKey:@"Ask siri on your iPhone/iPad"
						 };
			break;
		case SMB2_READ_ERR:
			errorDic = @{
						 NSLocalizedDescriptionKey:@"Read failed",
						 NSLocalizedRecoverySuggestionErrorKey:@"Elementary, my dear Watson."
						 };
			break;
		default:
			errorDic = @{
						 NSLocalizedDescriptionKey:@"unknown error",
						 NSLocalizedRecoverySuggestionErrorKey:@"May the force be with you"
						 };
			break;
	}
	return [[NSError alloc] initWithDomain:@"Neverland" code:code userInfo:errorDic];
}

@end
