//
//  TcpClient.m
//
//  Copyright (C) 2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2016/03/11	H.Kanemoto
//	Modified:
//
#import <Foundation/Foundation.h>
#import "TcpClient.h"

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <dirent.h>
#include <netdb.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/select.h>
#include <netinet/in.h>
#include <sys/time.h>
/*----------------------------------------------------------------------*
 *	TCP CLIENT
 *----------------------------------------------------------------------*/
@implementation TcpClient

-(id)init {
	if (self = [super init]) {
		self->fd = -1;
	}
	return self;
}

-(void)dealloc {
	DLog(@"");
	[self close];
}

-(BOOL)connect:(NSString *)addr Port:(int)port {
	const char *ip = [addr cStringUsingEncoding:NSASCIIStringEncoding];
	return [self connect:ip :port];
}
-(BOOL)connect:(const char *)addr :(int)port {
	struct sockaddr_in sa;
	sa.sin_family = AF_INET;
	sa.sin_port = htons(port);
	sa.sin_addr.s_addr = inet_addr(addr);
	if (sa.sin_addr.s_addr == 0xffffffff) {
		struct hostent *host;
		host = gethostbyname(addr);
		if (host == NULL)
			return false;
		sa.sin_addr.s_addr = *(unsigned int *)host->h_addr_list[0];
	}
	self->fd = socket(AF_INET, SOCK_STREAM, 0);
	[self set_nonblock_mode:true];
	connect(self->fd, (struct sockaddr *)&sa, sizeof(sa));
	fd_set			fdw;
	struct timeval	tv;
	FD_ZERO(&fdw);
	FD_SET(self->fd, &fdw);
	tv.tv_sec = 3;
	tv.tv_usec = 0;
	int retval = select(self->fd + 1, NULL, &fdw, NULL, &tv);
	if (retval <= 0) {
		[self close];
		return false;
	} else {
		int error = 0;
		int errlen = sizeof(error);
		getsockopt(self->fd, SOL_SOCKET, SO_ERROR, &error, (socklen_t *)&errlen);
		if (error != 0) {
			[self close];
			return false;//connect fail
		}
		[self set_nonblock_mode:false];
		int set = 1;
		setsockopt(self->fd, SOL_SOCKET, SO_NOSIGPIPE, (void *)&set, sizeof(int));
		return true;
	}
}

-(void)close {
	if (self->fd != -1) {
		close(self->fd);
		self->fd = -1;
	}
	
}

-(void)set_nonblock_mode:(bool) flag {
	int flags = fcntl(self->fd, F_GETFL, 0);
	if (flag) {
		fcntl(self->fd, F_SETFL, flags | O_NONBLOCK);
	} else {
		flags &= ~ O_NONBLOCK;
		fcntl(self->fd, F_SETFL, flags);
	}
}

-(void)set_keepalive {
	int option = 1;
	setsockopt(self->fd, SOL_SOCKET, SO_KEEPALIVE, (void*)&option, sizeof(option));
	/*
	option = 60;
	setsockopt(self->fd, IPPROTO_TCP, TCP_KEEPIDLE, (void*)&option, sizeof(option));
	option = 30;
	setsockopt(self->fd, IPPROTO_TCP, TCP_KEEPINTVL, (void*)&option, sizeof(option));
	option =  4;
	setsockopt(self->fd, IPPROTO_TCP, TCP_KEEPCNT, (void*)&option, sizeof(option));*/
}

-(long)millisec {
	struct timeval time;
	gettimeofday(&time, NULL);
	long msec = (time.tv_sec * 1000) + (time.tv_usec / 1000);
	return msec;
}

-(BOOL)recv_available:(int)sec uSec:(int)usec {
	if (self->fd == -1)
		return false;
	fd_set	fdset;
	struct timeval tv;
	tv.tv_usec = usec;
	tv.tv_sec = sec;
	FD_ZERO(&fdset);
	FD_SET(self->fd, &fdset);
	int ret = select(self->fd+1, &fdset, NULL, NULL, &tv);
	return ret == 1;
}

-(BOOL)send_available:(int)sec uSec:(int)usec {
	if (self->fd == -1)
		return false;
	fd_set	fdset;
	struct timeval tv;
	tv.tv_usec = usec;
	tv.tv_sec = sec;
	FD_ZERO(&fdset);
	FD_SET(self->fd, &fdset);
	int ret = select(self->fd+1, NULL, &fdset, NULL, &tv);
	return ret == 1;
}

-(int)recv:(char *)buffer Lenght:(int)len {
	if (self->fd == -1)
		return false;
	return (int)read(self->fd, buffer, len);
}

-(int)recv:(char *)buffer Lenght:(int)len mSec:(int)msec {
	if (self->fd == -1)
		return -1;
	long count = msec < 8 ? 1 : msec;
	int remain = len, read_len = 0;
	for (long s = [self millisec], lap = 0; lap < count; lap = [self millisec] - s) {
		if ([self recv_available:0 uSec:10000]) {
			int cnt = [self recv:&buffer[read_len] Lenght:remain];
			if (cnt < 0)
				return cnt;
			remain -= cnt;
			read_len += cnt;
			if (remain > 0)
				;
			else
				break;
		}
	}
	return read_len;
}

-(int)send:(char *)buffer Lenght:(int)len {
	if (self->fd == -1)
		return false;
	return (int)send(self->fd, buffer, len, 0);
}

-(int)send:(char *)buffer Lenght:(int)len mSec:(int)msec {
	if (self->fd == -1)
		return -1;
	long count = msec < 8 ? 1 : msec;
	int remain = len, write_len = 0;
	for (long s = [self millisec], lap = 0; lap < count; lap = [self millisec] - s) {
		if ([self send_available:0 uSec:10000]) {
			int cnt = [self send:&buffer[write_len] Lenght:remain];
			if (cnt < 0)
				return cnt;
			remain -= cnt;
			write_len += cnt;
			if (remain > 0)
				;
			else
				break;
		}
	}
	return write_len;
}

@end
