//
//  UCS2.m
//
//  Copyright (C) 2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2016/03/17	H.Kanemoto
//	Modified:
//
#import <Foundation/Foundation.h>
#import "UCS2.h"
#include <malloc/malloc.h>
/*----------------------------------------------------------------------*
 *	UCS2(UTF16LE)
 *----------------------------------------------------------------------*/
@implementation UCS2
/*----------------------------------------*
 *	Init/Dealloc
 *----------------------------------------*/
-(id)init {
	if (self = [super init]) {
		self->byte_len = 0;
		self->ptr = nil;
	}
	return self;
}

-(void)dealloc {
	DLog(@"");
	[self free];
}
/*----------------------------------------*
 *	Properties
 *----------------------------------------*/
-(int)getByteLen {
	return self->byte_len;
}
-(UInt8*)getPtr {
	return self->ptr;
}
/*----------------------------------------*
 *	Entry
 *----------------------------------------*/
-(BOOL)entry:(NSString*)target {
	if (target != nil) {
		int len = (int)[target lengthOfBytesUsingEncoding:NSUTF16LittleEndianStringEncoding];
		const char *ucs2 = [target cStringUsingEncoding:NSUTF16LittleEndianStringEncoding];
		return [self entry:(UInt8*)ucs2 length:len];
	}
	return false;
}

-(BOOL)entry:(UInt8*)ucs2 length:(int)bytelen {
	[self free];
	self->ptr = calloc(bytelen, 1);
	if (self->ptr != nil) {
		self->byte_len = bytelen;
		memcpy(self->ptr, ucs2, self->byte_len);
		return true;
	}
	return false;
}

-(void)free {
	if (self->ptr != nil) {
		free(self->ptr);
		assert(malloc_zone_check(NULL));
		self->ptr = nil;
	}
}
/*----------------------------------------*
 *	Text (convert to NSString)
 *----------------------------------------*/
-(NSString*)text {
	NSString *str = [[NSString alloc]
			  initWithBytes:(const char *)self->ptr
					 length:self->byte_len
				   encoding:NSUTF16LittleEndianStringEncoding];
	return str;
}
/*----------------------------------------*
 *	Factory
 *----------------------------------------*/
+(UCS2*)create:(NSString*)target {
	UCS2 *u = [[UCS2 alloc] init];
	if ([u entry:target])
		return u;
	else
		return nil;
}

+(UCS2*)create:(UInt8*)buffer length:(int)len {
	UCS2 *u = [[UCS2 alloc] init];
	if ([u entry:buffer length:len])
		return u;
	else
		return nil;
}

@end