//
//  bytecoder.h
//
//  Copyright (C) 2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2016/03/13	H.Kanemoto
//	Modified:
//
#ifndef bytecoder_h
#define bytecoder_h
#import "TBObject.h"
/*----------------------------------------------------------------------*
 *	Data buffer decoder/encoder
 *----------------------------------------------------------------------*/
@interface bytecoder : TBObject {
@protected
	void*		ptr;			//pointer to allocated memory
	int32_t		siz_alloc;		//size of allocated memory
	int32_t		siz_used;		//size of used memory
	int32_t		siz_remain;		//size of remain memory
	int32_t		local_offset;	//offset of pointer to memory
	int32_t		top_offset;		//offset of top of allocated memory
}
@property (nonatomic, readonly, assign, getter=getData) UInt8 *Data;
@property (nonatomic, readonly, assign, getter=getLenght) int Length;
@property (nonatomic, assign, getter=getTopOffset, setter=setTopOffset:) int TopOffset;
@property (nonatomic, readonly, assign, getter=getGlobalOffset) int GlobalOffset;
@property (nonatomic, readonly, assign, getter=getLocalOffset) int LocalOffset;
@property (nonatomic, assign, getter=getOffset, setter=setOffset:) int Offset;
@property (nonatomic, readonly, assign, getter=getPointer) void *Pointer;
@property (nonatomic, readonly, assign, getter=getUnreadCnt) int UnreadCnt;
@property (nonatomic, readonly, assign, getter=getRemain) int Remain;

-(BOOL)alloc_buffer:(int)initial_size;
-(BOOL)expand_buffer:(int)expand_size;
-(BOOL)recycle_buffer:(int)recycle_size;
-(void)freeptr;

-(bytecoder *)cutout:(uint32_t)offset Length:(uint32_t)len;

-(BOOL)push:(const UInt8 *)buffer Length:(int)len;
-(BOOL)push_byte:(UInt8)data;
-(BOOL)push_le_short:(UInt16)value;
-(BOOL)push_le_int:(UInt32)value;
-(BOOL)push_le_long:(UInt64)value;
-(BOOL)pad8bytes_align:(BOOL)global;
-(BOOL)pad4bytes_align:(BOOL)global;
-(void)consume:(int)siz;
-(void)rewind;

-(BOOL)pull:(UInt8 *)buffer Length:(int)len;
-(BOOL)pull_byte:(UInt8*)buffer;
-(BOOL)pull_le_short:(UInt16*)value;
-(BOOL)pull_le_int:(UInt32*)value;
-(BOOL)pull_le_long:(UInt64*)value;
-(int)skip8bytes_align:(BOOL)global;
-(int)skip4bytes_align:(BOOL)global;
-(BOOL)moveptr:(int)siz;
-(BOOL)moveptrto:(int)abs;
-(BOOL)moveglobalptrto:(int)globalabs;

@end
/*----------------------------------------------------------------------*
 *	Packet base
 *----------------------------------------------------------------------*/
@interface packetbase : TBObject {
	@protected
	bytecoder*		coder;
}
@property (nonatomic, readonly, getter=getCoder) bytecoder *Coder;

-(id)initWith:(bytecoder *)coder;
-(bytecoder *)detach;

@end
	
#endif /* bytecoder_h */
