//
//  ViewController.swift
//
//  Copyright (C) 2015-2016 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//	 Created:	2016/04/04	H.Kanemoto
//	Modified:
//
import UIKit
import CoreLocation
import GameController
import Metal
import MetalKit
import QuartzCore

struct Vertex {
	var position : vector_float2
	var pixel : vector_float2
}

open class ViewController: UIViewController {
	private let device = MTLCreateSystemDefaultDevice()!
	private var mtkView : MTKView! = nil
	private var commandQueue: MTLCommandQueue!
	private var renderPassDescriptor: MTLRenderPassDescriptor!
	private var bufferPosition: MTLBuffer!
	private var bufferColor: MTLBuffer!
	private var renderPipelineState: MTLRenderPipelineState!
	private var textureDescriptor : MTLTextureDescriptor!
	private let vertexData = [
		Vertex(position: [-1, -1], pixel: [0, 0]),
		Vertex(position: [1, -1], pixel: [1, 0]),
		Vertex(position: [-1, 1], pixel: [0, 1]),
		Vertex(position: [1, 1], pixel: [1, 1])
	]
	fileprivate var n80em:n80em!
	fileprivate var n80view:N80View!
	open var icadeview:iCadeReaderView!
	
	override open func loadView() {
		super.loadView()
		self.n80view = self.view as? N80View
		self.n80view.isUserInteractionEnabled = true
		self.n80view.isMultipleTouchEnabled = true
		self.n80view.backgroundColor = UIColor.black
	}
	
	override open func viewDidLoad() {
		super.viewDidLoad()
		self.n80em = (UIApplication.shared.delegate as! AppDelegate).N80EM
		self.prepareMetal()	// prepare metal objects
		
		//MFi Bluetooth detect
		GCController.startWirelessControllerDiscovery(completionHandler: {() -> Void in
			let appDelegate:AppDelegate = UIApplication.shared.delegate as! AppDelegate
			appDelegate.setupControllersCore()
		})
		
		//iCade Game Controller View登録
		/*self.icadeview = iCadeReaderView(frame: CGRect.zero)
		self.view.addSubview(self.icadeview)
		self.icadeview.active = true
		self.icadeview.delegate = self.n80view.VRPad*/
		
		//MFI Game Controllerハンドラ登録
		let appDelegate:AppDelegate = UIApplication.shared.delegate as! AppDelegate
		self.n80em.mficallback = {() -> Void in
			DispatchQueue.main.async(execute: {() -> Void in
				if appDelegate.useMFI {
					let ctr = appDelegate.mfictr[0]
                    if ctr.extendedGamepad != nil {
						self.n80view.VRPad.hitMFIButton(ctr)
						self.n80view.VRPad.hitMFITest(ctr)
					}
				}
			})
		}
		//仮想キーボードOnハンドラ登録
		self.n80view.VRPad.callbackButtonOn = {(Code:UInt8) -> Void in
            self.n80em.n80kb.onKeyDown(Code)
		}
		//仮想キーボードOffハンドラ登録
		self.n80view.VRPad.callbackButtonOff = {(Code:UInt8) -> Void in
			if Code == 0x81 {			//TAPE
				self.icadeview?.active = false
				self.performSegue(withIdentifier: N80Key.Shift ? "N80File" : "N80LocalFile", sender: nil)
            } else if Code == 0x83 {    //SAMBA
                self.icadeview?.active = false
                self.performSegue(withIdentifier: "N80File", sender: nil)
			} else if Code == 0x90 {	//Config
				self.icadeview?.active = false
				self.performSegue(withIdentifier: "N80Config", sender: nil)
			} else if Code == 0x82 {	//PCG
				self.n80em.reqPCGToggle()
			} else {					//Keys
                self.n80em.n80kb.onKeyUp(Code)
			}
		}
        //外部Keyboard接続イベントハンドラ登録
        if #available(iOS 14.0, *) {
            NotificationCenter.default.addObserver(forName: NSNotification.Name.GCKeyboardDidConnect, object: nil, queue: OperationQueue.main) {_ in
              if let keyboard = GCKeyboard.coalesced?.keyboardInput {
                keyboard.keyChangedHandler = {
                  (keyboard, key, keyCode, pressed) in
                  if pressed {
                      self.n80em.n80kb.kb_keydown(keyCode)
                  } else {
                      self.n80em.n80kb.kb_keyup(keyCode)
                    if keyCode == GCKeyCode.F9 {
                        self.icadeview?.active = false
                      self.performSegue(withIdentifier: "N80File", sender: nil)
                    } else if keyCode == GCKeyCode.F7 {
                        self.icadeview?.active = false
                        self.performSegue(withIdentifier: "N80Config", sender: nil)
                    } else if keyCode == GCKeyCode.F6 {
                        _ = self.n80view.VRPad.keyview.toggleShow()
                    }
                  }
                }
              }
            }
        } else {
          // Fallback on earlier versions
        }
		//不正命令実行時ハンドラ登録
		appDelegate.N80EM.haltcallback = {(halt:Int32) -> Void in
			var msg:String!
			switch halt {
			case 1:
				msg = "detect HALT instruction"
			case 2:
				msg = "detect illegal instruction"
			default:
				msg = "detect Unimplemented instruction"
			}
			let alert = UIAlertController(title: "N80", message: msg!, preferredStyle: UIAlertController.Style.alert)
			let action = UIAlertAction(title: "OK", style: UIAlertAction.Style.default, handler: nil)
			alert.addAction(action)
			self.present(alert, animated: true, completion: {() -> Void in
				self.n80em.reqReset()
			})
		}
        // create dummy file "Files App"
        let fm = FileManager.default
        let documentsPath = NSSearchPathForDirectoriesInDomains(.documentDirectory, .userDomainMask, true).first!
        let filePath = documentsPath + "/n80file.txt"
        if !fm.fileExists(atPath: filePath) {
            fm.createFile(atPath: filePath, contents: nil, attributes: [:])
        }
        //エミュレーター開始
		self.n80em.start()
		//Z80 thread 開始
		var _ = Thread.detachNewThreadSelector(#selector(ViewController.tick), toTarget: self, with: nil)
	}
	@objc open func tick() {
		self.n80em.z80Thread()
	}
	//-------------------------------------------------------------------------
	//	画面遷移前処理
	//-------------------------------------------------------------------------
	override open func prepare(for segue: UIStoryboardSegue, sender: Any?) {
		let appDelegate:AppDelegate = UIApplication.shared.delegate as! AppDelegate
		appDelegate.clearKey()
		
		if segue.identifier == "N80File" {
			// SMB File Selector移動前に、このViewControllerを引数として渡す
			let vc = segue.destination as! N80FileViewController
			vc.delegate = self
		} else if segue.identifier == "N80Config" {
			// SMB File Selector移動前に、このViewControllerを引数として渡す
			let vc = segue.destination as! N80ConfigViewController
			vc.delegate = self
		} else if segue.identifier == "N80LocalFile" {
			// Local File Selector移動前に、このViewControllerを引数として渡す
			let vc = segue.destination as! N80LocalFileViewController
			vc.delegate = self
		}
	}
	//-------------------------------------------------------------------------
	//	File Read完了処理
	//-------------------------------------------------------------------------
	open func recvCMTData(_ data:bytecoder) {
		// SMB File Selectorより読み込んだファイルデータ(CMT)を受け取る
		self.n80em.reqOpenFile(data, isN80: false)
		data.freeptr()
	}
	
	open func recvN80Data(_ data:bytecoder) {
		// SMB File Selectorより読み込んだファイルデータ(N80)を受け取る
		self.n80em.reqOpenFile(data, isN80: true)
		data.freeptr()
	}
	
	override open func didReceiveMemoryWarning() {
		super.didReceiveMemoryWarning()
		// Dispose of any resources that can be recreated.
	}
	//-------------------------------------------------------------------------
	//	METAL処理
	//-------------------------------------------------------------------------
	func prepareMetal() {
		let (w,h) = (view.frame.width,view.frame.height)
		let mtkViewFrame = CGRect(x: w/2 - 2*h/3, y: 0,width: 4*h/3, height: h)
		self.mtkView = MTKView(frame: mtkViewFrame, device: device)
		self.mtkView.device = device
		self.mtkView.framebufferOnly = true
		self.mtkView.preferredFramesPerSecond = 60
		self.mtkView.colorPixelFormat = .bgra8Unorm
		self.mtkView.clearColor = MTLClearColor(red: 0, green: 0, blue: 0, alpha: 1)
		self.mtkView.delegate = self
		view.insertSubview(mtkView, at: 0)

		// init MTLCommandQueue
		commandQueue = device.makeCommandQueue()
		
		// create Render Pass Descriptor
		// set behavor of this Render Pass when it will be executed
		// set background color to black
		renderPassDescriptor = MTLRenderPassDescriptor()
		renderPassDescriptor.colorAttachments[0].loadAction = MTLLoadAction.clear
		renderPassDescriptor.colorAttachments[0].storeAction = MTLStoreAction.store
		renderPassDescriptor.colorAttachments[0].clearColor = MTLClearColorMake(0.0, 0.0, 0.0, 1.0)
		// create vertex buffer
		let size = vertexData.count * MemoryLayout<Vertex>.size
		bufferPosition = device.makeBuffer(bytes: vertexData, length: size)
		// regist shader
		guard let library = device.makeDefaultLibrary() else {fatalError()}
		let descriptor = MTLRenderPipelineDescriptor()
		descriptor.vertexFunction = library.makeFunction(name: "myVertexShader")
		descriptor.fragmentFunction = library.makeFunction(name: "video_fragment")
		descriptor.colorAttachments[0].pixelFormat = .bgra8Unorm
		// create Render pipe-line state
		renderPipelineState = try! device.makeRenderPipelineState(descriptor: descriptor)
		// create texture descriptor (RGB[bitmap] -> BRG[metal])
		let appDelegate:AppDelegate = UIApplication.shared.delegate as! AppDelegate
		let bmpimg = appDelegate.BmpImg
		self.textureDescriptor = MTLTextureDescriptor.texture2DDescriptor(pixelFormat: MTLPixelFormat.rgba8Unorm, width: Int(bmpimg.width), height: Int(bmpimg.height), mipmapped: false)
	}
	func makeTexture() -> MTLTexture {
		let appDelegate:AppDelegate = UIApplication.shared.delegate as! AppDelegate
		let bmpimg = appDelegate.BmpImg
		let tex = device.makeTexture(descriptor: self.textureDescriptor)
		let rgn = MTLRegionMake2D(0, 0, Int(bmpimg.width), Int(bmpimg.height))
		// set off-screen bitmap to texture
		tex?.replace(region: rgn, mipmapLevel: 0, slice: 0, withBytes: bmpimg.bitmap, bytesPerRow: Int(bmpimg.bytesPerRow), bytesPerImage: Int(bmpimg.bytesPerRow*bmpimg.height))
		return tex!
	}
}
extension ViewController : MTKViewDelegate {
	
	public func mtkView(_ view: MTKView, drawableSizeWillChange size: CGSize) {
	}
	
	public func draw(in view: MTKView) {
		// crate texture
		let tex = makeTexture()
		// purchase drawable
		guard let drawable = view.currentDrawable else {fatalError()}
		renderPassDescriptor.colorAttachments[0].texture = drawable.texture
		// create Command Buffer
		guard let cmdBuffer = commandQueue.makeCommandBuffer() else {fatalError()}
		// create Encoder
		let encoder = cmdBuffer.makeRenderCommandEncoder(descriptor: renderPassDescriptor)!
		encoder.setRenderPipelineState(renderPipelineState)
		// send vertex points array to encoder
		encoder.setVertexBuffer(bufferPosition, offset: 0, index: 0)
		encoder.setFragmentTexture(tex, index: 0)
		// draw rectangle
		encoder.drawPrimitives(type: MTLPrimitiveType.triangleStrip, vertexStart: 0, vertexCount: 4)
		// end encoding
		encoder.endEncoding()
		// entry drawable
		cmdBuffer.present(drawable)
		// commit command buffer
		cmdBuffer.commit()
		//cmdBuffer.waitUntilCompleted()
		
		// request next off-screen bitmap painting to emulator
		//self.n80em.refleshScreen()
	}
}

